:
# SCCSID(@(#)mrgssv.sh	1.5) Modified 12:43:01 1/3/00
# This is a script that all Merge sh scripts can dot in and then
# call the function Set_System_Variables() to get the settings
# STATICMERGE and VARMERGE settings from /etc/default/merge.
# These specify the two main directories where all the Merge files
# are installed.
# The function also gets the SYTEMTYPE and MERGENAME settings from
# the file VARMERGE/Version.
# The DEFAULTMERGE variable is also set to "/etc/default/merge" so
# that if the calling script wants to use it, it can use the same one.
# (During installation some of these things might be temporarily
# located elsewhere, so it is preferable not to hardwire it's location
# in all the scripts.  Which is also part of the reason for using
# STATICMERGE and VARMERGE instead of hardwiring those diretories too.)
#
# Normally if it cannot get the settings, it prints error messages
# and exits thus aborting the script.  But if the function is called
# with parameter "noexit", it tries to guess at what the settings should
# be and does not exit.
# This can be used in scripts that are used when removing the package
# so that it is more likely that partial or damaged installations can
# be removed.  In this case, the variable FAIL_SSV can be tested
# to see if it recovered by guessing or if all was ok.  If it is set
# to "allok" then it is all ok.  Otherwise the caller must not
# make too many assumptions about the correctness of the settings.
# The caller may want to do it's own recovery or exiting.
#
# Summary:
# Input variables:
#	SSV_OPTION
# Variables set by function Set_System_Variables:
#	DEFAULTMERGE
#	STATICMERGE
#	VARMERGE
#	STATICMBIN
#	DOSWINBINDIR
#	SYSTEMTYPE
#	MERGENAME
#	VERFILE
# It also uses these temporary variables:
#	FAIL_SSV
# Also the function Set_System_Variables() can only be called once
# because it unsets itself as well as the support function Err_Msg_SSV.
# Note:
# Also the DM variable might be used:  If an error message is printed
# the function dots in setnls.sh (if it can) and uses DM.  If setnls.sh
# was dotted in, then when the function returns DM will be unset.
# Note that it only returns when error messages are printed when the
# "noexit" option is used, so normally there is no need to deal with
# the situation where the function dots in set_nls.sh.  But if "noexit"
# is used, the caller can detect the situation by testing for the
# variable MFILE which is set by setnls.sh and not dot in set_nls.sh again.
#

# Get the STATICMERGE, VARMERGE and SYSTEMTYPE settings.
Set_System_Variables()
{
	DEFAULTMERGE=/etc/default/merge
	export DEFAULTMERGE
	if [ "x$1" = "xnoexit" ]
	then
		FAIL_SSV="nofail"
	else
		FAIL_SSV="fail"
	fi
	if [ ! -s $DEFAULTMERGE ]
	then
		# (Note: Cannot nlsize these messages, because we don't
		#  know where the message catalog is located or even
		#  if it is installed.)
		if [ "$FAIL_SSV" = "fail" ]
		then
			echo "$0:"
			echo "Error: Missing file $DEFAULTMERGE."
			echo "Cannot continue."
			exit 1
		fi
		FAIL_SSV=Recovered
		if [ -d /opt/merge ]
		then
			STATICMERGE=/opt/merge
		elif [ -d /opt/win4lin ]
		then
			STATICMERGE=/opt/win4lin
		else
			STATICMERGE=/usr/lib/merge
		fi
		if [ -d /var/merge ]
		then
			VARMERGE=/var/merge
		elif [ -d /var/win4lin ]
		then
			VARMERGE=/var/win4lin
		else
			VARMERGE=/usr/merge
		fi
		STATICMBIN=/usr/bin
	else 
		eval `grep "^STATICMERGE=" $DEFAULTMERGE`
		eval `grep "^VARMERGE="    $DEFAULTMERGE`
		eval `grep "^STATICMBIN="   $DEFAULTMERGE`
		eval `grep "^DOSWINBINDIR=" $DEFAULTMERGE`
	fi
	
	if [ "x$STATICMERGE" = "x" ]
	then
		# (Note: Cannot nlsize these messages, because the nls
		#        message stuff is in the directory that is not defined.)
		echo $0:
		echo "Error: Missing STATICMERGE setting in file $DEFAULTMERGE."
		if [ "$FAIL_SSV" = "fail" ]
		then
			echo "Cannot continue."
			exit 1
		fi
		FAIL_SSV=Recovered
		if [ -d /opt/merge ]
		then
			STATICMERGE=/opt/merge
		elif [ -d /opt/win4lin ]
		then
			STATICMERGE=/opt/win4lin
		else
			STATICMERGE=/usr/lib/merge
		fi
	fi
	export STATICMERGE
	if [ ! -d $STATICMERGE ]
	then
		if [ "$FAIL_SSV" = "fail" ]
		then
			# (Note: Cannot nlsize these messages, because the nls
			#  message stuff is in the directory that is missing.)
			echo $0:
			echo "Error: Missing directory $STATICMERGE."
			echo "Cannot continue."
			exit 1
		fi
		FAIL_SSV=Recovered
	fi
	if [ "x$VARMERGE" = "x" ]
	then
		Err_Msg_SSV SSV_NO_SETTING \
		 "$0 Error: Missing VARMERGE setting in file $DEFAULTMERGE.\n" \
		 "$0" VARMERGE "$DEFAULTMERGE"
		 
		if [ "$FAIL_SSV" = "fail" ]
		then
			Err_Msg_SSV SSV_FAIL "Cannot continue.\n"
			exit 1
		fi
		FAIL_SSV=Recovered
		if [ -d /var/merge ]
		then
			VARMERGE=/var/merge
		elif [ -d /var/win4lin ]
		then
			VARMERGE=/var/win4lin
		else
			VARMERGE=/usr/merge
		fi
	fi
	export VARMERGE
	if [ ! -d $VARMERGE ]
	then
		Err_Msg_SSV SSV_NO_DIRECTORY \
			"$0 Error: Missing directory $VARMERGE.\n" \
			"$0" "$VARMERGE"
		if [ "$FAIL_SSV" = "fail" ]
		then
			Err_Msg_SSV SSV_FAIL "Cannot continue.\n"
			exit 1
		fi
		FAIL_SSV=Recovered
	fi
	VERFILE="$VARMERGE/Version"
	if [ ! -f "$VERFILE" ]
	then
		# Normally $VARMERGE/Version should exist, but
		# the ""backup" version is $STATICMERGE/Version.
		VERFILE="$STATICMERGE/Version"
	fi
	if [ ! -f "$VERFILE" ]
	then
		Err_Msg_SSV SSV_NO_FILE \
			"$0 Error: Missing file $VARMERGE/Version.\n" \
			"$0" "$VARMERGE/Version"
		if [ "$FAIL_SSV" = "fail" ]
		then
			Err_Msg_SSV SSV_FAIL "Cannot continue.\n"
			exit 1
		fi
		FAIL_SSV=Recovered
	else
		eval `grep "^SYSTEMTYPE=" $VERFILE`
		eval `grep "^MERGENAME="  $VERFILE`
	fi
	if [ "x$SYSTEMTYPE" = "x" ]
	then
		Err_Msg_SSV SSV_NO_SETTING \
		  "$0 Error: Missing SYSTEMTYPE setting in file $VERFILE.\n" \
		  "$0" SYSTEMTYPE "$VERFILE"

		if [ "$FAIL_SSV" = "fail" ]
		then
			Err_Msg_SSV SSV_FAIL "Cannot continue.\n"
			exit 1
		fi
		FAIL_SSV=Recovered
		SSV_UNAME=`uname -s`
		case "$SSV_UNAME" in
		UnixWare)
			SYSTEMTYPE=uw7
			MERGENAME="SCO Merge"
			;;
		SCO_SV)
			SYSTEMTYPE=sco32
			MERGENAME="SCO Merge"
			;;
		Linux)
			SYSTEMTYPE=linux
			MERGENAME="Win4Lin"
			;;
		*)
			SYSTEMTYPE=unknown
			;;
		esac
	fi
	export SYSTEMTYPE
	if [ "x$MERGENAME" = "x" ]
	then
		MERGENAME=Merge
	fi
	export MERGENAME
	if [ "x$STATICMBIN" = "x" ]
	then
		Err_Msg_SSV SSV_NO_SETTING \
		  "$0 Error: Missing STATICMBIN setting in file $DEFAULTMERGE.\n" \
		  "$0" STATICMBIN "$DEFAULTMERGE"

		if [ "$FAIL_SSV" = "fail" ]
		then
			Err_Msg_SSV SSV_FAIL "Cannot continue.\n"
			exit 1
		fi
		FAIL_SSV=Recovered
		STATICMBIN=/usr/bin
	fi
	if [ "x$DOSWINBINDIR" = "x" ]
	then

		if [ "$FAIL_SSV" = "fail" ]
		then

			Err_Msg_SSV SSV_NO_SETTING \
             "$0 Error: Missing DOSWINBINDIR setting in file $DEFAULTMERGE.\n" \
				"$0" DOSWINBINDIR "$DEFAULTMERGE"
			Err_Msg_SSV SSV_FAIL "Cannot continue.\n"
			exit 1
		fi
		FAIL_SSV=Recovered
		if [ "$SYSTEMTYPE" = "linux" ]
		then
			DOSWINBINDIR=/bin
		else
			DOSWINBINDIR=/usr/bin
		fi
	fi
	[ "$FAIL_SSV" != "Recovered" ] && FAIL_SSV=allok
	[ "x$DM" = "xLCC.MERGE.UNIX.SSV" ] && unset DM
	# Unset the functions so we cannot get called twice.
	unset Err_Msg_SSV
	unset Set_System_Variables
}

Err_Msg_SSV()
{
	if [ "x$DM" != "xLCC.MERGE.UNIX.SSV" ]
	then
		if [ -f $STATICMERGE/set_nls.sh ]
		then
			.  $STATICMERGE/set_nls.sh
			DM=LCC.MERGE.UNIX.SSV
		fi
	fi
	if [ "x$DM" = "xLCC.MERGE.UNIX.SSV" ]
	then
		usrMsg ECHO "$1" "$2" "$3" "$4" "$5"
	else
		# The second parameter is the default string.
		uname -s | grep -i linux > /dev/null
		if [ $? -eq 0 ]
		then
			echo -e "$2"
		else
			echo "$2"
		fi
	fi
}

# Now set them!
Set_System_Variables $SSV_OPTION
