/*
 *  FreeMWare: run multiple x86 operating systems concurrently
 *  Copyright (C) 1999  Kevin P. Lawton
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */

#ifndef __FREEMWARE_H__
#define __FREEMWARE_H__

#ifndef __KERNEL__
// kernel doesn't export these to user progs
typedef   signed char s8;
typedef unsigned char u8;

typedef   signed short s16;
typedef unsigned short u16;

typedef   signed int s32;
typedef unsigned int u32;
#endif


/*
 * maximum physical memory which can be used for
 * the VM environment.
 */
#define FMW_MAX_PHY_MEGS 8


// This is the context saved (on the stack) by the exception handler
// in the monitor, before it transitions out to the host
// context to service the event.
typedef struct {
  u32  gs;
  u32  fs;
  u32  ds;
  u32  es;

  u32  edi;
  u32  esi;
  u32  ebp;
  u32  dummy_esp;
  u32  ebx;
  u32  edx;
  u32  ecx;
  u32  eax;

  u32  event_info; // low byte is vector, 2nd byte is event code
  u32  error;      // pushed by int or dummied

  u32  eip;
  u32  cs;
  u32  eflags;
  u32  esp;
  u32  ss;
  } __attribute__ ((packed)) guest_context_t;



#define ASM_USED(x) ((void) (x))

// For generating a software interrupt
#define soft_int(n)                             \
    asm volatile (                              \
        "    movb %b0, __soft_int_vector \n"    \
        "    jmp __soft_int_n            \n"    \
        "__soft_int_n:                   \n"    \
        "    sti                         \n"    \
        "    .byte 0xcd                  \n"    \
        "__soft_int_vector:              \n"    \
        "    .byte 0x00                  \n"    \
        :                                       \
        : "r" ((u8) (n) )                       \
        : "memory"                              \
    )

/*
 *  ioctl() names
 */

#define FMWALLOCVPHYS   0x6b02
#define FMWRESET        0x6b03
#define FMWRUNGUEST     0x6b04
#define FMWTEARDOWN     0x6b05

#endif  // #ifndef __FREEMWARE_H__
