/*
 *  FreeMWare: run multiple x86 operating systems concurrently
 *  Copyright (C) 1999  Kevin P. Lawton
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */

#ifndef __TSS_H__
#define __TSS_H__

typedef struct {
    u16 back, RESERVED0;      // Backlink
    u32 esp0;                 // The CK stack pointer
    u16 ss0,  RESERVED1;      // The CK stack selector
    u32 esp1;                 // The parent KL stack pointer
    u16 ss1,  RESERVED2;      // The parent KL stack selector
    u32 esp2;                 // Unused
    u16 ss2,  RESERVED3;      // Unused
    u32 cr3;                  // The page directory pointer
    u32 eip;                  // The instruction pointer
    u32 eflags;               // The flags
    u32 eax, ecx, edx, ebx;   // The general purpose registers
    u32 esp, ebp, esi, edi;   // The special purpose registers
    u16 es,   RESERVED4;      // The extra selector
    u16 cs,   RESERVED5;      // The code selector
    u16 ss,   RESERVED6;      // The application stack selector
    u16 ds,   RESERVED7;      // The data selector
    u16 fs,   RESERVED8;      // And another extra selector
    u16 gs,   RESERVED9;      // ... and another one
    u16 ldt,  RESERVED10;     // The local descriptor table
    u16 trap;                 // The trap flag (for debugging)
    u16 io;                   // The I/O Map base address
} __attribute__ ((packed)) tss_t;

#endif  // __TSS_H__
