:
# SCCSID(@(#)inst_mrg.sh	1.122) Modified 01:05:23 12/22/99
# Copyright (c) 1994-1999 TreLOS, Inc.
#############################################################################
#
#       Merge Installation Function
#
#	This script is called by the top level Merge install script.
#	With Custom+, the top level install script is "ccs.sh".
#	With pkgadd, the top level install script is "postinstall".
#	(The source file to "postinstall" is "postin_pa.sh").
#
###############################################################################

#############################################################################
#
#	This script *must* be called with STATICMERGE and VARMERGE set.
#
#############################################################################

EXIT_NORMAL=0		# Normal exit value
EXIT_FAIL=1		# Failure exit value

if [ "x$STATICMERGE" = "x" -o "x$VARMERGE" = "x" ]
then
	# This should never happen.
	echo STATICMERGE or VARMERGE not set.
	echo Cannot continue install.
	exit $EXIT_FAIL
fi
if [ ! -f $STATICMERGE/bin/s_init.sh ]
then
	# This should never happen.
	echo Missing $STATICMERGE/bin/s_init.sh
	echo Cannot continue install.
	exit $EXIT_FAIL
fi
. $STATICMERGE/bin/s_init.sh
# "s_init.sh" set up variables and functions used by this script.
# (such as NULL, LOG_FILE, usrMsg, and everything from the Version file.)


# Function part -------

#####################################################################
# Main -
#####################################################################

TMPDIR=$VARMERGE/tmp
TMP1=$TMPDIR/post$$
TMP2=$TMPDIR/p2$$

#############################################################################
#	Make sure that files from a previous installation that
#	ought to be removed are removed.
#############################################################################

rm -f $VARMERGE/doing_mkimg $VARMERGE/auto_mkimg
rm -f $VARMERGE/doing_dosinst $VARMERGE/auto_dosinst

# Create an error log, with a note to the user, just in case ...

usrMsg LOG INST_LOG_HDR \
"# This is a log file created by the $MERGENAME installation process.\n\
# If there is an error during the $MERGENAME controlled portion of\n\
# installation, it will show up here.  This file is created during\n\
# installation, and may be deleted by the user.  This file may also\n\
# contain contain non-error output from other (standard) UNIX programs.\n" \
"$MERGENAME" "$MERGENAME"

echo "MERGECOPYRIGHT=$MERGECOPYRIGHT" >> $LOG_FILE
echo "FULLTITLE=$FULLTITLE" >> $LOG_FILE
echo "REL_COMMENT=$REL_COMMENT" >> $LOG_FILE
echo "MERGENAME=$MERGENAME" >> $LOG_FILE
echo "SYSTEMTYPE=$SYSTEMTYPE" >> $LOG_FILE


#############################################################################
#
#    Do generic Merge installation tasks
#
#	Set special "dos" permissions.
#	Add special DOS startup message (SCO only)
#	Add gettydefs if necessary
#	Make "doslp" printer name to use default printer device.
#	Add Merge mouse driver and ems driver to the global config.sys
#	Add path and prompt command to the global autoexec.bat file.
#    Solaris ONLY
#	Configure Merge mouse using the UI mergeadmin utility
#
#############################################################################

usrMsg ECHO INSTALL_2 \
	"Configuring $MERGENAME files.  Please wait...\n" "$MERGENAME"

#############################################################################
#	Set up for possible upgrade.
#############################################################################
if [ -f $VARMERGE/Version ]
then
	# Merge was previously installed.  Save the Version file.
	rm -f $VARMERGE/Version.prev
	mv $VARMERGE/Version $VARMERGE/Version.prev
	INSTALL_KIND=upgrade
else
	INSTALL_KIND=fresh
fi

# Copy the version file to $VARMERGE so that a copy of it is left
# behind when Merge is removed, so that the re-installation can
# see which version was installed.  Also any additions should be
# done to the copy in $VARMERGE.  The original MUST NOT be changed.
cp $STATICMERGE/Version $VARMERGE/Version
chown bin $VARMERGE/Version
chgrp bin $VARMERGE/Version
chmod 444 $VARMERGE/Version
if [ -f $VARMERGE/Version.prev ]
then
	. $VARMERGE/Version.prev
	OLDVER=$VERSION
	OLDMAJORVER=`echo $OLDVER | cut -c1`
	OLDMINORVER=`echo $OLDVER | cut -c3`
	. $VARMERGE/Version
	echo OLDVER=$OLDVER		>> $LOG_FILE
	echo OLDMAJORVER=$OLDMAJORVER	>> $LOG_FILE
	echo OLDMINORVER=$OLDMINORVER	>> $LOG_FILE
else
	OLDMAJORVER=
fi
if [ "$INSTALLTYPE" = "pkgadd" ]
then
	[ "x$PKGINST" = "x" ] && PKGINST=merge
	echo PKGNAME=$PKGINST >> $VARMERGE/Version
fi

#############################################################################
#	Create the /etc/default/merge file if it does not already exist.
#############################################################################
if [ ! -f /etc/default/merge ]
then
	if [ ! -d /etc/default ]
	then
		mkdir /etc/default
		chmod 775  /etc/default
		chown root /etc/default
		chgrp bin  /etc/default
	else
		chmod ugo+rx /etc/default
	fi
	echo "# $MERGENAME defaults" > /etc/default/merge
fi
chmod 664 /etc/default/merge		>> $NULL 2>&1
chgrp bin /etc/default/merge		>> $NULL 2>&1
chown bin /etc/default/merge		>> $NULL 2>&1

#############################################################################
# If the links.txt file exists, then make the indicated links.
#############################################################################
if [ -f $STATICMERGE/links.txt ]
then
	while read FROM TO
	do
		ln $FROM $TO
		[ $? -ne 0 ] && ln -s $FROM $TO
	done < $STATICMERGE/links.txt
fi
if [ "$SYSTEMTYPE" = "linux" ]
then
	TARG=/etc/rc.d
	if [ ! -d $TARG/rc2.d -a -d /etc/rc2.d ]
	then
		TARG=/etc
	fi
	# Set up the init scripts, which we want to be symbolic links.
	for i in rc2.d rc3.d rc4.d rc5.d
	do
		ln -s /etc/rc.d/init.d/Win4Lin $TARG/$i/S77Win4Lin
	done
fi

#############################################################################
if [ "$SYSTEMTYPE" = "linux" ]
then
	EX_CMD=
	for i in /bin/ex /usr/bin/ex
	do
		[ -f $i -o -L $i ] && EX_CMD=$i
	done
	if [ "x$EX_CMD" = "x" ]
	then
		# (Must be on a SuSE system!)
		# Some of our scripts use "ex" to edit files, so
		# we have to make sure that it is available.
		for i in /bin /usr/bin
		do
			if [ -f $i/vim -o -L $i/vim ]
			then
				ln -s $i/vim $i/ex
				break
			fi
			if [ -f $i/vi -o -L $i/vi ]
			then
				ln -s $i/vi $i/ex
				break
			fi
		done
	fi
fi

#############################################################################
#	Do the automatic localization setup.
#	(Note: initlocal.sh originated as one of i_loc_sco.sh, i_loc_sol.sh,
#	or i_loc_sv4.sh, depending on the system type.)
#############################################################################
[ -f $STATICMERGE/initlocal.sh ] && sh $STATICMERGE/initlocal.sh

#############################################################################
#	Set up the permissions for dosexec.
#############################################################################
sh $STATICMERGE/instdx.sh

#############################################################################
#  Create the image directories if not already there.
#############################################################################

if [ ! -d $VARMERGE/image ]
then
	rm -f     $VARMERGE/image
	mkdir -p  $VARMERGE/image		>> $NULL 2>&1
	chmod 775 $VARMERGE/image		>> $NULL 2>&1
	chgrp bin $VARMERGE/image		>> $NULL 2>&1
	chown bin $VARMERGE/image		>> $NULL 2>&1
fi
if [ ! -d $VARMERGE/image/std ]
then
	rm -f     $VARMERGE/image/std
	mkdir -p  $VARMERGE/image/std		>> $NULL 2>&1
	chmod 775 $VARMERGE/image/std		>> $NULL 2>&1
	chgrp bin $VARMERGE/image/std		>> $NULL 2>&1
	chown bin $VARMERGE/image/std		>> $NULL 2>&1
fi
for i in std std/mono std/cga std/vga
do
	if [ ! -d $VARMERGE/image/$i ]
	then
		rm -f     $VARMERGE/image/$i
		mkdir -p  $VARMERGE/image/$i	>> $NULL 2>&1
		chmod 775 $VARMERGE/image/$i	>> $NULL 2>&1
		chgrp bin $VARMERGE/image/$i	>> $NULL 2>&1
		chown bin $VARMERGE/image/$i	>> $NULL 2>&1
	fi
done

for i in $VARMERGE/image/std/*
do
	[ ! -d $i ] && continue
	# First do config.sys.
	THIS=
	if [ "$i" = "$VARMERGE/image/std/vga" ]
	then
		THIS=$STATICMERGE/for_image/config.vga
		[ ! -f $THIS ] && THIS=
	fi
	if [ "x$THIS" = "x" ]
	then
		THIS=$STATICMERGE/for_image/config.std
		[ ! -f $THIS ] && THIS=
	fi
	if [ "x$THIS" != "x" -a ! -f $i/config.sys ]
	then
		cp $THIS  $i/config.sys		>> $NULL 2>&1
		chmod 664 $i/config.sys		>> $NULL 2>&1
		chgrp bin $i/config.sys		>> $NULL 2>&1
		chown bin $i/config.sys		>> $NULL 2>&1
	fi

	# Now do autoexec.bat.
	THIS=
	if [ "$i" = "$VARMERGE/image/std/vga" ]
	then
		THIS=$STATICMERGE/for_image/autoexec.vga
		[ ! -f $THIS ] && THIS=
	fi
	if [ "x$THIS" = "x" ]
	then
		THIS=$STATICMERGE/for_image/autoexec.std
		[ ! -f $THIS ] && THIS=
	fi
	if [ "x$THIS" != "x" -a ! -f $i/autoexec.bat ]
	then
		cp $THIS  $i/autoexec.bat	>> $NULL 2>&1
		chmod 664 $i/autoexec.bat	>> $NULL 2>&1
		chgrp bin $i/autoexec.bat	>> $NULL 2>&1
		chown bin $i/autoexec.bat	>> $NULL 2>&1
	fi
done

#############################################################################
# Create a fresh $VAMREGE/instC directory.  This is where the special
# "snap.cfd" configuration has it's drive C located.
#############################################################################
rm -rf    $VARMERGE/instC
mkdir -p  $VARMERGE/instC	>> $NULL 2>&1
chmod 775 $VARMERGE/instC	>> $NULL 2>&1
chgrp bin $VARMERGE/instC	>> $NULL 2>&1
chown bin $VARMERGE/instC	>> $NULL 2>&1

#############################################################################
#  Create $VARMERGE/dosroot directory if not already there.
#############################################################################
if [ ! -d $VARMERGE/dosroot ]
then
	rm -f     $VARMERGE/dosroot
	mkdir -p  $VARMERGE/dosroot	>> $NULL 2>&1
	chmod 775 $VARMERGE/dosroot	>> $NULL 2>&1
	chgrp bin $VARMERGE/dosroot	>> $NULL 2>&1
	chown bin $VARMERGE/dosroot	>> $NULL 2>&1
fi
for i in dos share mkimgtmp
do
	if [ ! -d $VARMERGE/dosroot/$i ]
	then
		rm -f     $VARMERGE/dosroot/$i
		mkdir -p  $VARMERGE/dosroot/$i	>> $NULL 2>&1
		chmod 775 $VARMERGE/dosroot/$i	>> $NULL 2>&1
		chgrp bin $VARMERGE/dosroot/$i	>> $NULL 2>&1
		chown bin $VARMERGE/dosroot/$i	>> $NULL 2>&1
	fi
done
chmod 1777 $VARMERGE/dosroot/share	>> $NULL 2>&1
chmod 1777 $VARMERGE/dosroot/mkimgtmp	>> $NULL 2>&1

#############################################################################
#	Create the j:\mwindows directory ($VARMERGE/dosroot/mwindows).
#	Symbolically link all the files in $STATICMERGE/windrv/f
#	into the directory.
#############################################################################
SAVEDIR=`pwd`
cd $VARMERGE/dosroot
rm -f     mwindows/*
rmdir     mwindows	>> $NULL 2>&1
mkdir     mwindows	>> $NULL 2>&1
chgrp bin mwindows	>> $NULL 2>&1
chown bin mwindows	>> $NULL 2>&1
chmod 775 mwindows	>> $NULL 2>&1
cd $STATICMERGE/windrv/f
for i in *
do
	# If not a file or directory, skip it.
	[ ! -f $i -a ! -d $i ] && continue
	# If there already is a file or directory by
	# that name, then skip it and log the problem.
	if [ -f $VARMERGE/dosroot/mwindows/$i -o \
	     -d $VARMERGE/dosroot/mwindows/$i ]
	then
		echo "ERROR: Failed to clean dosroot/mwindows dir" >> $LOG_FILE 
		echo "       File $i is still there!" >> $LOG_FILE 
		continue
	fi
	ln -s $STATICMERGE/windrv/f/$i $VARMERGE/dosroot/mwindows/$i
done
cd $SAVEDIR

#############################################################################
#	Create the j:\mwin95 directory ($VARMERGE/dosroot/mwin95)
#	by symbolically linking to the $STATICMERNGE/mwin95 directory.
#############################################################################
SAVEDIR=`pwd`
cd $VARMERGE/dosroot
if [ -d mwin95 ]
then
	rm -rf mwin95
else
	rm -f  mwin95
fi
ln -s $STATICMERGE/mwin95 mwin95
cd $SAVEDIR

#############################################################################
#  Create $VARMERGE/rhome directory if not already there.
#  This is intended to be used as a private HOME directory
#  for root.  It will be used if MERGE_ROOT_HOME is set
#  (in /etc/default/merge or in the environment) to the
#  full path to this directory.
#############################################################################
if [ ! -d $VARMERGE/rhome ]
then
	rm -f     $VARMERGE/rhome
	mkdir -p  $VARMERGE/rhome	>> $NULL 2>&1
	chmod 755 $VARMERGE/rhome	>> $NULL 2>&1
	chgrp bin $VARMERGE/rhome	>> $NULL 2>&1
	chown root $VARMERGE/rhome	>> $NULL 2>&1
fi

# Just in case a previous version of Merge did not clean up,
# do this remove of a potential symbolic link first, so we
# can (re)create it cleanly.
rm -f $VARMERGE/dosroot/tmp 2>/dev/null

# Just in case a previous version of Merge used a regular directory
# for "tmp", and did not remove it, then leave it as is.
if [ ! -d  $VARMERGE/dosroot/tmp ]
then
	if [ -d /var/tmp ]
	then
		ln -s /var/tmp $VARMERGE/dosroot/tmp
	elif [ -d /usr/tmp ]
	then
		ln -s /usr/tmp $VARMERGE/dosroot/tmp
	else
		ln -s /tmp $VARMERGE/dosroot/tmp
	fi
fi

if [ -d $STATICMERGE/dosside ]
then
	[ -f $VARMERGE/dosroot/merge ] && rm -f $VARMERGE/dosroot/merge
	if [ ! -d $VARMERGE/dosroot/merge ]
	then
		ln -s $STATICMERGE/dosside $VARMERGE/dosroot/merge
	fi
fi

#############################################################################
#  Create $VARMERGE/dosi directory if not already there.
#############################################################################
VAR_DOSI=$VARMERGE/dosi
if [ ! -d $VAR_DOSI ]
then
	mkdir $VAR_DOSI
	chown bin $VAR_DOSI
	chgrp bin $VAR_DOSI
	chmod 775 $VAR_DOSI
fi

#############################################################################
#	Perform backwards compatibility stuff when a Merge version
#	previous to 3.2 was previously installed.
#############################################################################
U_3XFLAG=notupgrade

Upgrade_from_pre_32()
{
	[ -z "$OLDMAJORVER" ] && return
	[ $OLDMAJORVER -gt 3 ] && return
	[ -z "$OLDMINORVER" ] && OLDMINORVER=0
	[ $OLDMAJORVER -eq 3 -a $OLDMINORVER -ge 2 ] && return
	# The previous version was pre-3.2, so make the backwards
	# compatibility changes.
	# First do the backwards compatibility for the drive functionality.
	grep '^MERGE_CLASSIC_DRIVES=' /etc/default/merge  >> $NULL 2>&1
	if [ $? -ne 0 ]
	then
		# Was not already set, so add it!
		echo 'MERGE_CLASSIC_DRIVES=on' >> /etc/default/merge
	fi
	# Create a /usr/dbin directory.
	if [ ! -f /usr/dbin -a ! -d /usr/dbin ]
	then
		ln -s $VARMERGE/dosroot/dos /usr/dbin
	fi
	# Link files and directories from /usr/ldbin into $VARMERGE/dosroot.
	if [ -d /usr/ldbin ]
	then
		SAVEDIR=`pwd`
		cd /usr/ldbin
		for i in *
		do
			# If not a file or directory, skip it.
			[ ! -f $i -a ! -d $i ] && continue
			# If there already is a file or directory by
			# that name, then skip it.
			[ -f $VARMERGE/dosroot/$i ] && continue
			[ -d $VARMERGE/dosroot/$i ] && continue
			ln -s /usr/ldbin/$i $VARMERGE/dosroot/$i
		done
		cd $SAVEDIR
	fi
	# Remove items not used any more, or items that we want the
	# to use newer defaults.
	#  1: Use new default for MERGE_MAXOPEN.
	#  2: Remove "SMARTDRV_PARAM".  It is not used anymore.
	#	(The newer "MERGE_SMARTDRV_PARAM" is used instead, but
	#	for a slightly different purpose.) 
	cat /etc/default/merge \
		| grep -v "^MERGE_MAXOPEN=" \
		| grep -v "^SMARTDRV_PARAM=" \
		> $TMPDIR/edm$$
	cp $TMPDIR/edm$$ /etc/default/merge
	rm -f $TMPDIR/edm$$
	U_3XFLAG=upgrade3x
}
Upgrade_from_pre_32

#############################################################################
#	Perform backwards compatibility stuff the previous version of Merge
#	was 3.2 or higher 3.x.
#############################################################################
Upgrade_from_32()
{
	[ -z "$OLDMAJORVER"  ] && return
	[ $OLDMAJORVER -ne 3 ] && return
	[ -z "$OLDMINORVER"  ] && return
	[ $OLDMINORVER -lt 2 ] && return
	#
	# When upgrading from Merge 3.2+, we want to use the new default
	# for MERGE_AUTOCREATE_LOCKDIR.  So remove the current setting
	# so later code will set it to the new default.
	cat /etc/default/merge \
		| grep -v "^MERGE_AUTOCREATE_LOCKDIR=" \
		> $TMPDIR/edm$$
	cp $TMPDIR/edm$$ /etc/default/merge
	rm -f $TMPDIR/edm$$
	U_32FLAG=on
	U_3XFLAG=upgrade3x
}
Upgrade_from_32

#############################################################################
#	If we are doing an upgrade from Merge 3.x, then record this fact
#	in /etc/default/merge.
#############################################################################
Upgrade_3x()
{
	[ "$U_3XFLAG" != "upgrade3x" ] && return
	grep "^UPGRADEDFROM3X=" /etc/default/merge	>> $NULL 2>&1
	# (grep returns 0 if a match found)
	if [ $? -ne 0 ]
	then
		echo UPGRADEDFROM3X=on	>> /etc/default/merge
	fi
}
Upgrade_3x

#############################################################################
# Add misc things to /etc/default/merge
#############################################################################
#
# Remove certain old settings that we want to update.
#  1: Always update VARMERGE, STATICMERGE, STATICMBIN, and DOSWINBINDIR.
#  2: If the old default of "MERGE_CD_DEVNAME=unknown", replace with
#     the new default.
cat /etc/default/merge \
	| grep -v "^VARMERGE=" \
	| grep -v "^STATICMERGE=" \
	| grep -v "^STATICMBIN=" \
	| grep -v "^DOSWINBINDIR=" \
	| grep -v "^MERGE_CD_DEV_NAME=unknown" \
	> $TMPDIR/edm$$
cp $TMPDIR/edm$$ /etc/default/merge
rm -f $TMPDIR/edm$$

# Put in current VARMERGE and STATICMERGE settings.
EchoText VARMERGE=\"$VARMERGE\"		>> /etc/default/merge
EchoText STATICMERGE=\"$STATICMERGE\"	>> /etc/default/merge
EchoText STATICMBIN=\"$STATICMBIN\"	>> /etc/default/merge
EchoText DOSWINBINDIR=\"$DOSWINBINDIR\"	>> /etc/default/merge

# Put in CD-ROM device setting, if is not already there.
grep "^MERGE_CD_DEV_NAME=" /etc/default/merge	>> $NULL 2>&1
# (grep returns 0 if a match found)
if [ $? -ne 0 ]
then
	if [ "$SYSTEMTYPE" = "sco32" -o "$SYSTEMTYPE" = "sco" ]
	then
		echo MERGE_CD_DEV_NAME=/dev/cd0	>> /etc/default/merge
	elif [ "$SYSTEMTYPE" = "uw7" ]
	then
		CDNAME=unknown
		if [ -d /dev/cdrom ]
		then
			for i in /dev/cdrom/*
			do
				if [ -b $i ]
				then
					CDNAME="$i"
					break;
				fi
			done
		fi
		echo MERGE_CD_DEV_NAME="$CDNAME"	>> /etc/default/merge
	elif [ "$SYSTEMTYPE" = "linux" ]
	then
		echo MERGE_CD_DEV_NAME=/dev/cdrom	>> /etc/default/merge
	else
		# If we get here, it means that the people who ported Merge
		# for this system forgot to port this section.
		echo MERGE_CD_DEV_NAME=unknown	>> /etc/default/merge
	fi
fi

#
# Make sure that
# 	MERGE_RLOCK_FILETABLE_SIZE,
#	MERGE_RLOCK_OPENTABLE_SIZE,
#	MERGE_RLOCK_LOCKTABLE_SIZE, and
#	MERGE_MAXOPEN
# are at least as large as the values needed to run Winstone96 and Winstone97
#
MERGE_RLOCK_FILETABLE_SIZE=0
MERGE_RLOCK_OPENTABLE_SIZE=0
MERGE_RLOCK_LOCKTABLE_SIZE=0
MERGE_MAXOPEN=0
grep '^MERGE_RLOCK_' /etc/default/merge > $TMPDIR/mrl$$
grep '^MERGE_MAXOPEN' /etc/default/merge >> $TMPDIR/mrl$$
sed \
	-e '/^MERGE_RLOCK_FILETABLE_SIZE/d' \
	-e '/^MERGE_RLOCK_OPENTABLE_SIZE/d' \
	-e '/^MERGE_RLOCK_LOCKTABLE_SIZE/d' \
	-e '/^MERGE_MAXOPEN/d' \
    /etc/default/merge > $TMPDIR/edm$$
cp $TMPDIR/edm$$ /etc/default/merge
rm -f $TMPDIR/edm$$
. $TMPDIR/mrl$$
rm -f $TMPDIR/mrl$$
[ "$MERGE_RLOCK_FILETABLE_SIZE" -ge 512 ] || MERGE_RLOCK_FILETABLE_SIZE=512
[ "$MERGE_RLOCK_OPENTABLE_SIZE" -ge 768 ] || MERGE_RLOCK_OPENTABLE_SIZE=768
[ "$MERGE_RLOCK_LOCKTABLE_SIZE" -ge 1024 ] || MERGE_RLOCK_LOCKTABLE_SIZE=1024
[ "$MERGE_MAXOPEN" -ge 512 ] || MERGE_MAXOPEN=512
echo "MERGE_RLOCK_FILETABLE_SIZE=$MERGE_RLOCK_FILETABLE_SIZE" \
	>> /etc/default/merge
echo "MERGE_RLOCK_OPENTABLE_SIZE=$MERGE_RLOCK_OPENTABLE_SIZE" \
	>> /etc/default/merge
echo "MERGE_RLOCK_LOCKTABLE_SIZE=$MERGE_RLOCK_LOCKTABLE_SIZE" \
	>> /etc/default/merge
echo "MERGE_MAXOPEN=$MERGE_MAXOPEN" \
	>> /etc/default/merge

unset MERGE_RLOCK_FILETABLE_SIZE
unset MERGE_RLOCK_OPENTABLE_SIZE
unset MERGE_RLOCK_LOCKTABLE_SIZE
unset MERGE_MAXOPEN

# "yes" and "no" settings are obsolete and must replaced with "on" and "off".
cat /etc/default/merge | sed -e "s/=yes/=on/g" -e "s/=no/=off/g" > $TMPDIR/edm$$
cp $TMPDIR/edm$$ /etc/default/merge
rm -f $TMPDIR/edm$$

# Put in MERGE_AUTOBACKGROUND setting, if is not already there.
grep "^MERGE_AUTOBACKGROUND=" /etc/default/merge	>> $NULL 2>&1
if [ $? -ne 0 ]
then
	echo MERGE_AUTOBACKGROUND=off	>> /etc/default/merge
fi

# Put in DOS_LOG setting, if is not already there.
grep "^DOS_LOG=" /etc/default/merge	>> $NULL 2>&1
if [ $? -ne 0 ]
then
	echo DOS_LOG=+	>> /etc/default/merge
fi

# Put XCRT_LOG setting, if is not already there.
grep "^XCRT_LOG=" /etc/default/merge	>> $NULL 2>&1
if [ $? -ne 0 ]
then
	echo XCRT_LOG=+	>> /etc/default/merge
fi

# Put in MERGE_SMARTDRV_PARAM setting, if is not already there.
# The setting is "512 512" to force the smartdrv buffer size of 512K.
grep "^MERGE_SMARTDRV_PARAM=" /etc/default/merge	>> $NULL 2>&1
if [ $? -ne 0 ]
then
	echo MERGE_SMARTDRV_PARAM="512 512"	>> /etc/default/merge
fi

# Put in MERGE_ROOT_HOME setting, if is not already there.
grep "^MERGE_ROOT_HOME=" /etc/default/merge	>> $NULL 2>&1
if [ $? -ne 0 ]
then
	# When a fresh install (make sure it is fresh by making sure
	# that the directory "/.merge" does not exist) make the "root"
	# user's DOS/Windows home directory be $VARMERGE/rhome.
	if  [ "$INSTALL_KIND" = "fresh" -a ! -d /.merge ]
	then
		echo MERGE_ROOT_HOME=$VARMERGE/rhome	>> /etc/default/merge
	else
		# The root user has already used Merge in a previous
		# version of Merge, with the home drive being "/".
		echo MERGE_ROOT_HOME=/			>> /etc/default/merge
	fi
fi

######################################################################
# Put in a bunch of settings if they are not already there.
######################################################################
grep "^MERGE_NETWORK=" /etc/default/merge	>> $NULL 2>&1
if [ $? -ne 0 ]
then
    echo MERGE_NETWORK=off			>> /etc/default/merge
fi

grep "^MERGE_AUTOCREATE_LOCKDIR=" /etc/default/merge	>> $NULL 2>&1
if [ $? -ne 0 ]
then
    echo MERGE_AUTOCREATE_LOCKDIR=on		>> /etc/default/merge
fi

grep "^MERGE_REL_HOMEDRV_DIR=" /etc/default/merge	>> $NULL 2>&1
if [ $? -ne 0 ]
then
    echo MERGE_REL_HOMEDRV_DIR=win		>> /etc/default/merge
fi

grep "^MERGE_MAP_HASH_LEN=" /etc/default/merge	>> $NULL 2>&1
if [ $? -ne 0 ]
then
    echo MERGE_MAP_HASH_LEN=5		>> /etc/default/merge
fi

grep "^MERGE_MAX_XDISPLAY=" /etc/default/merge		>> $NULL 2>&1
if [ $? -ne 0 ]
then
    echo # MERGE_MAX_XDISPLAY is set to the word auto or use form WIDTHxHIGHT \
							>> /etc/default/merge
    echo # E.g. 1024x768				>> /etc/default/merge
    echo MERGE_MAX_XDISPLAY=auto			>> /etc/default/merge
fi

grep "^MERGE_CAPSLOCK_SHIFT_ENABLE=" /etc/default/merge	>> $NULL 2>&1
if [ $? -ne 0 ]
then
	echo MERGE_CAPSLOCK_SHIFT_ENABLE=on	>> /etc/default/merge
fi

grep "^MERGE_XBELL_MILLISEC=" /etc/default/merge	>> $NULL 2>&1
if [ $? -ne 0 ]
then
    echo MERGE_XBELL_MILLISEC=250			>> /etc/default/merge
fi

grep "^MERGE_XBELL_ENABLE=" /etc/default/merge		>> $NULL 2>&1
if [ $? -ne 0 ]
then
    echo MERGE_XBELL_ENABLE=on				>> /etc/default/merge
fi

grep "^MERGE_STATUS_SHIFTLOCK_ENABLE=" /etc/default/merge >> $NULL 2>&1
if [ $? -ne 0 ]
then
    echo MERGE_STATUS_SHIFTLOCK_ENABLE=off		>> /etc/default/merge
fi

grep "^MERGE_STATUS_SOUND_ENABLE=" /etc/default/merge	>> $NULL 2>&1
if [ $? -ne 0 ]
then
    echo MERGE_STATUS_SOUND_ENABLE=off			>> /etc/default/merge
fi

grep "^MERGE_STATUS_SOUND_MSG=" /etc/default/merge	>> $NULL 2>&1
if [ $? -ne 0 ]
then
    echo MERGE_STATUS_SOUND_MSG=Bzzzzz			>> /etc/default/merge
fi

grep "^MERGE_CD_SPOOF_OPEN_RETRY_SECONDS=" /etc/default/merge >> $NULL 2>&1
if [ $? -ne 0 ]
then
    echo MERGE_CD_SPOOF_OPEN_RETRY_SECONDS=1		>> /etc/default/merge
fi

grep "^MERGE_CD_RELEASE_SECONDS=" /etc/default/merge	>> $NULL 2>&1
if [ $? -ne 0 ]
then
    echo MERGE_CD_RELEASE_SECONDS=18			>> /etc/default/merge
fi

grep "^MERGE_FLOPPY_RELEASE_SECONDS=" /etc/default/merge >> $NULL 2>&1
if [ $? -ne 0 ]
then
	echo MERGE_FLOPPY_RELEASE_SECONDS=30		>> /etc/default/merge
fi

if [ "$SYSTEMTYPE" = "sco32" -o "$SYSTEMTYPE" = "sco" ]
then
	grep "^MERGE_DIRECT_FLOPPY_ENABLE=" /etc/default/merge	>> $NULL 2>&1
	if [ $? -ne 0 ]
	then
		echo MERGE_DIRECT_FLOPPY_ENABLE=off	>> /etc/default/merge
	fi
fi

grep "^MERGE_POSITION=" /etc/default/merge >> $NULL 2>&1
if [ $? -ne 0 ]
then
	echo MERGE_POSITION=		>> /etc/default/merge
fi

if [ "$SYSTEMTYPE" = "uw7" ]
then
	grep "^MERGE_FONT_SERVER=" /etc/default/merge	>> $NULL 2>&1
	if [ $? -ne 0 ]
	then
		echo MERGE_FONT_SERVER=off	>> /etc/default/merge
	fi
fi

if [ "$SYSTEMTYPE" = "linux" ]
then
	grep "^MERGE_HELP_BROWSER=" /etc/default/merge >> $NULL 2>&1
	if [ $? -ne 0 ]
	then
		# Poke around to see which if any html browsers
		# are available.  Preferred is kdehelp, since it
		# is lightweight.
		FOO=`which kdehelp 2>/dev/null`
		[ ! -x "$FOO" ] && FOO=
		if [ -z "$FOO" ]
		then
			# Perhaps kdehelp is not on the standard path.
			# Try in "/opt/kde/bin".
			if [ -x /opt/kde/bin/kdehelp ]
			then
			      FOO=/opt/kde/bin/kdehelp
			fi
		fi
		if [ -z "$FOO" ]
		then
			# Is netscape installed?
			FOO=`which netscape 2>/dev/null`
			if [ -x "$FOO" ]
			then
				# Use netscape via our script "netscapefile".
				FOO=/usr/bin/netscapefile
			else
				FOO=
			fi
		fi
		if [ -z "$FOO" ]
		then
			# Is lynx installed?
			FOO=`which lynx 2>/dev/null`
			if [ -x "$FOO" ]
			then
				# Use lynx in an xterm.
				FOO="xterm -e $FOO"
			else
				FOO=
			fi
		fi
		if [ -n "$FOO" ]
		then
			EchoText MERGE_HELP_BROWSER="\"$FOO\"" \
				>> /etc/default/merge
		else
			EchoText '# MERGE_HELP_BROWSER="htmlBrowserProgram"'\
				>> /etc/default/merge
		fi
	fi
fi

grep "^MERGE_FIRST_USER_DOSINST=" /etc/default/merge >> $NULL 2>&1
if [ $? -ne 0 ]
then
	EchoText MERGE_FIRST_USER_DOSINST=off	>> /etc/default/merge
fi

grep "^MERGE_X_LOCK_KEYS_TOGGLE=" /etc/default/merge >> $NULL 2>&1
if [ $? -ne 0 ]
then
	if [ "$SYSTEMTYPE" = "linux" ]
	then
		# XFree86 does not "toggle" the lock type keys.
		EchoText MERGE_X_LOCK_KEYS_TOGGLE=off	>> /etc/default/merge
	else
		EchoText MERGE_X_LOCK_KEYS_TOGGLE=on	>> /etc/default/merge
	fi
fi

grep "^MERGE_CDROM_START=" /etc/default/merge >> $NULL 2>&1
if [ $? -ne 0 ]
then
	EchoText MERGE_CDROM_START=N	>> /etc/default/merge
fi


#############################################################################
#       Put 8-bit tty line info at the end of the /etc/gettydefs file,
#	unless it is already there.
#############################################################################

if [ -r $STATICMERGE/gettydefs ]
then
	grep 9600NP $ROOT/etc/gettydefs		>> $NULL 2>&1
	# grep returns 1 if no matches found
	if [ $? -eq 1 ]
	then
		cat $STATICMERGE/gettydefs >> $ROOT/etc/gettydefs
	fi
fi

echo  >> $LOG_FILE 
#############################################################################
#	Add doslp printer device
#############################################################################
if [ -f $STATICMERGE/bin/instdoslp ]
then
	. $STATICMERGE/bin/instdoslp
fi

#############################################################################
#	Add Merge mouse driver and ems driver to the global config.sys
#		if they are not already there.
#	 (grep returns 0 if matches found)
#############################################################################
echo  >> $LOG_FILE
		# Hardwired system-wide config.sys location.
GLOBAL_CONFIGSYS=$VARMERGE/dosroot/config.sys
usrMsg TRACE INST_CONFIG_SYS "Updating global config.sys ...\n"

rm -f $TMP1
if [ -f $GLOBAL_CONFIGSYS ]
then
	echo "config.sys exists. Do charconv" >> $LOG_FILE
	# Do a DOS to UNIX conversion.
	$STATICMERGE/charconv -d -x -z $GLOBAL_CONFIGSYS $TMP1 >> $LOG_FILE 2>&1
	if [ ! -f $TMP1 ]
	then
		echo "WARNING:charconv failed." >> $LOG_FILE
		# Somehow charconv failed, so recover by just copying the file.
		cp $GLOBAL_CONFIGSYS $TMP1
	fi
else
	echo "config.sys does not exist. Use empty file"  >> $LOG_FILE #TESTING#
	> $TMP1
	# If config.sys exist in the root directory and $GLOBAL_CONFIGSYS
	# doesn't exists then move /config.sys to the place where it should
	# be and make the ln for backward compatibility.
	if [ -f /config.sys -a ! -f $GLOBAL_CONFIGSYS ]
	then
		echo "Move /config.sys to $GLOBAL_CONFIGSYS" >> $LOG_FILE
		mv /config.sys $GLOBAL_CONFIGSYS
		ln -s $GLOBAL_CONFIGSYS /config.sys
	fi
fi

> $TMP2
					# Hardwired drive letter j:
					# Hardwired DOS path
grep -i '\\merge\\lib\\mouse.sys' $TMP1	>> $NULL 2>&1
[ $? -ne 0 ] && EchoText 'device=j:\\merge\\lib\\mouse.sys' >> $TMP2

					# Hardwired drive letter j:
					# Hardwired DOS path
grep -i '\\merge\\lib\\emm.sys' $TMP1		>> $NULL 2>&1
[ $? -ne 0 ] && EchoText 'device=j:\\merge\\lib\\emm.sys' >> $TMP2
cat $TMP1 >> $TMP2

rm -f $GLOBAL_CONFIGSYS
# Do a UNIX to DOS conversion.
$STATICMERGE/charconv -p -x $TMP2 $GLOBAL_CONFIGSYS >> $LOG_FILE 2>&1
if [ ! -s $GLOBAL_CONFIGSYS ]
then
	echo "WARNING:charconv failed." >> $LOG_FILE
	# Somehow charconv failed, so recover by just renaming the file.
	mv $TMP2 $GLOBAL_CONFIGSYS
fi
rm -f $TMP1 $TMP2

chmod 664 $GLOBAL_CONFIGSYS >> $NULL 2>&1
chgrp bin $GLOBAL_CONFIGSYS >> $NULL 2>&1
chown bin $GLOBAL_CONFIGSYS >> $NULL 2>&1

ls -l $GLOBAL_CONFIGSYS >> $LOG_FILE

#############################################################################
#	Add PATH and PROMPT to the global autoexec.bat if not already set.
#############################################################################
echo  >> $LOG_FILE
usrMsg TRACE INST_AUTOEXEC "Updating global autoexec.bat ...\n"

		# Hardwired system-wide autoexec.bat location.
GLOBAL_AUTOEXEC=$VARMERGE/dosroot/autoexec.bat

rm -f $TMP1
if [ -f $GLOBAL_AUTOEXEC ]
then
	echo "autoexec.bat exists. Do charconv" >> $LOG_FILE
	# Do a DOS to UNIX conversion.
	$STATICMERGE/charconv -d -x -z $GLOBAL_AUTOEXEC $TMP1 >> $LOG_FILE 2>&1
	if [ ! -f $TMP1 ]
	then
		echo "WARNING:charconv failed." >> $LOG_FILE
		# Somehow charconv failed, so recover by just copying the file.
		cp $GLOBAL_AUTOEXEC $TMP1
	fi
else
	echo "autoexec.bat does not exist. Use empty file"  >> $LOG_FILE
	> $TMP1
	# If autoexec.bat exist in the root directory and $GLOBAL_AUTOEXEC
	# doesn't exists then move /autoexec.bat to the place where it should
	# be and make the ln for backward compatibility.
	if [ -f /autoexec.bat -a ! -f $GLOBAL_AUTOEXEC ]
	then
		echo "Move /autoexec.bat to $GLOBAL_AUTOEXEC" >> $LOG_FILE
		mv /autoexec.bat $GLOBAL_AUTOEXEC
		ln -s $GLOBAL_AUTOEXEC /autoexec.bat
	fi
fi

> $TMP2

# Add PATH and PROMPT (note: grep returns 0 if matches found)
grep -i '^path' $TMP1		>> $NULL 2>&1
RET1=$?
grep -i '^@path' $TMP1		>> $NULL 2>&1
RET2=$?
			# Hardwired drive letter j: and path.
[ $RET1 -ne 0 -a $RET2 -ne 0 ] && \
	EchoText 'path j:\\dos;j:\\merge;' >> $TMP2

grep -i '^prompt' $TMP1		>> $NULL 2>&1
RET1=$?
grep -i '^@prompt' $TMP1		>> $NULL 2>&1
RET2=$?
[ $RET1 -ne 0 -a $RET2 -ne 0 ] && echo '@prompt $p$g' >> $TMP2
cat $TMP1 >> $TMP2

rm -f $GLOBAL_AUTOEXEC
# Do a UNIX to DOS conversion.
$STATICMERGE/charconv -p -x $TMP2 $GLOBAL_AUTOEXEC >> $LOG_FILE 2>&1
if [ ! -s $GLOBAL_AUTOEXEC ]
then
	echo "WARNING:charconv failed." >> $LOG_FILE
	# Somehow charconv failed, so recover by just renaming the file.
	mv $TMP2 $GLOBAL_AUTOEXEC
fi
rm -f $TMP1 $TMP2

chmod 664 $GLOBAL_AUTOEXEC >> $NULL 2>&1
chgrp bin $GLOBAL_AUTOEXEC >> $NULL 2>&1
chown bin $GLOBAL_AUTOEXEC >> $NULL 2>&1

ls -l $GLOBAL_AUTOEXEC >> $LOG_FILE

#############################################################################
#	Compile the terminfo files.
#############################################################################

if [ "$SYSTEMTYPE" = "sco32" -o "$SYSTEMTYPE" = "sco" ]
then
	SAVEDIR=`pwd`
	cd $STATICMERGE/terminfo
	for i in kt70pcix.ti kt70ix-nm.ti wy60ix.ti wy60ix-nm.ti wy60pc.ti
	do
		tic -v $i >> $LOG_FILE 2>&1
	done
	cd $SAVEDIR
fi
	
#############################################################################
#	Integrate the help files
#############################################################################

if [ "$SYSTEMTYPE" = "uw7" ]
then
	#
	#	Integrate documentation.
	#
	_gcd="/usr/man/bin"
	usrMsg ECHO INST_DOCINTSTART "Integrating documentation...\n"
	$_gcd/config_help -n MergeAdminG xcrt winsetup >> $LOG_FILE 2>&1
	rc=$?
     	$_gcd/config_man -n 1merge >> $LOG_FILE 2>&1
	rc2=$?; [ $rc2 -gt $rc ] && rc=$rc2
     	$_gcd/config_views >> $LOG_FILE 2>&1
	rc2=$?; [ $rc2 -gt $rc ] && rc=$rc2
	# Note: Not using the "-f" option to config_search to force
	# the doc indexing to happen.  SCO says that should not be needed.
     	$_gcd/config_search >> $LOG_FILE 2>&1
	rc2=$?; [ $rc2 -gt $rc ] && rc=$rc2
	if [ $rc -ne 0 ]; then
		usrMsg LOG INST_DOCINTERR \
		  "R($rc); warning/error(s) detected during integration.\n" \
		  "$rc"
	fi
	unset _gcd
fi
#############################################################################
#	FONT SERVER -- Font Server -- font server
#############################################################################

if [ "$SYSTEMTYPE" = "uw7" ]
then
	if [ -x $STATICMERGE/bin/font_serv_setup.sh ]
	then
		export STATICMERGE VARMERGE
		$STATICMERGE/bin/font_serv_setup.sh
	fi
fi

#############################################################################
#	Create the admin files.
#############################################################################
New_admin_create()
{
	[ ! -d $VARMERGE/admin ] && mkdir $VARMERGE/admin
	cd $STATICMERGE/admin
	cp * $VARMERGE/admin
	cd /
}

echo "setup $VARMERGE/admin"  >> $LOG_FILE #######TESTING#
if [ "$SYSTEMTYPE" != "sco32" -a "$SYSTEMTYPE" != "sco" ]
then
	# The first non-SCOSVR3.2 version was Merge 4.1.0,
	# so there is no need to do an upgrade from anything
	# earilier than that.
	if [ ! -d $VARMERGE/admin ]
	then
		# First time install.  Just copy the files into place.
		echo "First time install admin" >> $LOG_FILE
		New_admin_create
	fi

elif [ ! -d $VARMERGE/admin ]
then
	# (SCO32 specific stuff only in this section.)
	# This is a fresh install of Merge, or an upgrade from an old version
	# before we started using winadmin.
	# An upgrade from an old version before we started using winadmin
	# is only possible on SCO SVR3.2 systems.

	# The admin directory doesn't exist.  Set it up.
	if [ -s /etc/dosdev ]
	then
		# Don't try to convert the dummy dosdev file.
		cmp -s $STATICMERGE/dosdev /etc/dosdev
		[ $? -eq 0 ] && rm -f /etc/dosdev
	fi
	if [ -s /etc/dosdev ]
	then
		# Convert the /etc/dosdev file to the new admin format.
		echo "/etc/dosdev already exists from previous version." \
			>> $LOG_FILE #TESTING#
		echo "Converting it to the new admin format." \
			>> $LOG_FILE #TESTING#
		$STATICMERGE/admconvert >> $LOG_FILE 2>&1

		if [ $? -ne 0 -a ! -s $VARMERGE/admin/tokens ]
		then
		    echo "Admconvert failed, using defaults." \
			>> $LOG_FILE #TESTING#
		    New_admin_create
		else
		    # Install some additional device definitions.
		    EXTRA_COM1=tty2a
		    EXTRA_COM2=tty1a
		    echo "Installing extra COM1 device." \
			>> $LOG_FILE #TESTING#
		    winadmin com1 list com1-$EXTRA_COM1 >> $LOG_FILE 2>&1
		    if [ $? -ne 0 ]
		    then
			winadmin com1 add \
com1-$EXTRA_COM1:$STATICMERGE/admin/com1-$EXTRA_COM1.cm1:/dev/$EXTRA_COM1 \
	>> $LOG_FILE 2>&1
		    fi

		    echo "Installing extra COM2 device." \
			>> $LOG_FILE #TESTING#
		    winadmin com2 list com2-$EXTRA_COM2 >> $LOG_FILE 2>&1
		    if [ $? -ne 0 ]
		    then
			winadmin com2 add \
com2-$EXTRA_COM2:$STATICMERGE/admin/com2-$EXTRA_COM2.cm2:/dev/$EXTRA_COM2 \
	>> $LOG_FILE 2>&1
		    fi
		fi
		
		[ ! -d $VARMERGE/backup ] && \
			mkdir $VARMERGE/backup >> $LOG_FILE 2>&1
		chmod 775 $VARMERGE/backup
		chown bin $VARMERGE/backup
		chgrp bin $VARMERGE/backup
		[ -f $VARMERGE/backup/dosdev ] && \
		  mv $VARMERGE/backup/dosdev $VARMERGE/backup/dosdev.$$
		mv /etc/dosdev $VARMERGE/backup/dosdev >> $LOG_FILE 2>&1

		# Copy a dummy dosdev to /etc
		cp $STATICMERGE/dosdev /etc/dosdev
		chmod 444 /etc/dosdev
		chown bin /etc/dosdev
		chgrp bin /etc/dosdev
	else
		# First time install.  Just copy the files into place.
		echo "First time install admin" >> $LOG_FILE
		New_admin_create
	fi

else
	# (SCO32 specific stuff only in this section.)
	# The directory $VARMERGE/admin exists from a previous release, so
	# add any tokens that have been added since we started using winadmin.
	# (Again, this is only possible on SCO systems.)

	# Add the homevdrv token for the standard home directory virtual drive.
	winadmin dosdrive list homevdrv >> $LOG_FILE 2>&1
	if [ $? -ne 0 ]
	then
	    winadmin dosdrive add \
		homevdrv:$STATICMERGE/admin/homevdrv.drv:'$HOME/vdrive.dsk'
			>> $LOG_FILE 2>&1
	fi
	LP0=`winadmin printer list lp0`
	if [ "x$LP0" = "xlp0:DOS Direct Port 1" ]
	then
		# Update the "full name" of the item to include the irq setting.
		TMPT=$TMPDIR/tokens.$$
		IRQ_P0=`winadmin printer printdef lp0 | grep irq= | cut -d= -f2`
		IRQ_P2=`winadmin printer printdef lp2 | grep irq= | cut -d= -f2`
		ORIG_0="lp0:lp0.prt:DOS Direct Port 1"
		ORIG_2="lp2:lp2.prt:DOS Direct Port 2"
		cat $VARMERGE/admin/tokens \
			| sed -e "s/$ORIG_0/$ORIG_0 - IRQ $IRQ_P0/g" \
			| sed -e "s/$ORIG_2/$ORIG_2 - IRQ $IRQ_P2/g" \
			> $TMPT
		cp $TMPT $VARMERGE/tokens
		rm -f $TMPT
	fi
fi
# In Merge 4.3, the "default" and "none" printer tokens were added.
# Add them if needed.
winadmin printer list default >> $LOG_FILE 2>&1
if [ $? -ne 0 ]
then
	winadmin printer add "default:$STATICMERGE/admin/default.prt:Default Printer" \
		>> $LOG_FILE 2>&1
fi
winadmin printer list none >> $LOG_FILE 2>&1
if [ $? -ne 0 ]
then
	winadmin printer add "none:$STATICMERGE/admin/none.prt:No Printer" \
		>> $LOG_FILE 2>&1
fi

chmod 775 $VARMERGE/admin
chown bin $VARMERGE/admin
chgrp bin $VARMERGE/admin
chmod 664 $VARMERGE/admin/*
chown bin $VARMERGE/admin/*
chgrp bin $VARMERGE/admin/*

#############################################################################
# Set up some special configuration files.
#############################################################################

# Install the system default configuration files.

# First make sure any previous versions are removed.
rm -f $VARMERGE/.merge/save_dos.cfd
rm -f $VARMERGE/.merge/save_snap.cfd
rm -f $VARMERGE/.merge/save_win.cfw

if [ ! -d $VARMERGE/.merge ]
then
	rm -f $VARMERGE/.merge
	mkdir $VARMERGE/.merge
fi
chmod 775 $VARMERGE/.merge
chown bin $VARMERGE/.merge
chgrp bin $VARMERGE/.merge

for i in dos.cfd snap.cfd win.cfw
do
	if [ -f $VARMERGE/.merge/$i ]
	then
		mv $VARMERGE/.merge/$i $VARMERGE/.merge/save_$i >> $NULL 2>&1
	fi
	cp $STATICMERGE/$i $VARMERGE/.merge/$i
	chmod 664 $VARMERGE/.merge/$i
	chown bin $VARMERGE/.merge/$i
	chgrp bin $VARMERGE/.merge/$i
done

#############################################################################
cp $STATICMERGE/final_remove.sh $VARMERGE/final_remove.sh
chmod 444 $VARMERGE/final_remove.sh

#############################################################################
#        Make sure that old image files are removed.
#############################################################################

IMAGEDIR=$VARMERGE/image/std
rm -f $IMAGEDIR/mono/mono.img
rm -f $IMAGEDIR/cga/cga.img
rm -f $IMAGEDIR/ega/ega.img $IMAGEDIR/ega/ega.rom
rm -f $IMAGEDIR/vga/vga.img $IMAGEDIR/vga/vga.rom

exit $EXIT_NORMAL
