:
# SCCSID(@(#)rem_mrg.sh	1.52) Modified 15:32:02 12/11/99
# Copyright (c) 1994-1999 TreLOS, Inc.
###############################################################################
#
#	Merge Removal Script.
#
#	Undo the changes that Merge "installation" did.
#
#	This script is called by the top level Merge Remove script.
#	With Custom+, the top level remove script is "ccs.sh".
#	With pkgadd, the top level remove script is "preremove".
#	(The source file to "preremove" is "remove_pa.sh")
#
###############################################################################

#############################################################################
#
#	This script *must* be called with STATICMERGE and VARMERGE set.
#
#############################################################################

EXIT_NORMAL=0		# Normal exit value
EXIT_FAIL=1		# Failure exit value

if [ "x$STATICMERGE" = "x" -o "x$VARMERGE" = "x" ]
then
	# This should never happen.
	echo STATICMERGE or VARMERGE not set.
	echo Cannot continue install.
	exit $EXIT_FAIL
fi
if [ ! -f $STATICMERGE/bin/s_init.sh ]
then
	# This should never happen.
	echo Missing $STATICMERGE/bin/s_init.sh
	echo Cannot continue install.
	exit $EXIT_FAIL
fi
. $STATICMERGE/bin/s_init.sh
# "s_init.sh" set up variables and functions used by this script.
# (such as NULL, LOG_FILE, usrMsg, and everything from the Version file.)

TDIR=$VARMERGE/tmp
if [ ! -d $TDIR ]
then
	mkdir -p  $TDIR
	chmod 775 $TDIR
	chown bin $TDIR
	chgrp bin $TDIR
fi

			# System-wide autoexec.bat location.
GLOBAL_AUTOEXEC=$VARMERGE/dosroot/autoexec.bat
			# System-wide config.sys location.
GLOBAL_CONFIGSYS=$VARMERGE/dosroot/config.sys

###############################################################################
#
#	The first part of this file is the function definitions.
#	The main part of the scripts starts at "Main".
#
###############################################################################


Remove_doslp()
{
	if [ ! -f $STATICMERGE/bin/instdoslp ]
	then
		# Nothing to do if doslp was not installed in the first place.
		return
	fi
	echo "Remove_doslp()" >> $LOG_FILE
	DTP=/usr/X/adm/dtprinter
	if [ "$UNIX_REL" != "3.2" -a -x $DTP ]
	then
	    # Remove the dosmodel-printer from the PrintMgr printer table.
	    # (Note: The program /usr/X/adm/dtprinter in SVR4.2p14.1 was linked
	    #  incorrectly, so must specify LD_LIBRARY_PATH to make it work.)
	    LD_LIBRARY_PATH=/usr/X/lib $DTP -d dosmodel-printer >>$LOG_FILE 2>&1
	fi
	LPADMIN="/usr/lib/lpadmin"
	if [ ! -x $LPADMIN ]
	then
		usrMsg TRACE MISSING_PROG "Missing $LPADMIN program.\n" $LPADMIN
		usrMsg TRACE REM_PRINTER_NOT \
			"Not removing the DOS printer interface.\n"
		return 0
	fi
	REJECT=`$STATICMERGE/findonpath reject`
	if [ $? -ne 0 ]
	then
		usrMsg TRACE MISSING_PROG "Missing reject program.\n" reject
		usrMsg TRACE REM_PRINTER_NOT \
			"Not removing the DOS printer interface.\n"
		return 0
	fi
	DISABLE=`$STATICMERGE/findonpath disable`
	if [ $? -ne 0 ]
	then
		usrMsg TRACE MISSING_PROG "Missing disable program.\n" disable
		usrMsg TRACE REM_PRINTER_NOT \
			"Not removing the DOS printer interface.\n"
		return 0
	fi
	usrMsg TRACE REM_PRINTER "Removing the DOS printer interface ...\n"

	reject doslp			>> $LOG_FILE 2>&1
	# On some systems, the disable "-c" option is not supported,
	# so try it first with the option, and then without it.
	disable -c doslp		>> $LOG_FILE 2>&1
	disable doslp			>> $LOG_FILE 2>&1
	$LPADMIN -xdoslp	>> $LOG_FILE 2>&1
	return 0
}

#############################################################################
#
#	Merge pre-remove function.
#
#############################################################################

PreRemove()
{
	echo "PreRemove()" >> $LOG_FILE

	#####################################################################
	#
	#	Remove files created by the Merge installation.
	#	Remove our changes to pre-existing files.
	#
	#####################################################################

	usrMsg ECHO REMOVE_3 "Removing $MERGENAME files.  Please wait...\n" \
		"$MERGENAME"

	#####################################################################

	rm -f	$VARMERGE/ss/inst_icon.sh \
		$VARMERGE/ss/inst_font.sh \
		$VARMERGE/ss/inst_misc.sh

	# Remove the system default configuration files.
	rm -f	$VARMERGE/.merge/dos.cfd  \
		$VARMERGE/.merge/snap.cfd \
		$VARMERGE/.merge/win.cfw

	rmdir	$VARMERGE/.merge >> $NULL 2>&1

	#####################################################################
	#
	#      Remove MS-DOS files and the image files.
	#
	#####################################################################

	echo >> $LOG_FILE
	usrMsg TRACE REM_DOS_FILES "Removing DOS files (from dosinstall) ...\n"

	$STATICMERGE/dosremove silent

	#####################################################################
	#
	#	Remove merge parts of LP system
	#
	#####################################################################

	echo >> $LOG_FILE
	Remove_doslp

	#####################################################################
	#
	# Remove files that did not come from the installation disks.
	#
	#####################################################################

	rm -f $VARMERGE/console.disp
	rm -f $VARMERGE/dosenv.def
	if [ "$SYSTEMTYPE" != "linux" ]
	then
		# SVR3's lp model directory is /usr/spool/lp/model
		# SVR4's lp model directory is /usr/lib/lp/model
		rm -f /usr/spool/lp/model/doslp
		rm -f /usr/lib/lp/model/doslp
	fi

	rm -f $VARMERGE/doing_mkimg $VARMERGE/auto_mkimg
	rm -f $VARMERGE/doing_dosinst $VARMERGE/auto_dosinst
	rm -f $VARMERGE/dospartitions
	rm -f $VARMERGE/Version.prev
	rm -f $VARMERGE/nodefaultdos

	rm -f /etc/dosdev
	#####################################################################
	# If the links.txt file exists, then remove the indicated links.
	#####################################################################
	if [ -f $STATICMERGE/links.txt ]
	then
		while read FROM TO
		do
			rm -f $TO
		done < $STATICMERGE/links.txt
	fi
	if [ "$SYSTEMTYPE" = "linux" ]
	then
		# Remove the symbolic links.
		for i in rc2.d rc3.d rc4.d rc5.d
		do
			rm -f /etc/rc.d/$i/S77Win4Lin
			rm -f /etc/$i/S77Win4Lin
		done
	fi

	#####################################################################
	#	Remove Merge mouse driver and ems driver from the
	#	global config.sys.  (grep returns 0 if matches found)
	#####################################################################
	T1=$TDIR/t1.$$
	T2=$TDIR/t2.$$

	echo >> $LOG_FILE
	usrMsg TRACE REM_CONFIG_SYS "Updating global config.sys ...\n"

	if [ -f $GLOBAL_CONFIGSYS ]
	then
		grep '\\merge\\lib\\mouse.sys' $GLOBAL_CONFIGSYS >> $NULL 2>&1
		if [ $? -eq 0 ]
		then
			# When found merge items, remove them.
			grep -v '\\merge\\lib\\mouse.sys' $GLOBAL_CONFIGSYS \
									> $TMPF
			cp $TMPF $GLOBAL_CONFIGSYS
			rm -f $TMPF
		fi
	
		grep '\\merge\\lib\\emm.sys' $GLOBAL_CONFIGSYS	>> $NULL 2>&1
		if [ $? -eq 0 ]
		then
			# When found merge items, remove them.
			grep -v '\\merge\\lib\\emm.sys' $GLOBAL_CONFIGSYS > $TMPF
			cp $TMPF $GLOBAL_CONFIGSYS
			rm -f $TMPF
		fi
	fi

	# If zero length, then remove it.
	[ ! -s $GLOBAL_CONFIGSYS ] && rm -f $GLOBAL_CONFIGSYS

	#####################################################################
	#	Remove Merge addition to $GLOBAL_AUTOEXEC
	#####################################################################

	echo >> $LOG_FILE
	usrMsg TRACE REM_AUTOEXEC "Updating global autoexec.bat ...\n"
	# Do a DOS to UNIX conversion.
	if [ -f $GLOBAL_AUTOEXEC ]
	then
		$STATICMERGE/charconv -d -x -z $GLOBAL_AUTOEXEC $T1 \
							>> $LOG_FILE 2>&1
		grep 'path .:\\dos;.:\\merge;$' $T1 >> $NULL 2>&1
		RET1=$?
		grep 'prompt $p$g$'	$T1 >> $NULL 2>&1
		RET2=$?
		if [ $RET1 -eq 0 -o $RET2 -eq 0 ]
		then	# Found something to remove
			grep -v 'path .:\\dos;.:\\merge;$'	$T1 > $T2
			grep -v 'prompt $p$g$'			$T2 > $T1
			if [ ! -s $T1 ]
			then
				# If zero length, then remove it.
				rm -f $GLOBAL_AUTOEXEC
			else
				# Do a UNIX to DOS conversion.
				$STATICMERGE/charconv -p -x \
					$T1 $GLOBAL_AUTOEXEC >> $LOG_FILE 2>&1
			fi
		fi
		rm -f $T1 $T2
	fi

	#####################################################################
	#
	#      Final cleanup
	#
	#####################################################################

	# Remove symbolic links created by inst_mrg.sh

	if [ -d $VARMERGE/dosroot ]
	then
		# First remove things from "J:" ($VARMERGE/dosroot)
		SAVEDIR=`pwd`
		cd $VARMERGE/dosroot
		rm -f merge
		rm -f mwin95
		rm -f mwindows/*
		rmdir mwindows		>> $NULL 2>&1
		cd $SAVEDIR
	fi

	echo "clean image directory config.sys" >> $LOG_FILE

	# Remove image config.sys files if NOT modified.
	for i in $VARMERGE/image/std/*/config.sys
	do
		[ ! -f $i ] && continue
		cmp -s $i $STATICMERGE/for_image/config.std	>> $NULL 2>&1
		if [ $? -eq 0 ]
		then
			# Identical, so remove.
			rm -f $i
		fi
		cmp -s $i $STATICMERGE/for_image/config.vga	>> $NULL 2>&1
		if [ $? -eq 0 ]
		then
			# Identical, so remove.
			rm -f $i
		fi
	done
	
	echo "clean image directory autoexec.bat" >> $LOG_FILE

	# Remove image autoexec.bat files if NOT modified.
	for i in $VARMERGE/image/std/*/autoexec.bat
	do
		[ ! -f $i ] && continue
		cmp -s $i $STATICMERGE/for_image/autoexec.std	>> $NULL 2>&1
		if [ $? -eq 0 ]
		then
			# Identical, so remove.
			rm -f $i
		fi
		cmp -s $i $STATICMERGE/for_image/autoexec.vga	>> $NULL 2>&1
		if [ $? -eq 0 ]
		then
			# Identical, so remove.
			rm -f $i
		fi
	done

	### Remove temp files that normally should have been
	### automatically removed by the process that created them.

	# Remove files created by /usr/bin/dos.
	rm -f $VARMERGE/dosroot/tmp/ae[0-9]*.bat
	rm -f $VARMERGE/tmp/save_msg.*

	# Remove temp files created by mkimg
	rm -rf $VARMERGE/dosroot/mkimgtmp

	# Remove temp directory created by dosinstall.
	rm -rf $VARMERGE/dosroot/dosi_tmp

	# Remove significantly old log files.
	[ -d $VARMERGE/log ] && \
		find $VARMERGE/log -atime +14 -type f -exec rm -f {} \;

	# Remove the C drive directory that snap.cfd specifies.
	rm -rf $VARMERGE/instC

	
	### The Merge specific directories that need removing when empty.
	DIRS="$VARMERGE/image/std/mono	$VARMERGE/image/std/cga \
		$VARMERGE/image/std/ega	$VARMERGE/image/std/vga \
		$VARMERGE/image/std	$VARMERGE/image \
		$VARMERGE/dosroot/dos	$VARMERGE/dosroot/tmp \
		$VARMERGE/dosroot/share \
		$VARMERGE/dosroot"

	for dir in $DIRS
	do
		# Note: the "-L" test option does not work on Solaris,
		# so use "-h" to test for symbolic links.
		if [ -h $dir ]
		then
			# Is a symbolic link, so remove it.
			rm -f $dir
		elif [ -d $dir ]
		then
			# Remove the directory if it is empty.
			rmdir $dir	>> $NULL 2>&1
		fi
	done

}


###############################################################################
# End of function definitions
###############################################################################

###############################################################################
#
# Start of Main part of script.
#
###############################################################################

TMPF=$VARMERGE/tmp/mrm$$

#############################################################################
#
#      Now go do it!
#
#############################################################################

### MUST be in ROOT for removal
cd $ROOT/

PreRemove

usrMsg LOG REM_DONE "Removal of $MERGENAME is complete.\n" "$MERGENAME"

exit $EXIT_NORMAL
