#!/bin/sh
# SCCSID(@(#)bootstrap.sh	1.6) Modified 02:04:27 12/3/99
# Create a minimal MS-DOS 7.x boot floppy from the CAB files on the Win9x CD.
# To bootstrap this we use a minimal "Free DOS" boot disk image so
# we can run Microsoft's "extract" command.
# There are three parameters:
#   1: The cab file directory.  This is also where we put the final "b.dsk".
#	This directory must be writable!
#   2: The so-called "first" cab file name.
#   3: The windows version: "95classic", "95osr2", or "98".

. /etc/mrgssv.sh

WINCAB="$1"
FIRST_CAB="$2"
WIN_VERSION="$3"
if [ "x$WINCAB" = "x" -o "x$FIRST_CAB" = "x" -o "x$WIN_VERSION" = "x" ]
then
	echo Bad parameters to $0
	echo $*
	exit 1
fi

DOSDISK=$STATICMERGE/dosdisk
DSKDIR=$STATICMERGE/diskimages

if [ ! -d $WINCAB ]
then
	echo Error, Directory $WINCAB does not exist.
	exit 1
fi
if [ ! -f $WINCAB/$FIRST_CAB ]
then
	echo Error, File $FIRST_CAB does not exist.
	exit 1
fi

if [ "$SYSTEMTYPE" = "linux" ]
then
	EOPT="-e"
else
	EOPT=
fi
ECHOE()
{
	echo $EOPT $*
}

# The DDOPT setting is the optiosn for the $DOSDISK command.
# The extra "-e" option can be used when debugging.
# DDOPT="-s -e"
DDOPT="-s"

# The FROMCD location is the temporary place where we put the extracted files.
FROMCD="$WINCAB/fromcd"
[ ! -d $FROMCD ] && mkdir $FROMCD
if [ ! -d $FROMCD ]
then
	echo "Cannot use $FROMCD"
	exit 1
fi
cd $FROMCD

rm -f command.com winboot.sys io.sys msdos.sys format.com sys.com b.dsk

# First get a copy of the "Free DOS" boot disk image.
# This is bootable and only has command.com and kernel.sys on it.

if [ -f $VARMERGE/bsboot.dsk ]
then
	cp $VARMERGE/bsboot.dsk t0.dsk
elif [ -f $DSKDIR/freeboot.dsk ]
then
	cp $DSKDIR/freeboot.dsk t0.dsk
else
	echo "Error: No boot floppy image availabe to boot from."
	exit 1
fi
chmod +w t0.dsk

# Put the quit and extract commands on the freedos boot disk.
echo "copyto quit.com $STATICMERGE/dosside/quit.com" | $DOSDISK $DDOPT t0.dsk
echo "copyto extract.exe $WINCAB/extract.exe"	     | $DOSDISK $DDOPT t0.dsk

Extract()
{
	FileName=$1
	CabFile=$3
	echo Extract $FileName
	if [ "$FileName" = "command.com" ]
	then
		Hack=command
	else
		Hack=not
	fi

	rm -f a.dsk b.dsk
	cp t0.dsk a.dsk
	cp $DSKDIR/f.dsk b.dsk
	chmod +w b.dsk
	# We are going to put the cab files on drive "B" and extract
	# the files to drive "A".

	rm -f a.bat
	if [ "$Hack" = "command" ]
	then
		# We are extracting command.com, move the boot one aside.
		echo 'ren command.com c.com'		| unix2dos >> a.bat
	fi
	# Extract the file
	echo a:extract /Y /L 'a:\' b:$CabFile $FileName	| unix2dos >> a.bat
	if [ "$Hack" = "command" ]
	then
		# We attempted to extract command.com, so now move it to a
		# special name.  This way when we pick it up we know it
		# is the extracted one.
		echo 'ren command.com newcom.com'	| unix2dos >> a.bat
		FileName=newcom.com
	fi
	echo 'a:quit'					| unix2dos >> a.bat

	echo "copyto autoexec.bat a.bat" | $DOSDISK $DDOPT a.dsk
	rm -f a.bat

	echo "copyto $CabFile $WINCAB/$CabFile" | $DOSDISK $DDOPT b.dsk

	OPS="-l +b -p -c"
	$DOSWINBINDIR/dos $OPS +aa:=`pwd`/a.dsk +ab:=`pwd`/b.dsk ver &
	DOS_ID=$!

	# After 15 seconds, kill the DOS process.
	TIMER_ID=`$STATICMERGE/bin/waitkill 15 $DOS_ID`

	# Now wait for the DOS process to finish or be killed.
	wait $DOS_ID
	
	kill $TIMER_ID 2> /dev/null

	echo "copyfrom $FileName $FileName" | $DOSDISK $DDOPT a.dsk
	if [ "$Hack" = "command" ]
	then
		[ -f newcom.com ] && mv newcom.com command.com
	fi

	rm -f a.dsk b.dsk
}

# Sometimes the vm process dies (due to bugs in freedos?).
# So if an extract fails, retry a few times.
echo Extracting files
if [ -f $WINCAB/PRECOPY1.CAB ]
then
	PRECAB=PRECOPY1.CAB
else
	PRECAB=precopy1.cab
fi
for i in 1 2 3 4 5 6 7 8 9
do
	Extract winboot.sys from $PRECAB
	[ -f winboot.sys ] && break
	ECHOE "Retry \c"
done
for i in 1 2 3 4 5 6 7 8 9
do
	Extract command.com from $PRECAB
	[ -f command.com ] && break
	ECHOE "Retry \c"
done
for i in 1 2 3 4 5 6 7 8 9
do
	Extract format.com from $FIRST_CAB
	[ -f format.com ] && break
	ECHOE "Retry \c"
done
for i in 1 2 3 4 5 6 7 8 9
do
	Extract sys.com from $FIRST_CAB
	[ -f sys.com ] && break
	ECHOE "Retry \c"
done

rm -f t0.dsk

for i in winboot.sys command.com format.com sys.com
do
	if [ ! -f $i ]
	then
		echo "Error:"
		echo "Failed to extract file $i from the Windows cab file."
		exit 1
	fi
done
echo ";FORMAT" | unix2dos >  msdos.sys
echo ""        | unix2dos >> msdos.sys

# Fix up any damaged winboot.sys file.
WINBOOTSUM=`sum -r winboot.sys | cut -f1 -d' '`
BS_DIR=$STATICMERGE/bs
case $WINBOOTSUM in
30498)
	# This is the checksum of the bad Classic version.
	$STATICMERGE/bin/xor winboot.sys $BS_DIR/iosys_classic.xor > io.sys
	;;
63483)
	# This is the checksum of the bad OSR2 version.
	$STATICMERGE/bin/xor winboot.sys $BS_DIR/iosys_osr2.xor > io.sys
	;;
00000)
	# TBD for Win98 -- Need to find out what the check sum is for this.
	# This is the checksum of the bad Win98 version.
	$STATICMERGE/bin/xor winboot.sys $BS_DIR/iosys_98.xor > io.sys
	;;
*)
	cat winboot.sys > io.sys
	;;
esac

TOTAL=`cat sys.com | $STATICMERGE/bin/scan_bin -c`
if [ "x$TOTAL" = "x" ]
then
	echo Error:
	echo "scan failure on sys.com"
	TOTAL=0
fi
if [ "$WIN_VERSION" = "95classic" ]
then
    END=`cat sys.com | $STATICMERGE/bin/scan_bin -c 0x59 0x53 0 0 0x55 0xaa`
elif [ "$WIN_VERSION" = "95osr2" ]
then
    END=`cat sys.com | $STATICMERGE/bin/scan_bin -c 0x40 0xff 0 0 0x55 0xaa`
elif [ "$WIN_VERSION" = "98" ]
then
	# TBD for Win98 -- boot track pattern in sys.com
    END=`cat sys.com | $STATICMERGE/bin/scan_bin -c 0 0 0x55 0xaa`
else
    echo Unknown Windows version $WIN_VERSIOIN
    exit 1
fi
if [ $TOTAL -le $END ]
then
	echo "Error:"
	echo "Cannot use the sys.com file."
	exit 1
fi
if [ $END -lt 512 ]
then
	echo "Error:"
	echo "Cannot use the sys.com command."
	exit 1
fi
START=`expr $END - 512`

# Now create a bootable MS-DOS floppy with the newly extracted DOS files.
rm -f a.dsk b.dsk
# Now extract the boot track from the sys.com file to put on the virtual disk.
# The first 512 (0x200) is our the virtual floppy header.
# The second 512 is the boot track for the virtual floppy that we get from
# the sys.com file.
# The rest is the FAT table (the filesystem format).
if [ "$WIN_VERSION" = "95classic" ]
then
	INPUTFILE="$DSKDIR/fbtc.dsk"
fi
if [ "$WIN_VERSION" = "95osr2" ]
then
	INPUTFILE="$DSKDIR/fbt2.dsk"
fi
if [ "$WIN_VERSION" = "98" ]
then
	INPUTFILE="$DSKDIR/fbt8.dsk"
fi
cat $INPUTFILE | $STATICMERGE/bin/scan_bin           -m 512 >  a.dsk
cat sys.com    | $STATICMERGE/bin/scan_bin -s $START -m 512 >> a.dsk
cat $INPUTFILE | $STATICMERGE/bin/scan_bin -s 1024          >> a.dsk

cp $INPUTFILE a.dsk

echo Copying files to the disk
echo "copyto io.sys      io.sys"      | $DOSDISK $DDOPT a.dsk
echo "copyto msdos.sys   msdos.sys"   | $DOSDISK $DDOPT a.dsk
echo "copyto command.com command.com" | $DOSDISK $DDOPT a.dsk
echo "copyto sys.com     sys.com"     | $DOSDISK $DDOPT a.dsk
echo "copyto quit.com $STATICMERGE/dosside/quit.com" | $DOSDISK $DDOPT a.dsk

cp $DSKDIR/f.dsk b.dsk
chmod +w b.dsk

rm -f a.bat
echo 'sys b:'			| unix2dos >> a.bat
echo 'a:quit'			| unix2dos >> a.bat
echo "copyto autoexec.bat a.bat" | $DOSDISK $DDOPT a.dsk
rm -f a.bat

# Now run the sys command to make the b disk bootable.
OPS="-l +b -p -c"
$DOSWINBINDIR/dos $OPS +aa:=`pwd`/a.dsk +ab:=`pwd`/b.dsk ver

cp b.dsk $WINCAB/b.dsk

cd $VARMERGE
rm -f $FROMCD/*
rmdir $FROMCD > /dev/null 2>&1

# All done!

# End
exit 0
