#!/bin/sh
# SCCSID(@(#)clean_boot.sh	1.1) Modified 16:28:50 8/8/99
# This a script that will produce a "clean" a boot floppy image that
# only has the minimum needed to bootstrap with.
# The input floppy image must be bootable and have a "sys" command available.
# No change is made to the input floppy image.

. /etc/mrgssv.sh

DOSDISK=$STATICMERGE/dosdisk
DSKDIR=$STATICMERGE/diskimages

ORIG_FLOP="$1"
BFLOP="$2"
if [ "$SYSTEMTYPE" = "linux" ]
then
	EOPT="-e"
else
	EOPT=
fi

Echo()
{
	echo $EOPT $*
}

if [ "x$ORIG_FLOP" = "x" -o "x$BFLOP" = "x" ]
then
	Echo "The filename of the boot floppy must be specified,"
	Echo "and a filename to write the cleaned version too."
	exit 1
fi
if [ ! -f "$ORIG_FLOP" ]
then
	Echo "The input filename $ORIG_FLOP is not valid."
	exit 2
fi
if [ ! -r "$ORIG_FLOP" ]
then
	Echo "The file $ORIG_FLOP is not readable."
	exit 2
fi
XXX=`echo dir | "$DOSDISK" -s "$ORIG_FLOP" | grep command.com`
if [ "x$XXX" = "x" ]
then
	Echo "The file $ORIG_FLOP does not appear to be a boot floppy."
	exit 3
fi

# At this point we probably have a bootable input file.
# Now make sure we can write properly to the output file.
cp "$ORIG_FLOP" "$BFLOP"
chmod u+rw "$BFLOP"
if [ ! -f "$BFLOP" ]
then
	Echo "Unable to write to the output file $BFLOP"
	exit 4
fi
if [ ! -w "$BFLOP" ]
then
	Echo "The file $BFLOP is read-only."
	Echo "It must be writable."
	exit 3
fi
# Make sure the copy actually worked
cmp -s "$ORIG_FLOP" "$BFLOP"
if [ $? -ne 0 ]
then
	Echo "Unable to write to the output file $BFLOP"
	exit 4
fi

# So now ready to make BFLOP into a bootable floppy.

# All the above script is just to validate that the floppy image file is
# a writeable floppy image and that is it probably bootable.
DDOPT="-s"
DDOPT=


TMPFILE=$HOME/cb.$$.tmp

rm -f "$TMPFILE"
echo 'sys b:'	| "$STATICMERGE/charconv" -p -x >> "$TMPFILE"
echo 'a:quit'	| "$STATICMERGE/charconv" -p -x >> "$TMPFILE"
echo rename autoexec.bat xxxxexec.bat		     | "$DOSDISK" -s "$BFLOP"
echo copyto autoexec.bat "$TMPFILE"                  | "$DOSDISK" -s "$BFLOP"
echo copyto quit.com "$STATICMERGE/dosside/quit.com" | "$DOSDISK" -s "$BFLOP"

cp "$DSKDIR/f.dsk" "$TMPFILE"
chmod u+rw "$TMPFILE"

# Now run the sys command to make the b disk bootable.
OPS="-l +b -p -c"
$DOSWINBINDIR/dos $OPS "+aa:=$BFLOP" "+ab:=$TMPFILE" ver
RETVAL=$?
cp "$TMPFILE" "$BFLOP"
rm -f "$TMPFILE"

exit $RETVAL
