:
# SCCSID(@(#)part_set.sh	1.38) Modified 20:41:53 12/10/99
########################################################################
#
#	Copyright (c) 1991-1999 TreLOS, Inc.
#	All rights reserved.
#	This is an unpublished work containing CONFIDENTIAL INFORMATION
#	that is the property of TreLOS, Inc.
#	Any unauthorized use, duplication or disclosure is prohibited.
#
########################################################################
#
# Initialize the DOS partition aliases in the Merge administration.
#
# Note: The MSDOS 5.0 doc says that the primary partitions of
# all the disks are assigned drive letters first, then followed
# by the drives in the extended partitions.
#
########################################################################

# This function converts the an integer number to a drive letter.
GET_DRIVE_LETTER()
{
	COUNT=1
	for i in c d e f g h i j k l m n o p q r s t u v w x y z
	do
		if [ $COUNT = $1 ]
		then
			D_LETTER=$i
			break
		fi
		COUNT=`expr $COUNT + 1`
	done
}

EE()
{
	if [ "x$SYSTEMTYPE" = "xlinux" ]
	then
		echo -e $*
	else
		echo $*
	fi
}

ASSIGN_DRIVE_LETTER()
{
	# This functon has two args.  The first is the device name,
	# and the second is the extended partition number.  When the second
	# arg is not specified, then the partition is a primary partition.
	Devname=$1
	Pnum=$2
	if [ -n "$Pnum" -a "$Pnum" = "0" ]
	then
		return
	fi
	GET_DRIVE_LETTER $DRIVE_NUMBER
	# Put the alias in the appropriate files.
	if [ -z "$Pnum" ]
	then
		EE "dos${D_LETTER}\t${Devname}\tNative DOS $D_LETTER:" \
			>> $T_FILE
	else
		EE "dos${D_LETTER}\t${Devname},${Pnum}\tNative DOS $D_LETTER:" \
			>> $T_FILE
	fi

	DRIVE_NUMBER=`expr $DRIVE_NUMBER + 1`
}

ASSIGN_EXTENDED()
{
	$STATICMERGE/dosdisk -t -d $1 > $TF 2>> $NULL
	[ $? -ne 0 ] && return
	# Found a DOS partition.
	for SUBPART in `cat $TF`
	do
		# Only use extended partition parts
		[ "x$SUBPART" = "x0" ] && continue
		ASSIGN_DRIVE_LETTER $1 $SUBPART
		[ $DRIVE_NUMBER = 25 ] && break
	done
}

# This is the function that loops through the Unix devices and
# find the DOS extended partitions, so it can create aliases for them.
SCO32_GET_EXTENDED()
{
	# Search the Unix devices, looking for DOS extended partitions.
	for MAINPART in $NUMBERLIST
	do
		[ $DRIVE_NUMBER = 25 ] && break
		DEV="/dev/dsk/$1s$MAINPART"
		[ ! -b $DEV ] && continue
		ASSIGN_EXTENDED $DEV
	done
	rm -f $TF
}

SCO32_DOS_PARTITIONS()
{
	# Scan the disk partitions to find all the DOS partitions,
	# and create aliases for them.
	> $T_FILE
	NUMBERLIST="1 2 3 4"
	DRIVE_NUMBER=1

	DISK_NUM=0
	# First do primary partitions.
	while [ 1 ]
	do
		if [ -b /dev/dsk/${DISK_NUM}s1 ]
		then
			for PART in $NUMBERLIST
			do
				DEV=/dev/dsk/${DISK_NUM}s${PART}
				[ ! -b $DEV ] && break
				$STATICMERGE/dosdisk -t -d $DEV > $TF 2>> $NULL
				[ $? -ne 0 ] && continue
				grep "0" $TF >> $NULL
				[ $? -ne 0 ] && continue
				# Found a primary DOS partition.
				ASSIGN_DRIVE_LETTER $DEV
			done
			DISK_NUM=`expr $DISK_NUM + 1`
		else
			break
		fi
	done
	rm -f $TF

	DISK_NUM=0
	# Now do extended partitions.
	while [ 1 ]
	do
		if [ -b /dev/dsk/${DISK_NUM}s1 ]
		then
			SCO32_GET_EXTENDED $DISK_NUM
			DISK_NUM=`expr $DISK_NUM + 1`
		else
			break
		fi
	done
}

UW_DOS_PARTITIONS()
{
	# Scan the disk partitions to find all the DOS partitions,
	# and create aliases for them.
	> $T_FILE
	DRIVE_NUMBER=1

	# The partitions are named using this form:
	#	/dev/dsk/c<CONTROLLER>b<BUS>t<TARGET>d<DISKNUMBER>p<PARTITION>
	# CONTROLLER numbers vary from 0 to 31
	#        BUS numbers vary from 0 to  7
	#     TARGET numbers vary from 0 to 31
	# DISKNUMBER numbers vary from 0 to 31
	#  PARTITION numbers vary from 1 to  4
	# For example, the four partitions on the first disk will be:
	#	/dev/dsk/c0b0t0d0p1
	#	/dev/dsk/c0b0t0d0p2
	#	/dev/dsk/c0b0t0d0p4
	#	/dev/dsk/c0b0t0d0p3
	# 
	# Make a list of the disks, minus the "p<PARTITION>" part.
	# This needs to be sorted properly, in the minor number order.
	# Note that "ls" will not do this unless all the numbers are single
	# digits, (which it probably will be on most systems).
	# For example ls will put "c0b0t0d10p1" before "c0b0t0d9p1".
	# For this first version, using the straight output of "ls" is ok,
	# but we soon need to write a program that sorts the list properly
	# by the device's minor number.
	ls /dev/dsk/c*b*t*d*p1 | sed -e "/p1\$/s///" > $X_FILE

	# First do primary partitions.
	for BASE in `cat $X_FILE`
	do
		for PART in p1 p2 p3 p4
		do
			DEV=${BASE}${PART}
			# echo Try $DEV
			[ ! -b $DEV ] && break
			$STATICMERGE/dosdisk -t -d $DEV > $TF 2>> $NULL
			[ $? -ne 0 ] && continue
			grep "0" $TF >> $NULL
			[ $? -ne 0 ] && continue
			# Found a primary DOS partition.
			ASSIGN_DRIVE_LETTER $DEV
		done
	done
	rm -f $TF

	# Now do extended partitions.
	for BASE in `cat $X_FILE`
	do
		# Search the Unix devices, looking for DOS extended partitions.
		for PART in p1 p2 p3 p4
		do
			[ $DRIVE_NUMBER = 25 ] && break
			DEV=${BASE}${PART}
			# echo Try $DEV
			[ ! -b $DEV ] && continue
			ASSIGN_EXTENDED $DEV
		done
	done
	rm -f $TF
}

LINUX_DOS_PARTITIONS()
{
	# Scan the disk partitions to find all the DOS partitions,
	# and create aliases for them.
	# The partitions are named using this form:
	#	/dev/hd<DISKID><PARTITION>
	#  DISKID numbers vary from a to h (?)
	#  PARTITION numbers vary from 1 to  16
	#		(For the entire disk no DISID number)
	#	1 thru 4 are the four real partitions
	#	5 thru 16 are the virtual partitions
	# For example,
	#	/dev/dsk/hda	-- Entire disk
	#	/dev/dsk/hda1	-- DOS partition
	#	/dev/dsk/hda2	-- Typically swap
	#	/dev/dsk/hda3	-- Typically mounted as /
	#	/dev/dsk/hda4	-- Extended DOS partition
	#	/dev/dsk/hda5	-- First DOS drive in extended DOS partition
	#	/dev/dsk/hda6	-- Second DOS drive in extended DOS partition
	# 
	> $T_FILE
	DISKLIST="a b c d e f g h"
	DRIVE_NUMBER=1

	# First do primary partitions.
	for DISKTYPE in hd sd
	do
		NUMBERLIST="1 2 3 4"
		for DISK_NUM in $DISKLIST
		do
			[ ! -b /dev/$DISKTYPE$DISK_NUM ] && break
			for PART in $NUMBERLIST
			do
				DN=$DISKTYPE$DISK_NUM$PART
				grep "$DN" /proc/partitions >> $NULL
				[ $? -ne 0 ] && continue
				DEV=/dev/$DN
				[ ! -b $DEV ] && break
				$STATICMERGE/dosdisk -t -d $DEV > $TF 2>> $NULL
				[ $? -ne 0 ] && continue
				grep "0" $TF >> $NULL
				[ $? -ne 0 ] && continue
				# Found a real primary DOS partition.
				ASSIGN_DRIVE_LETTER $DEV
			done
		done
	done

	# Now do the extended.
	for DISKTYPE in hd sd
	do
		NUMBERLIST="5 6 7 8 9 10 11 12 13 14 15 16"
		for DISK_NUM in $DISKLIST
		do
			[ ! -b /dev/$DISKTYPE$DISK_NUM ] && break
			for PART in $NUMBERLIST
			do
				DN=$DISKTYPE$DISK_NUM$PART
				grep "$DN" /proc/partitions >> $NULL
				[ $? -ne 0 ] && continue
				DEV=/dev/$DN
				[ ! -b $DEV ] && break
				$STATICMERGE/dosdisk -t -d $DEV > $TF 2>> $NULL
				[ $? -ne 0 ] && continue
				grep "0" $TF >> $NULL
				[ $? -ne 0 ] && continue
				# On Linux, the tool says this is a primary
				# because that is how Linux presents it to
				# us, but we know it is from the extended
				# partition because of the device node number.
				ASSIGN_DRIVE_LETTER $DEV
			done
		done
	done
	rm -f $TF
}

UPDATE_ADMIN()
{
	# See if new or different partitions.
	cmp -s $T_FILE $REAL_TFILE
	[ $? -eq 0 ] && return

	# There is a difference, so update the administration.

	if [ -f $REAL_TFILE ]
	then
		rm -f ${REAL_TFILE}.old
		mv $REAL_TFILE ${REAL_TFILE}.old
	fi
	if [ -f $T_FILE ]
	then
		mv $T_FILE $REAL_TFILE
		chmod 664  $REAL_TFILE
		chgrp bin  $REAL_TFILE
		chown bin  $REAL_TFILE
	fi
	
	# Run the script that checked the difference between
	# the current and old $REAL_TFILE and updates the
	# DOS drive definitions to match.
	$STATICMERGE/bin/update_dosdrive.sh

	return
}


########################################################################
# End of function definitions
#############################

########################################################################
# Start of Main part of script
##############################

NULL=/dev/null

# Get the STATICMERGE, VARMERGE settings.
. /etc/mrgssv.sh

PATH=/bin:$PATH; export PATH
TF=$VARMERGE/tmp.$$
# The script "set_nls.sh" uses STATICMERGE to set MFILE.
if [ ! -r $STATICMERGE/set_nls.sh ]
then
	# Required Merge file missing.
	exit 0
fi
DM="LCC.MERGE.UNIX.PART_SET"
. $STATICMERGE/set_nls.sh

T_FILE=$VARMERGE/t.$$
X_FILE=$VARMERGE/x.$$
EX_FILE=$VARMERGE/ext.$$
REAL_TFILE=$VARMERGE/dospartitions


# The Version file sets SYSTEMTYPE.
. $VARMERGE/Version

if [ "x$SYSTEMTYPE" = "xsco32" ]
then
	SCO32_DOS_PARTITIONS
elif [ "x$SYSTEMTYPE" = "xuw7" ]
then
	UW_DOS_PARTITIONS
elif [ "x$SYSTEMTYPE" = "xlinux" ]
then
	LINUX_DOS_PARTITIONS
fi
UPDATE_ADMIN
rm -f $T_FILE
rm -f $X_FILE
rm -f $EX_FILE
rm -f $TF

exit 0
