:
# SCCSID(@(#)unixdrives.sh	1.4) Modified 12:46:12 1/3/00
#****************************************************************************/
#
#	Copyright (c) 1997-1999 TreLOS, Inc.
#	All rights reserved.
#	This is an unpublished work containing CONFIDENTIAL INFORMATION
#	that is the property of TreLOS, Inc.
#	Any unauthorized use, duplication or disclosure is prohibited.
#
#****************************************************************************/
# Utility functions for dealing with configuration "unix drives".

# Two functions:
#	get_personal_drive_letter
#	get_drive_root.

# Function to return the personal drive letter from the given configuration.
# Returns it in the variable PERS_DRV_LETTER.
get_personal_drive_letter()
{
	PERS_DRV_LETTER=`winconfig $1 list personaldrive | cut -d= -f2`
}

# Function to return the full UNIX path to where a drive letter is rooted,
# as specified in the given configuration.
# $1 is the lowercase drive letter, and $2 is the configuration name.
# Returns it in the variable DRV_ROOT.
# The returned directory string is checked for the right form but the
# directory is not tested for existence.
# Return value 0 when successful.
# Return value 1 when the drive is not a UNIX drive.
# Return value 2 or 3 when drive is not properly specified in configuration:
#    2 when the variable specified is not set.
#    3 when the variable's value does not shart with "/".
get_drive_root()
{
	DRV_ROOT=
	G_D_R_DEF=`winconfig $2 list drive | \
		grep "drive=$1,unix," | cut -d= -f2 | cut -d, -f3`

	if [ "_$G_D_R_DEF" = "_" ]
	then
		# Is not a UNIX drive.
		return 1
	fi
	if [ "/" = `echo $G_D_R_DEF | cut -c1-1` ]
	then
		# It is a full path specificiation.
		DRV_ROOT=`echo $G_D_R_DEF | cut -d, -f1`
		return 0
	fi
	# Is a two part specification (slash delimited).
	# The first item is a variable name (to look up in
	# /etc/default/merge or failing that to get it from
	# the environment) that has a full path directory
	# specification.
	# The second part is the relative path from where
	# the variable indicates.
	G_D_R_ENV_PART=`echo $G_D_R_DEF | cut -d/ -f1`
	if [ "$G_D_R_ENV_PART" = "home" ]
	then
		G_D_R_ENV_PART=HOME
	fi
	if [ "$G_D_R_ENV_PART" = "HOME" -a "_$HOME" = "_/" ]
	then
		G_D_R_ENV_PART=MERGE_ROOT_HOME
	fi
	# First do the leading part.
	# Get the setting from /etc/default/merge if it is there.
	DRV_ROOT=`grep "^${G_D_R_ENV_PART}=" /etc/default/merge \
			| cut -d= -f2`
	if [ "_$DRV_ROOT" = "_" ]
	then
		# Was not in /etc/default/merge, so get from env.
		G_D_R_T1='echo $'`eval echo $G_D_R_ENV_PART`
		DRV_ROOT=`eval $G_D_R_T1`
	fi
	if [ -z "$DRV_ROOT" ]
	then
		# Error: No such variable.
		return 2
	fi
	if [ "/" != `echo $DRV_ROOT | cut -c1-1` ]
	then
		# Error: no leading slash, so cannot be a full path
		# specification.
		return 3
	fi
	# Now add the relative part.
	DRV_ROOT=$DRV_ROOT/`echo $G_D_R_DEF | cut -d/ -f2-`
	return 0
}
