#!/bin/sh
# SCCSID(@(#)update_dosdrive.sh	1.1) Modified 22:11:23 12/9/99
##############################################################################
# Do the automatic creation and deletion of DOS partition device definitions.
# This gets run each time the system is restarted and when the product
# package is installed.
# This is typically called from part_set.sh and uses the file that
# part_set.sh creates in $VARMERGE: dospartitions
##############################################################################

##############################################################################
# Functions
##############################

# Match the second field in lines from stdin with the parameter.
# If a match is found then the return value is zero.
# When no match, return non-zero.
Match_2()
{
	while read NTOK NDV NDSC
	do
		[ -z "$NDV" ] && break
		if [ "$1" = "$NDV" ]
		then
			return 0
		fi
	done
	return 1
}

# Check for items listed in the first file file that are not in the second file.
# The item to check is the second one in each line.
Find_Unique_2()
{
	F1="$1"
	F2="$2"
	[ ! -f $F1 ] && return
	cat $F1 | while read TOKEN DEV DESC
	do
		[ -z "$TOKEN" ] && break
		MATCH=nomatch
		if [ -f $F2 ]
		then
			cat $F2 | Match_2 $DEV
			[ $? -eq 0 ] && MATCH=match
		fi
		[ "$MATCH" = "nomatch" ] && echo $TOKEN $DEV $DESC
	done
} 

# Check for devices listed in the old file that are not
# in the new file, so we can delete the winadmin definition.
Remove_unused()
{
	Find_Unique_2 $CUR_DEFS $NEW_DP | while read TK DV DSC
	do
		# Remove the drive description from this partition.
		winadmin dosdrive delete $TK
	done
}

Make_unique_token()
{
	OT=$1
	# Check to see if the drive token exists.
	winadmin dosdrive list $OT >/dev/null 2>&1
	if [ $? -ne 0 ]
	then
		# The token does not already exist.
		echo $1
		return
	fi
	CNT=1
	while true
	do
		NT=${OT}${CNT}
		winadmin dosdrive list $NT >/dev/null 2>&1
		if [ $? -ne 0 ]
		then
			# The token does not already exist.
			echo $NT
			return
		fi
		CNT=`expr $CNT + 1`
	done
}

# Check for devices listed in the new file that are not
# in the old file, so we can add the winadmin definition.
Add_new()
{
	Find_Unique_2 $NEW_DP $CUR_DEFS | while read TK DV DSC
	do
		TF=$VARMERGE/tmp.$$
		# Add the drive description from this partition.
		# First make sure that the token is available.
		Tkn=`Make_unique_token $TK`
		if [ "$Tkn" != "$TK" ]
		then
			cp $NEW_DP $TF
			cat $TF | sed -e "s/^$TK/$Tkn/" > $NEW_DP
		fi
		rm -f $TF
		echo attachtype=drive   >  $TF
		echo drivename=$DV      >> $TF
		echo failureaction=warn >> $TF
		if [ -d $DV ]
		then
			PERM=`ls -l $DV | cut -c2-9`
		else
			PERM="not_a_device"
		fi
		if [ "$PERM" != "rw-rw-rw" ]
		then
			# The device is not generally read and writeable,
			# so set the access only for root.
			echo useraccess=root    >> $TF
		fi
	
		# Install the new definition
		winadmin dosdrive add "$Tkn:$TF:$DSC"
	
		rm -f $TF
	done
} 

Make_Cur_Defs_List()
{
	rm -f $CUR_DEFS
	winadmin dosdrive list | cut -f1 -d: | while read TOK
	do
		DEV=`winadmin dosdrive printdef $TOK | grep "^drivename=" \
			| sed -e "s/^drivename=//"`
		[ -z "$DEV" ] && continue
		DSC=`winadmin dosdrive list $TOK | cut -f2- -d:`
		echo "$TOK $DEV $DSC" >> $CUR_DEFS
	done
}

##############################################################################
# Start of Main part of script
##############################

NULL=/dev/null

# Get the STATICMERGE, VARMERGE settings.
. /etc/mrgssv.sh

PATH=/bin:$PATH; export PATH
# The script "set_nls.sh" uses STATICMERGE to set MFILE.
if [ ! -r $STATICMERGE/set_nls.sh ]
then
	# Required Merge file missing.
	exit 0
fi
DM="LCC.MERGE.UNIX.PART_SET"
. $STATICMERGE/set_nls.sh

NEW_DP=$VARMERGE/dospartitions
CUR_DEFS=$VARMERGE/dospartitions.now

Make_Cur_Defs_List

# Now all is set up, so call the functions to removed definitions
# that don't match existing devices, and then add needed new ones.

Remove_unused
Add_new

rm -f $CUR_DEFS
exit 0
