#!/bin/sh
# SCCSID(@(#)final_remove.sh	1.1) Modified 00:16:49 8/8/99
#
# --------------------- TOTAL REMOVAL SCRIPT ------------------------------

# After the packaged product is removed, configuration data and installed
# DOS and Windows files are left on the system so that when a newer version
# is installed, users retain all their settings, the installations of
# DOS and Windows, and all their applications and data.
#
# For the situation where after the removed there is no intent to
# ever reinstall, then it is ok to remove all the configration files
# as well as the DOS and Windows installations.

# -----------------------------------------------------------------------
# Each user will have to remove all their own files.  This is how:
#
# Each user's "Drive" directories are typically subdirectories in their
# home directories.  A typical location for the "C" drive is "$HOME/win".
# As long as no files are under that directory that the user wants
# to keep, then such directories can be removed.
# 
# Each user's configuration files are kept in the directory "$HOME/.merge".
# This directory can be removed if the user does not want to keep their
# personal windows and dos configuration settings.
# 

# -----------------------------------------------------------------------
# To fully remove all the shared DOS and Windows files and configurations
# then run this script.  It will not ask you if you really want to
# delete it all, it will just do it.  Of course you have to be logged
# in as root to do this.  There are only a few directories to remove
# so you could just remove those by hand after making sure this
# is really what you want to do.
#
# One other reason you might want to remove this stuff is if you
# want to reinstall the package, but you want to simulte a fresh
# install.

STATICMERGE=
VARMERGE=

# The script /etc/mrgssv uses the settings in /etc/default/merge
# to determine where all the interesting directories are.
if [ -f /etc/mrgssv.sh ]
then
	. /etc/mrgssv.sh
fi


if [ -z "$STATICMERGE" ]
then
	# Either /etc/mrgssv.sh was not there or could not for some
	# reason set STATICMERGE, so guess at the location.
	STATICMERGE=/opt/win4lin
	[ ! -d $STATICMERGE ] && STATICMERGE=/usr/lib/merge
	[ ! -d $STATICMERGE ] && STATICMERGE=
fi
if [ -z "$VARMERGE" ]
then
	# Either /etc/mrgssv.sh was not there or could not for some
	# reason set VARMERGE, so guess at the location.
	VARMERGE=/var/win4lin
	[ ! -d $VARMERGE ] && VARMERGE=/usr/merge
	[ ! -d $VARMERGE ] && VARMERGE=
fi

# The STATICMERGE location should have been removed when the package
# was removed, so do this just to make sure it all got removed as intended.
if [ -n "$STATICMERGE" ]
then
	echo "Removing $STATICMERGE"
	rm -rf $STATICMERGE
fi

# The VARMERGE location is where all the configuration files are
# stored as well as the installed DOS files and the Windows cab files.
if [ -n "$VARMERGE" ]
then
	echo "Removing $VARMERGE"
	rm -rf $VARMERGE
fi

# Now that the directories are removed, remove the last two files.

echo "Removing /etc/mrgssv.sh and /etc/default/merge"
rm -f /etc/mrgssv.sh
rm -f /etc/default/merge

echo "All Done."
