#!/bin/sh
# SCCSID(@(#)i_loc_linux.sh	1.1) Modified 19:13:00 6/29/99
########################################################################
#
#	Copyright (c) 1999 TreLOS, Inc.
#
########################################################################
#	Script to initialize the locale info in /etc/default/merge
#	based on UNIX system settings.  This script is only to be
#	called during installation.
#	Note: If the locale info is already set in /etc/default/merge,
#	then this script will not change the settings.
#
#	NOTE: The file localedata in the unix/janus directory contains
#	information about the locale.  Someday, we may want to change
#	this file to get the locale information from localedata so that
#	the locale information is encoded in only one place.
#
#	Linux version
#
########################################################################


# Get the VARMERGE and STATICMERGE settings
. /etc/mrgssv.sh

[ "x$NULL" = "x" ] && NULL=/dev/null
if [ "x$LOG_FILE" = "x" ]
then
	if [ -d $VARMERGE/log ]
	then
		LOG_FILE=$VARMERGE/log/NlsInit.log
		rm -f  $LOG_FILE
	else
		LOG_FILE=$NULL
	fi
fi
export NULL LOG_FILE
date >> $LOG_FILE

Set_DOS_locale()
{
	# Run the script that actually makes the change.  Use the
	# "AddOnly" option, so any previous settings will not be changed.
	if [ "$REPLACE" = "no" ]
	then
	    sh $STATICMERGE/setdosloc.sh $1 AddOnly
	else
	    sh $STATICMERGE/setdosloc.sh $1 
	fi
}

# On Linux, how do we find out which language-keyboard is
# being used on the console?
# For now, just set up the default by setting KEYBOARD=notspecified.

KEYBOARD=notspecified

echo "Localize /etc/default/merge"	>> $LOG_FILE
echo "KEYBOARD=$KEYBOARD"		>> $LOG_FILE

if [ "X$1" = "Xreplace" ]
then
    REPLACE=yes
    shift
else
    REPLACE=no
fi

# Call Set_DOS_locale, with a parameter that is based on
# the DOS country code for the given country.
case $KEYBOARD in
BE)	# Belgian
	Set_DOS_locale 32
	;;
CA)	# Canadian (IBM 445)
	# Don't know what DOS setup this corresponds to
	# so pick Canadian-French.
	Set_DOS_locale 2
	;;
DK)	# Danish
	Set_DOS_locale 45
	;;
NL)	# Dutch
	Set_DOS_locale 31
	;;
SE)	# Finnish
	Set_DOS_locale 358
	;;
FR)	# French
	Set_DOS_locale 33
	;;
fr_CA)	# French-Canadian
	Set_DOS_locale 2
	;;
DE)	# German
	Set_DOS_locale 49
	;;
IS)	# Icelandic
	# Don't know what DOS country this corresponds to.
	# So use International English (61).
	Set_DOS_locale 61
	;;
IT)	# Italian
	Set_DOS_locale 39
	;;
IT2)	# Italian (IBM 142)
	Set_DOS_locale 39b
	;;
A01)	# Japanese (A01)
	Set_DOS_locale 81a
	;;
AX)	# Japanese (AX)
	Set_DOS_locale 81x
	;;
es)	# Latin-American, Mexican
	Set_DOS_locale 3
	;;
NO)	# Norwegian
	Set_DOS_locale 47
	;;
PT)	# Portuguese
	Set_DOS_locale 351
	;;
ES)	# Spanish
	Set_DOS_locale 34
	;;
SE)	# Swedish
	Set_DOS_locale 46
	;;
fr_CH)	# Swiss-French
	Set_DOS_locale 41f
	;;
de_CH)	# Swiss-German
	Set_DOS_locale 41g
	;;
GB)	# UK English
	Set_DOS_locale 44
	;;
US)	# US (Latin-1 or ASCII)
	Set_DOS_locale 1
	;;
*)
	# Default to the US keyboard type
	Set_DOS_locale 1
	;;
esac
exit 0
