:
# SCCSID(@(#)mrg_setup.sh	1.13) Modified 11:39:43 12/10/99
########################################################################
#
#	Copyright (c) 1997-1999 TreLOS, Inc.
#	All rights reserved.
#	This is an unpublished work containing CONFIDENTIAL INFORMATION
#	that is the property of TreLOS, Inc.
#	Any unauthorized use, duplication or disclosure is prohibited.
#
########################################################################

# This script does setup of the system that is needed after the system
# is rebooted.
# (This script is also run at the end of Merge installation on systems
# with loadable drivers.)

if [ -z "$SYSTEMTYPE" ]
then
	# This should never happen!
	echo $0 ERROR: SYSTEMTYPE not set
	exit 1
fi
if [ -z "$STATICMERGE" ]
then
	# This should never happen!
	echo $0 ERROR: STATICMERGE not set
	exit 1
fi
if [ -z "$VARMERGE" ]
then
	# This should never happen!
	echo $0 ERROR: VARMERGE not set
	exit 1
fi

NULL=${NULL:-/dev/null}
export NULL
TMPDIR=$VARMERGE/tmp

########################################################################
# Function definitions
######################

SET_RLOCK_TABLES()
{
	# Check to see if we need to set any rlock table sizes

	PARAM=""

	grep "^MERGE_RLOCK_FILETABLE_SIZE" /etc/default/merge >$TMPDIR/rlock$$
	if [ $? -eq 0 ]
	then
		. $TMPDIR/rlock$$
		PARAM="filetable=$MERGE_RLOCK_FILETABLE_SIZE"
	fi

	grep "^MERGE_RLOCK_OPENTABLE_SIZE" /etc/default/merge >$TMPDIR/rlock$$
	if [ $? -eq 0 ]
	then
		. $TMPDIR/rlock$$
		PARAM="$PARAM opentable=$MERGE_RLOCK_OPENTABLE_SIZE"
	fi

	grep "^MERGE_RLOCK_LOCKTABLE_SIZE" /etc/default/merge >$TMPDIR/rlock$$
	if [ $? -eq 0 ]
	then
		. $TMPDIR/rlock$$
		PARAM="$PARAM locktable=$MERGE_RLOCK_LOCKTABLE_SIZE"
	fi

	rm -f $TMPDIR/rlock$$
	if [ "$PARAM" != "" ]
	then
		# First remove the shared memory segment and then set the
		# parameters.  This is necessary because (1) PCI may have
		# already created the segment and (2) this shell script
		# (setup_mrg.sh) may be being run without having rebooted
		# the system on platforms that support loadable drivers.
		# In such a scenario, there may already be a shared memory
		# segment with old parameters hanging around from a
		# previous version of merge.
		$STATICMERGE/rlockshm -r >/dev/null 2>&1
		$STATICMERGE/rlockshm $PARAM
	fi
}

GET_DISPLAY()
{
	for i in $STATICMERGE/ss/*
	do
		[ -x $i ] && $i >> $NULL 2>&1
	done
	for i in $VARMERGE/ss/*
	do
		[ -x $i ] && $i >> $NULL 2>&1
	done
	[ ! -x $STATICMERGE/display ] && return
	if [ -c /dev/console ]
	then
		CON_DEVICE=/dev/console
	elif [ -c /dev/con ]
	then
		CON_DEVICE=/dev/con
	else
		return
	fi
	CON_FILE=$VARMERGE/console.disp
	$STATICMERGE/display $CON_DEVICE > $CON_FILE 2>>$NULL
	chmod 644 $CON_FILE
	return
}

DOS_PARTITIONS()
{
	# Scan the disk partitions to find all the DOS partitions, and
	# create winadmin tokens for them, and make them writeable
	# by everyone.  The "part_set.sh" script does all of this.

	$STATICMERGE/bin/part_set.sh >$NULL 2>&1

	return
}

# Server attach floppy configure.
# The MERGE_FD_CHANGE_LINE setting is used to disable the automatic
# use of the hardware floppy change line.
# To disable the use, we set it to "off".
# If it is not set or set to anything else (e.g. "on") then Merge makes
# use of it if possbile.
# On Unixware we only want to try to use the hardware floppy change line
# if the UNIX driver has the support enabled.
# So far this is only for Unixware systems because we only know
# how to tell if the Unixware floppy driver has this enabled or not.
FD_CHANGE_LINE_CONFIGURE()
{
	# NOTE: The "idtune -g" option DOES NOT exist on non-Unixware systems,
	# and trying to us it on SCO SVR3.2 screws up the mtune file!
	# So this function should only be called on UW7 systems!
	Flag=x
	Fd_Door_Sense=`/etc/conf/bin/idtune -g FD_DOOR_SENSE | cut -c1`
	grep "^MERGE_FD_CHANGE_LINE=off" /etc/default/merge >> $NULL 2>&1
	if [ $? -eq 0 ]
	then
		# Merge is configured to say "No change line support"
		if [ "$Fd_Door_Sense" = "1" ]
		then
			# Diskette change IS supported by the floppy driver,
			# So need to reconfigure Merge.
			Flag=change
		fi
	else
		# Merge is configured to say "Use hardware change line."
		if [ "$Fd_Door_Sense" != "1" ]
		then
			# Diskette change is NOT supported by the floppy driver,
			# So need to reconfigure Merge.
			Flag=change
		fi
	fi
	if [ "$Flag" = "change" ]
	then
		grep -v "^MERGE_FD_CHANGE_LINE=" /etc/default/merge > $TMPDIR/fc$$
		cp $TMPDIR/fc$$ /etc/default/merge
		rm -f $TMPDIR/fc$$
		if [ "$Fd_Door_Sense" = "1" ]
		then
			echo "MERGE_FD_CHANGE_LINE=on" >> /etc/default/merge
		else
			echo "MERGE_FD_CHANGE_LINE=off" >> /etc/default/merge
		fi
	fi
}

# When doing direct attach floppy, do the special setup that this requires.
# (Direct attach floppy is only available on SCO, and is disabled by default.
# So only run the "setup_flop.sh" when the feature is enabled
# in /etc/default/merge.)
SCO_DA_FLOPPY_CONFIGURE()
{
	[ ! -x $STATICMERGE/setup_flop.sh ] && return
	grep "^MERGE_DIRECT_FLOPPY_ENABLE" /etc/default/merge >$TMPDIR/ff$$
	if [ $? -ne 0 ]
	then
		rm -f $TMPDIR/ff$$
		return
	fi
	. $TMPDIR/ff$$
	rm -f $TMPDIR/ff$$
	if [ "z$MERGE_DIRECT_FLOPPY_ENABLE" = "zon" ]
	then
		$STATICMERGE/setup_flop.sh
	fi
	return
}


MOUNT_MERGE_MEMFS()
{
	MEMFS_MOUNT_POINT=$VARMERGE/memfs
	if [ ! -d $MEMFS_MOUNT_POINT ] ; then
		mkdir $MEMFS_MOUNT_POINT
	fi
	chmod 1777 $MEMFS_MOUNT_POINT
	chgrp bin  $MEMFS_MOUNT_POINT
	chown bin  $MEMFS_MOUNT_POINT

	if [ "$SYSTEMTYPE" = "uw7" -o "$SYSTEMTYPE" = "gemini" ]
	then
		# If not alreay mounted, mount memfs.
		#
		mount | grep -q memfs
		if [ $? -ne 0 ]
		then
			mount -F memfs -o swapmax=2137418112,rootmode=1777 \
					$MEMFS_MOUNT_POINT $MEMFS_MOUNT_POINT
		fi
	fi
}

LOAD_DRIVERS()
{
	# Load the Linux drivers.
	[ -x $STATICMERGE/drivers/tools/loadem ] && \
	     $STATICMERGE/drivers/tools/loadem
}

########################################################################
# End of function definitions
#############################

########################################################################
# Start of Main part of script
##############################

case "$SYSTEMTYPE" in

	linux)
		SET_RLOCK_TABLES
		GET_DISPLAY
		DOS_PARTITIONS
		MOUNT_MERGE_MEMFS
		LOAD_DRIVERS
		;;
	sco32|sco)
		SET_RLOCK_TABLES
		GET_DISPLAY
		DOS_PARTITIONS
		SCO_DA_FLOPPY_CONFIGURE
		;;
	uw7|gemini)
		SET_RLOCK_TABLES
		GET_DISPLAY
		DOS_PARTITIONS
		FD_CHANGE_LINE_CONFIGURE
		MOUNT_MERGE_MEMFS
		;;
	*)
		# This should never happen
		echo $0 ERROR: unknown SYSTEMTYPE $SYSTEMTYPE
		exit 1
		;;
esac

exit 0
