#!/bin/sh
# SCCSID(@(#)postinst_rpm.sh	1.9) Modified 23:19:24 1/12/00
### Copyright (c) 1999 TreLOS, Inc.
#############################################################################
#
#       RPM "postinstall" script
#
#############################################################################
#
# NOTE:	There are no useful environment variables available.
# At this point only the read-only files have been installed.
# Most importantly /etc/default/merge might not exist.  It is
# created or updated by inst_mrg.sh.  We determine STATICMERGE
# and VARMERGE and inst_mrg.sh puts that into /etc/default/merge.

# This is where STATICMBIN, DOSWINBINDIR, STATICMERGE and VARMERGE are
# defined for Linux systems.

STATICMBIN=/usr/bin
DOSWINBINDIR=/bin
export STATICMBIN DOSWINBINDIR

# Note: DOSWINBINDIR is where "dos" and "win" are to be used from.
# This might be different from STATICMBIN on some systems which is
# where all the other public Merge executables are located.
# (On Linux systems, "dosemu" can be installed as "usr/bin/dos",
# so we have to avoid putting it there, and internally we cannot
# depend on the PATH being set correctly so that our "dos" is
# found first!)

STATICMERGE=/opt/win4lin
VARMERGE=/var/win4lin
export STATICMERGE VARMERGE

PATH=/bin:/usr/bin:$ROOT/usr/bin:$PATH; export PATH

if [ ! -f $STATICMERGE/Version ]
then
	echo "ERROR: Cannot find Version file\n       Cannot install Merge\n"
	exit 1
fi
# The Version file sets: MERGENAME= INSTALLTYPE= SYSTEMTYPE=
. $STATICMERGE/Version

if [ ! -d $VARMERGE ]
then
	mkdir -p $VARMERGE > /dev/null 2> /dev/null
fi
if [ -d $VARMERGE ]
then
	chown bin $VARMERGE > /dev/null 2> /dev/null
	chgrp bin $VARMERGE > /dev/null 2> /dev/null
	chmod 775 $VARMERGE > /dev/null 2> /dev/null
fi
# For compatibility with programs and scripts that still have
# /usr/lib/merge and /usr/merge hardwired:
rm -rf /usr/lib/merge /usr/merge
ln -s $STATICMERGE /usr/lib/merge
ln -s $VARMERGE /usr/merge

# See if there is a developer's debug variables file.
if [ -f $VARMERGE/dbg_vars.sh ]
then
	. $VARMERGE/dbg_vars.sh
fi

LOGDIR=$VARMERGE/log
[ ! -d $LOGDIR ] && mkdir -p $LOGDIR
chmod 775 $LOGDIR
chown bin $LOGDIR
chgrp bin $LOGDIR

TMPDIR=$VARMERGE/tmp
[ -d $TMPDIR ] && rm -rf $TMPDIR
mkdir -p $TMPDIR
chmod 775 $TMPDIR
chown bin $TMPDIR
chgrp bin $TMPDIR

if [ -z "$MERGETRACE" ]
then
	NULL=/dev/null
else
	NULL=$LOGDIR/inst.nul
	rm -rf $NULL
fi

LOG_FILE=$LOGDIR/inst.log
rm -rf $LOG_FILE
date > $LOG_FILE

#------------------------------------------
# The script "set_nls.sh" uses STATICMERGE to set MFILE.
. $STATICMERGE/set_nls.sh

# DM is the domain for messages for this script.
DM="LCC.MERGE.UNIX.INSTALL"

export LOGDIR LOGTYPE NULL LOG_FILE
export MFILE DM SYSTEMTYPE

# Set up the signals, just in case the user hits a button ...

interruptProc()
{
	trap 1 2 3 15
	usrMsg LOG INST_CANCEL "Installation cancelled by the user.\n"
	exit 2
}
trap interruptProc 1 2 3 15

# Create an error log, with a note to the user, just in case ...

usrMsg LOG INST_LOG_HDR \
"# This is a log file created by the $MERGENAME installation process.\n\
# If there is an error during the $MERGENAME controlled portion of\n\
# installation, it will show up here.  This file is created during\n\
# installation, and may be deleted by the user.  This file may also\n\
# contain contain non-error output from other (standard) UNIX programs.\n" \
"$MERGENAME" "$MERGENAME"

echo "SYSTEMTYPE=$SYSTEMTYPE" >> $LOG_FILE
echo "UNIX_REL=$UNIX_REL" >> $LOG_FILE

# Print out the Merge package name, Version number, and Copyright.
# (The package name and copyright are NOT nls-ized.)
if [ -z "$MERGETRACE" -a -x /usr/bin/tput ]
then
	/usr/bin/tput clear 2>> $NULL
fi

AGREE=no
if [ -f "$VARMERGE/agree.txt" ]
then
	grep "^The user agreed to the End User License Agreement\$" \
		"$VARMERGE/agree.txt" > /dev/null 2>&1
	if [ $? -eq 0 ]
	then
		AGREE=yes
	fi
fi
if [ "$AGREE" != "yes" -a ! -z "$DISPLAY" ]
then
	echo "End User License Agreement verification on DISPLAY=$DISPLAY"
	xterm -title "License Agreement" -sb -geometry 80x30 -fn fixed \
			-leftbar -e $STATICMERGE/bin/ask_eula.sh
	if [ $? -ne 0 ]
	then
		echo
		echo "Could not use xterm.  DISPLAY is set to '$DISPLAY'"
		echo "but that X display apparently cannot be used."
		echo "Perhaps you need to run 'xhost +' so that $DISPLAY"
		echo "can be used.  Or perhaps you should try logging"
		echo "in as root before installing this package."
		echo
		echo "Package installation NOT completed."
		echo "You must remove this Win4Lin package before attempting"
		echo "to reinstall it."
		echo "(If you installed this using 'rpm', then the command"
		echo "'rpm -e Win4Lin' removes the package.)"
		echo
		exit 1
	fi
	if [ -f "$VARMERGE/agree.txt" ]
	then
		grep "^The user agreed to the End User License Agreement\$" \
			"$VARMERGE/agree.txt" > /dev/null 2>&1
				> /dev/null 2>&1
		if [ $? -eq 0 ]
		then
			AGREE=yes
		fi
	fi
	if [ "$AGREE" != "yes" ]
	then
		echo "Package installation not completed."
		exit 1
	fi
fi

# This section is exempt from NLSizing
echo							| tee -a $LOG_FILE
echo "****************************************************************" \
							| tee -a $LOG_FILE
echo "$FULLTITLE"					| tee -a $LOG_FILE
[ -n "$REL_COMMENT" ] && echo				| tee -a $LOG_FILE
[ -n "$REL_COMMENT" ] && echo "$REL_COMMENT"		| tee -a $LOG_FILE
echo							| tee -a $LOG_FILE
echo "$MERGECOPYRIGHT"					| tee -a $LOG_FILE
echo "All Rights Reserved."				| tee -a $LOG_FILE
echo "****************************************************************" \
							| tee -a $LOG_FILE
echo							| tee -a $LOG_FILE

usrMsg ECHO INSTALL_1 "Installing $MERGENAME files.  Please wait...\n" \
	"$MERGENAME"

#######################################################################
# Install Merge files.
sh $STATICMERGE/Install/inst_mrg.sh
RETVAL=$?

if [ $RETVAL -ne 0 ] 
then
	usrMsg WARN INST_MRG_SH "Warning: Problems installing Merge files.\n"
fi

sh $STATICMERGE/Install/inst_xmerge.sh
RETVAL=$?

if [ $RETVAL -ne 0 ] 
then
	usrMsg WARN INST_X_SH "Problems installing Merge X files.\n"
fi

Create_Default_Set()
{
	# All we have to do is to create the default.set link.
	# When there is already a default.set file AND it is 
	# for DOS 7, then we do nothing and just return.
	# Otherwise we create it as a link to a ".set" file for
	# a preinstalled DOS, if any.

	VAR_DOSI=$VARMERGE/dosi
	STAT_DOSI=$STATICMERGE/dosi

	if [ -h $VAR_DOSI/default.set -o -f $VAR_DOSI/default.set ]
	then
		grep "DOS 7" $VAR_DOSI/default.set > /dev/null 2>&1
		[ $? -eq 0 ] && return
	fi
	# Determine the ".set" file to use.
	# Use the last one found that has a VDLFILE entry.
	# (The VDLFILE entry means that it has associated preinstalled files.)
	SET_FILE=notfound
	for i in $STAT_DOSI/[123456789]*.set
	do
		grep "^VDLFILE=" $i > /dev/null 2>&1
		[ $? -eq 0 ] && SET_FILE="$i"
	done
	[ "$SET_FILE" = "notfound" ] && return
	
	rm -f $VAR_DOSI/default.set
	if [ ! -d $VAR_DOSI ]
	then
		mkdir $VAR_DOSI
		chown bin $VAR_DOSI
		chgrp bin $VAR_DOSI
		chmod 775 $VAR_DOSI
	fi
	# Everything is ready so make a symbolic link to create default.set.
	ln -s $SET_FILE $VAR_DOSI/default.set
	return
}
Create_Default_Set

# Get ready to run win4lin!

sh $STATICMERGE/mrg_setup.sh

# If there is a DOS available to install then install it!
dosinstall noprompt default > $VARMERGE/log/autodosi.log 2>&1

usrMsg LOG INST_DONE "Installation of $MERGENAME is complete.\n" "$MERGENAME"

usrMsg ECHO INST_RELOG_CSH "\n\
All users that use the C shell (csh) or similar shells\n\
(e.g. tcsh) need to log out and log back in so that the\n\
newly installed commands can be used.\n\
\n"

if [ "$AGREE" != "yes" ]
then
	echo
	echo "You must read and agree with the End User License Agreement"
	echo "in file $STATICMERGE/license.txt before using Win4Lin."
	echo "If you do not agree, you must not use Win4Lin and you must"
	echo "remove the Win4Lin package.  (e.g. rpm -e Win4Lin)"
	echo
fi

exit 0
