:
# SCCSID(@(#)set_nls.sh	1.26 LCC) Modified 16:47:01 7/2/99
#############################################################################
# Set NLS environment variables for other Merge scripts and programs.
#############################################################################
#
#	Copyright (c) 1990-1999 TreLOS, Inc.
#	All rights reserved.
#	This is an unpublished work containing CONFIDENTIAL INFORMATION
#	that is the property of TreLOS, Inc.
#	Any unauthorized use, duplication or disclosure is prohibited.
#
#############################################################################
# Provides the following functions:
#	usrMsg
#	EchoText
# Sets the following variables:
#	MFILE
# Note:
# STATICMERGE must already be set!
# VARMERGE is used if it is set!
# DEFAULTMERGE is used if it is set.

DEFAULTMERGE=${DEFAULTMERGE:-/etc/default/merge}
set_defaults_from_file()
{
	[ -z "$DEFAULT_LANG_VAR" ] && \
		eval `grep "^DEFAULT_LANG_VAR=" $1`
	[ -z "$DEFAULT_UNIXCODESET_VAR" ] && \
		eval `grep "^DEFAULT_UNIXCODESET_VAR=" $1`
}
set_systemtype_from_file()
{
	[ -z "$SYSTEMTYPE" ] && eval `grep "^SYSTEMTYPE=" $1`
}
if [ -s $DEFAULTMERGE ]
then
	set_defaults_from_file $DEFAULTMERGE
fi
if [ -s "$STATICMERGE/Version" ]
then
	set_defaults_from_file $STATICMERGE/Version
	set_systemtype_from_file $STATICMERGE/Version
fi
if [ -s "$VARMERGE/Version" ]
then
	set_defaults_from_file $VARMERGE/Version
	set_systemtype_from_file $VARMERGE/Version
fi

DEFAULT_LANG_VAR=${DEFAULT_LANG_VAR:-english}
DEFAULT_UNIXCODESET_VAR=${DEFAULT_UNIXCODESET_VAR:-88591}

if [ -s $DEFAULTMERGE ]
then
	[ -z "$MERGE_UNIX_LANGUAGE" ] && \
		eval `grep "^MERGE_UNIX_LANGUAGE=" $DEFAULTMERGE`
	[ -z "$MERGE_UNIX_CODESET" ] && \
		eval `grep "^MERGE_UNIX_CODESET=" $DEFAULTMERGE`
fi

STATICMBIN=${STATICMBIN:-/usr/bin}
LMFM=$STATICMBIN/lmfmsg


# For SCO32 the file /etc/default/lang contains to value for LANG. That
# file contains all of the possible setting for lang and the first one
# in the file is the one we want.
LANG=${LC_ALL:-${LC_MESSAGES:-${LANG}}}	# LC_ALL and LC_MESSAGES overrides LANG
if [ -z "$LANG" ]
then
	if [ "$SYSTEMTYPE" = "sco32" -o "$SYSTEMTYPE" = "sco" ]
	then

		LANG=`grep '^LANG=' /etc/default/lang | head -1 | sed -e 's/LANG=//p'`
	elif [ "$SYSTEMTYPE" = "uw7" -o "$SYSTEMTYPE" = "gemini" ]
	then
		eval `defadm locale`  2>/dev/null
	elif [ "$SYSTEMTYPE" = "linux" ]
	then
		eval `locale | grep "^LANG="`
	else
		echo "Unknown SYSTEMTYPE"
	fi
fi

if [ "x$LANG" = "x" -o "x$LANG" = "xPOSIX" ]
then
	FRONTPART=${DEFAULT_LANG_VAR}
	BACKPART=${DEFAULT_UNIXCODESET_VAR}
else
	# At this poing LANG is set.  Determine the component parts.
	FRONTPART=`echo $LANG | cut -f1,1 -d"."`
	echo $LANG | grep  "\\." > /dev/null 2>&1
	if [ $? -eq 0 ]
	then
		BACKPART=`echo $LANG | cut -f2,2 -d"."`
	else
		BACKPART=${DEFAULT_UNIXCODESET_VAR}
	fi
fi
# MERGE_UNIX_LANGUAGE and MERGE_UNIX_CODESET are used instead if they are set.
[ -n "$MERGE_UNIX_LANGUAGE" ] &&  FRONTPART=$MERGE_UNIX_LANGUAGE
[ -n "$MERGE_UNIX_CODESET"  ] &&  BACKPART=$MERGE_UNIX_CODESET

if [ -z "$MFILE" ]
then
	if [ -n "$MERGEMSGPATH" ]
	then
		MFILE=$MERGEMSGPATH/${FRONTPART}.${BACKPART}
		if [ ! -f $MFILE ]
		then
			MFILE=$MERGEMSGPATH/${FRONTPART}.lmf
		fi
		if [ ! -f $MFILE ]
		then
			MFILE=$MERGEMSGPATH/english.lmf
		fi
	fi
	if [ -z "$MFILE" -o ! -f "$MFILE" ]
	then
		MFILE=$STATICMERGE/mergemsg/${FRONTPART}.${BACKPART}
	fi
	if [ ! -f $MFILE ]
	then
		MFILE=$STATICMERGE/mergemsg/${FRONTPART}.lmf
	fi
	if [ ! -f $MFILE ]
	then
		MFILE=$STATICMERGE/mergemsg/english.lmf
	fi
	export MFILE
fi

# This function is used to handle the error messages.  The calling sequence is:
#	usrMsg <code> <nls-name> <default-message> [<nls-params>]
# <code> is ECHO, LOG, WARN, ERROR, TRACE, or STRING.
# <nls-name> is the NLS name used for this string in the message file.
# <default-message> is the default message string.
# <nls-params> are the (optional) parameters for the NLS string.  These
# should be the same shell variables that are used in the default string.
#
# NOTE: *ALL* user output should be sent through this function, to ensure
# that it goes to both the log file, and to the user's terminal, and to make
# sure that all strings are 'internationalized'.

# Explanation of the <code> values:
#	ECHO	-print to user and log file
#	LOG	-print to log file only
#	WARN	-same a ECHO, but "WARNING: " is prefixed.
#	ERROR	-same a ECHO, but "ERROR: " is prefixed.
#	TRACE	-print to log file, and to user when MERGETRACE is defined.
#	STRING	-put the output in the variable "USRMSG_STRING".

# All output goes to the logfile defined by $LOG_FILE.  If the caller
# does not define LOG_FILE, then no logging will be done.  

usrMsg()
{
	[ "x$USRMSGNULL" = "x" ] && USRMSGNULL=/dev/null
	USRMSGLOGFILENAME=${LOG_FILE:-/dev/null}
	if [ $# -lt 3 ]
	then
		EchoText "INTERNAL: Bad usrMsg() param count." \
			| tee -a $USRMSGLOGFILENAME
	fi

	# Get the first three parameters, and shift them out.  This leaves all
	# of the rest of the argument line available as $*.

	MSG_TYPE="$1"
	STRING_NAME="$2"
	DEFAULT_STRING="$3"
	shift 3

	# Next, determine what we are going to print at the beginning of the
	# message, and whether it will cause a failure or not.

	MM="LCC.MERGE.UNIX.SET_NLS"
	PREFIX_STR=""
	if [ "$MSG_TYPE" = "WARN" ]
	then
		PREFIX_STR=`$LMFM -n $MFILE $MM.WARN_PREFIX 2>> $USRMSGNULL`
		if [ $? -ne 0 -o "z$PREFIX_STR" = "z" ]
		then
			PREFIX_STR="WARNING: "
		fi
		MSG_TYPE=ECHO
	elif [ "$MSG_TYPE" = "ERROR" ]
	then
		ERROR_FOUND=TRUE
		PREFIX_STR=`$LMFM -n $MFILE $MM.ERR_PREFIX 2>> $USRMSGNULL`
		if [ $? -ne 0 -o "z$PREFIX_STR" = "z" ]
		then
			PREFIX_STR="ERROR: "
		fi
		MSG_TYPE=ECHO
	elif [ "$MSG_TYPE" = "TRACE" ]
	then
		if [ -n "$MERGETRACE" ]
		then
			MSG_TYPE=ECHO
		fi
	elif [ "$MSG_TYPE" != "ECHO" -a "$MSG_TYPE" != "LOG" \
		-a "$MSG_TYPE" != "STRING" ]
	then
		EchoText "INTERNAL: Bad usrMsg() msg type" \
			| tee -a $USRMSGLOGFILENAME
	fi

	arg1="$1"
	arg2="$2"
	arg3="$3"
	arg4="$4"
	arg5="$5"
	arg6="$6"
	arg7="$7"
	arg8="$8"
	if [ $# -gt 8 ]
	then
		shift 8
	else
		shift $#
	fi

	# Now, try to get the 'internationalized' version, but fall back to
	# the default if necessary.
	MSGSTR=`$LMFM -n $MFILE $DM.$STRING_NAME \
	   "$arg1" "$arg2" "$arg3" "$arg4" "$arg5" "$arg6" "$arg7" "$arg8" \
	    $* 2>> $USRMSGNULL`
	if [ $? -ne 0 -o "z$MSGSTR" = "z" ]
	then
		MSGSTR="$DEFAULT_STRING"
	fi
	USRMSG_STRING="${PREFIX_STR}${MSGSTR}"

	if [ "$MSG_TYPE" = "ECHO" ]
	then
		if [ "x$CCS_LOG" = "xccs_log" ]
		then
			log "${USRMSG_STRING}\c" 
		else
			EchoText "${USRMSG_STRING}\c"
		fi
	fi
	if [ "$MSG_TYPE" != "STRING" ]
	then
		EchoText "${USRMSG_STRING}" >> $USRMSGLOGFILENAME
	fi
}

# The EchoText function provides a standard "echo" function
# for all platforms.  The important thing here is that the
# echo command recognize the special "\n" and "\c" formats.
EchoText()
{
	if [ "x$SYSTEMTYPE" = "xlinux" ]
	then
		echo -e $*
	else
		echo $*
	fi
}
