#!/bin/sh
# SCCSID(@(#)setdosloc.sh	1.5) Modified 12:33:26 1/3/00
########################################################################
#Script to set the DOS locale info in system wide default file.
########################################################################

# This script is called with two parameters
# The first is the "country number" code.  This is based on the DOS
# "country number", with an additional character in some cases that
# is used to indicate one of several keyboards used in that country.

# The second parameter is optional.  When the second parameter
# is the word "AddOnly", then existing setting in $DEFAULTMERGE
# will not be changed, only added if they are not already there.

# These are the setting that this script determines:
#    DOS Country number: COUNTRY_NUM
#    DOS KEYB command keyboard code: KEYB_CODE
#    Initial codepage: ICP
#    Hardware codepage: HWCP
#    Soft_codepage: SCP

# NOTE:	The file localedata in the unix/janus directory contains
#	information about the locale.  Someday, we may want to change
#	this file to get the locale information from localedata so that
#	the locale information is encoded in only one place.
#

# Get the VARMERGE and DEFAULTMERGE settings.
. /etc/mrgssv.sh

[ "x$NULL" = "x" ] && NULL=/dev/null
if [ "x$LOG_FILE" = "x" ]
then
	if [ -d $VARMERGE/log ]
	then
		LOG_FILE=$VARMERGE/log/log.sdl
	else
		LOG_FILE=/dev/null
	fi
fi
TMPFILE=$VARMERGE/tmp/il.$$

# Defaults:
COUNTRY_NUM=
KEYB_CODE=
ICP=
HWCP=
SCP=
echo "Set_DOS_locale $1 $2" >> $LOG_FILE
case $1 in
1)	# U.S.
	# Use all defaults.
	;;
2)	# Canadian-French
	COUNTRY_NUM=2
	KEYB_CODE=cf
	ICP=850
	HWCP=437
	SCP=850
	;;
3)	# Latin America
	COUNTRY_NUM=3
	KEYB_CODE=la
	ICP=850
	HWCP=437
	SCP=850
	;;
31)	# Netherlands
	COUNTRY_NUM=31
	KEYB_CODE=nl
	ICP=850
	HWCP=437
	SCP=850
	;;
32)	# Belgium
	COUNTRY_NUM=32
	KEYB_CODE=be
	ICP=850
	HWCP=437
	SCP=850
	;;
32f)	# Belgium - French
	COUNTRY_NUM=32
	KEYB_CODE=fr
	ICP=850
	HWCP=437
	SCP=850
	;;
33)	# France
	COUNTRY_NUM=33
	KEYB_CODE=fr
	ICP=850
	HWCP=437
	SCP=850
	;;
33a)	# France (DOS keyboard id 120)
	COUNTRY_NUM=33
	# Note that DR-DOS does not make use of the id option.
	KEYB_CODE=fr/id:120
	ICP=850
	HWCP=437
	SCP=850
	;;
33b)	# France (DOS keyboard id 189)
	COUNTRY_NUM=33
	# Note that DR-DOS does not make use of the id option.
	KEYB_CODE=fr/id:189
	ICP=850
	HWCP=437
	SCP=850
	;;
34)	# Spain
	COUNTRY_NUM=34
	KEYB_CODE=sp
	ICP=850
	HWCP=437
	SCP=850
	;;
36)	# Hungary
	COUNTRY_NUM=36
	KEYB_CODE=hu
	ICP=850
	HWCP=437
	SCP=850
	;;
38)	# Yugoslavia
	COUNTRY_NUM=38
	KEYB_CODE=yu
	ICP=850
	HWCP=437
	SCP=850
	;;
39)	# Italy
	COUNTRY_NUM=39
	KEYB_CODE=it
	ICP=850
	HWCP=437
	SCP=850
	;;
39a)	# Italy	(keyboard id 141)
	COUNTRY_NUM=39
	# Note that DR-DOS does not make use of the id option.
	KEYB_CODE=it/id:141
	ICP=850
	HWCP=437
	SCP=850
	;;
39b)	# Italy	(keyboard id 142)
	COUNTRY_NUM=39
	# Note that DR-DOS does not make use of the id option.
	KEYB_CODE=it/id:142
	ICP=850
	HWCP=437
	SCP=850
	;;
41f)	# Switzerland (French)
	COUNTRY_NUM=41
	KEYB_CODE=sf
	ICP=850
	HWCP=437
	SCP=850
	;;
41g)	# Switzerland (German)
	COUNTRY_NUM=41
	KEYB_CODE=sg
	ICP=850
	HWCP=437
	SCP=850
	;;
42c)	# Czech
	COUNTRY_NUM=42
	KEYB_CODE=cz
	ICP=850
	HWCP=437
	SCP=850
	;;
42s)	# Slovak
	COUNTRY_NUM=42
	KEYB_CODE=sl
	ICP=850
	HWCP=437
	SCP=850
	;;
44)	# United Kingdom
	COUNTRY_NUM=44
	KEYB_CODE=uk
	ICP=437
	HWCP=437
	SCP=850
	;;
44a)	# United Kingdom (keyboard id 166)
	COUNTRY_NUM=44
	# Note that DR-DOS does not make use of the id option.
	KEYB_CODE=uk/id:166
	ICP=437
	HWCP=437
	SCP=850
	;;
44b)	# United Kingdom (keyboard id 168)
	COUNTRY_NUM=44
	# Note that DR-DOS does not make use of the id option.
	KEYB_CODE=uk/id:168
	ICP=437
	HWCP=437
	SCP=850
	;;
45)	# Denmark
	COUNTRY_NUM=45
	KEYB_CODE=dk
	ICP=850
	HWCP=437
	SCP=850
	;;
46)	# Sweden
	COUNTRY_NUM=46
	KEYB_CODE=sv
	ICP=850
	HWCP=437
	SCP=850
	;;
47)	# Norway
	COUNTRY_NUM=47
	KEYB_CODE=no
	ICP=850
	HWCP=437
	SCP=850
	;;
48)	# Poland
	COUNTRY_NUM=48
	KEYB_CODE=pl
	ICP=850
	HWCP=437
	SCP=850
	;;
49)	# Germany
	COUNTRY_NUM=49
	KEYB_CODE=gr
	ICP=850
	HWCP=437
	SCP=850
	;;
55)	# Brazil
	COUNTRY_NUM=55
	KEYB_CODE=br
	ICP=850
	HWCP=437
	SCP=850
	;;
61)	# International English
	COUNTRY_NUM=61
	# International English does not use keyb code.
	KEYB_CODE=
	ICP=437
	HWCP=437
	SCP=850
	;;
81|81a|81x)	# Japan
	# Japanese setup is done when Japanese DOS is installed.
	;;
351)	# Portugal
	COUNTRY_NUM=351
	KEYB_CODE=po
	ICP=850
	HWCP=437
	SCP=850
	;;
358)	# Finland
	COUNTRY_NUM=358
	KEYB_CODE=su
	ICP=850
	HWCP=437
	SCP=850
	;;
*)
	echo Error: Unknown country number.
	echo Assuming defaults.
	# Use all defaults.
	;;
esac
echo "COUNTRY_NUM=$COUNTRY_NUM"	>> $LOG_FILE
echo "KEYB_CODE=$KEYB_CODE"	>> $LOG_FILE
echo "ICP=$ICP"			>> $LOG_FILE
echo "HWCP=$HWCP"		>> $LOG_FILE
echo "SCP=$SCP"			>> $LOG_FILE
# Set the locale variables in $DEFAULTMERGE (if they are
# not already set).

UPDATE_COUNTRY=yes
UPDATE_KEYB=yes
UPDATE_CODEPAGE=yes
UPDATE_DOS_IMAGE_NLSFUNC=yes
UPDATE_DOS_VGA_HARDWARE_CODEPAGE=yes
UPDATE_DOS_VGA_SOFT_CODEPAGE=yes
if [ "x$2" = "xAddOnly" ]
then
	grep "^COUNTRY=" $DEFAULTMERGE >> $NULL 2>&1
	[ $? -eq 0 ] && UPDATE_COUNTRY=no
	grep "^KEYB=" $DEFAULTMERGE >> $NULL 2>&1
	[ $? -eq 0 ] && UPDATE_KEYB=no
	grep "^CODEPAGE=" $DEFAULTMERGE >> $NULL 2>&1
	[ $? -eq 0 ] && UPDATE_CODEPAGE=no
	grep "^DOS_IMAGE_NLSFUNC=" $DEFAULTMERGE >> $NULL 2>&1
	[ $? -eq 0 ] && UPDATE_DOS_IMAGE_NLSFUNC=no
	grep "^DOS_VGA_HARDWARE_CODEPAGE=" $DEFAULTMERGE >> $NULL 2>&1
	[ $? -eq 0 ] && UPDATE_DOS_VGA_HARDWARE_CODEPAGE=no
	grep "^DOS_VGA_SOFT_CODEPAGE=" $DEFAULTMERGE >> $NULL 2>&1
	[ $? -eq 0 ] && UPDATE_DOS_VGA_SOFT_CODEPAGE=no
else
	rm -f $TMPFILE
	cat $DEFAULTMERGE \
	| grep -v "^COUNTRY=" \
	| grep -v "^KEYB=" \
	| grep -v "^CODEPAGE=" \
	| grep -v "^DOS_IMAGE_NLSFUNC=" \
	| grep -v "^DOS_VGA_HARDWARE_CODEPAGE=" \
	| grep -v "^DOS_VGA_SOFT_CODEPAGE=" \
	> $TMPFILE
	cp $TMPFILE $DEFAULTMERGE
	rm -f $TMPFILE
fi

if [ "$UPDATE_COUNTRY" = "yes" ]
then
	echo "COUNTRY=$COUNTRY_NUM" >> $DEFAULTMERGE
fi
if [ "$UPDATE_KEYB" = "yes" ]
then
	echo "KEYB=$KEYB_CODE" >> $DEFAULTMERGE
fi
if [ "$UPDATE_CODEPAGE" = "yes" ]
then
	if [ -z "$ICP" ]
	then
		echo "CODEPAGE=" >> $DEFAULTMERGE
	else
		echo "CODEPAGE=pc$ICP" >> $DEFAULTMERGE
	fi
fi

# Nlsfunc is needed for code page switching (as configured by
# setting DOS_VGA_HARDWARE_CODEPAGE and DOS_VGA_SOFT_CODEPAGE).
#
# Note: for DR-DOS, in order to have an initial code page other
# than the default (437), nlsfunc must be loaded, even if code
# page switching is not going to be done.
# So if any code page other than 437 is going to be used with
# DR-DOS, Merge must be set up (via the file $DEFAULTMERGE)
# for code page switching.
# 
if [ "$UPDATE_DOS_IMAGE_NLSFUNC" = "yes" ]
then
	if [ "x$HWCP" != "x" -o "x$SCP" != "x" ]
	then
		echo "DOS_IMAGE_NLSFUNC=on" >> $DEFAULTMERGE
	else
		echo "DOS_IMAGE_NLSFUNC=off" >> $DEFAULTMERGE
	fi
fi
if [ "$UPDATE_DOS_VGA_HARDWARE_CODEPAGE" = "yes" ]
then
	echo "DOS_VGA_HARDWARE_CODEPAGE=$HWCP" >> $DEFAULTMERGE
fi
if [ "$UPDATE_DOS_VGA_SOFT_CODEPAGE" = "yes" ]
then
	echo "DOS_VGA_SOFT_CODEPAGE=$SCP" >> $DEFAULTMERGE
fi

echo "New $DEFAULTMERGE:" >> $LOG_FILE
cat $DEFAULTMERGE >> $LOG_FILE
echo "=========================" >> $LOG_FILE
