:
# SCCSID(@(#)inst_font.sh	1.14) Modified 00:49:13 7/8/99

### Copyright (c) 1991-1999 TreLOS, Inc.
### All rights reserved.
### This is an unpublished work containing CONFIDENTIAL INFORMATION
### that is the property of TreLOS, Inc.
### Any unauthorized use, duplication or disclosure is prohibited.

########################################################################
#
# 	Font installation/removal script for the Merge X Clients Package.
#
# ----------------------------------------------------------------------
#	First the the correct X font directory is determined,
#	and then the fonts in the installed into or removed from
#	that X font directory.
#
#	This script has one parameter, either "install" or "remove".
#	(If no parameters, then assume "install".)
#
# ----------------------------------------------------------------------
# In X11R2, all fonts are located in the "/usr/X/lib/fonts" directory.
# In X11R3, fonts are distributed between several directories under
#	the "/usr/X/lib/fonts" directory.  We put ours in the "misc"
#	subdirectory, and then run "mkfontdir" to install them.
# In SCO XSight, fonts are distributed between several directories
#	under the "/usr/lib/X11/fonts" directory.  We put ours in the
#	"misc" subdirectory, and then run "mkfontdir" to install them.

REMOVE_OPTION=remove
INST_OR_REM=$1
INST_OR_REM=${INST_OR_REM:-install}

. /etc/mrgssv.sh

if [ -z "$LOG_FILE" ]
then
	LOG_FILE=$VARMERGE/tmp/inst_font.log
	rm -f $LOG_FILE
fi
export LOG_FILE
echo "Font $INST_OR_REM" >> $LOG_FILE

# The Version file sets "INSTALLTYPE", which we later test to
# determine which kind of removal to do.
eval `grep "^INSTALLTYPE=" $STATICMERGE/Version`

# Linux 2.2.x systems have all the X stuff under "/usr/X11R6".

R2_FONTDIR=/usr/X/lib/fonts
R3_FONTDIR=/usr/X/lib/fonts/misc
SCO_FONTDIR=/usr/lib/X11/fonts/misc
LNX_FONTDIR=/usr/X11R6/lib/X11/fonts/misc
SVR4_XBIN=/usr/X/bin
SCO_XBIN=/usr/bin/X11
LNX_XBIN=/usr/X11R6/bin

SS_DIR=$VARMERGE/ss
rm -f $SS_DIR/inst_font.sh
reschedule_exit()
{
	if [ "$INST_OR_REM" = "$REMOVE_OPTION" ]
	then
		rm -f $SS_DIR/inst_font.sh
		rmdir $SS_DIR > /dev/null 2>&1
		exit $1
	fi
	if [ ! -d $SS_DIR ]
	then
		mkdir $SS_DIR
		chown bin $SS_DIR
		chgrp bin $SS_DIR
		chmod 755 $SS_DIR
	fi
	echo :  > $SS_DIR/inst_font.sh
	echo "sh $STATICMERGE/xc/install/inst_font.sh install" \
	       >> $SS_DIR/inst_font.sh
	chmod 555 $SS_DIR/inst_font.sh
	chgrp bin $SS_DIR/inst_font.sh
	chown bin $SS_DIR/inst_font.sh
	exit $1
}

if   [ -d $LNX_FONTDIR ] ; then FONTDIR=$LNX_FONTDIR
elif [ -d $SCO_FONTDIR ] ; then FONTDIR=$SCO_FONTDIR
elif [ -d $R3_FONTDIR ]  ; then FONTDIR=$R3_FONTDIR
elif [ -d $R2_FONTDIR ]  ; then FONTDIR=$R2_FONTDIR
else
	# Cannot install/remove fonts because the font
	# directory does not exist.
	reschedule_exit 1
fi

if   [ -d $LNX_XBIN ]  ; then XBIN=$LNX_XBIN
elif [ -d $SCO_XBIN ]  ; then XBIN=$SCO_XBIN
elif [ -d $SVR4_XBIN ] ; then XBIN=$SVR4_XBIN
else
	# Cannot install/remove fonts because the X bin
	# directory does not exist.
	reschedule_exit 1
fi

Install_Fonts()
{
	if [ -f $VARMERGE/installedff ]
	then
		# Already installed.
		return
	fi

	cd $STATICMERGE/xc/fonts
	# First compile the source files to PCF.
	FONT_COMP=$XBIN/bdftopcf
	if [ -x $FONT_COMP ]
	then
		for Name in *.bdf
		do
			[ ! -s $Name ] && continue
			PCF_Name="`basename $Name bdf`pcf"
			$FONT_COMP -o $FONTDIR/$PCF_Name $Name
			echo $FONTDIR/$PCF_Name >> $VARMERGE/installedff
			echo "install $PCF_Name" >> $LOG_FILE
	
			# Document this font source file.
			export Name
			cat $FONTDIR/$Name 2>/dev/null | while true
			do
				# This loop is running in a sub-shell,
				# so an "exit" will only exit the loop.
				read A B C
				if [ -z "$A" ]
				then
					exit
				fi 
				if [ "$A" = "FONT" ]
				then
					# List the font name and source file.
					echo "$B\t$Name" >> $VARMERGE/fonts.src
					exit
				fi
			done
		done
	else
		# link/copy the SNF files if they exist.  (They don't exist
		# for SCO and ESMP.)
		for Name in *.snf
		do
			[ ! -f $Name ] && continue
			# Just copy in the snf files.
			ln -s `pwd`/$Name $FONTDIR/$Name
			# Remember the file so we can de-install it later.
			echo $FONTDIR/$Name >> $VARMERGE/installedff
			echo "install $Name" >> $LOG_FILE
		done
	fi
	
	if [ "$INSTALLTYPE" = "pkgadd" ]
	then
		if [ -f $VARMERGE/fonts.src ]
		then
			installf $PKGINST $VARMERGE/fonts.src f 664 bin bin
		fi

		if [ -f $VARMERGE/installedff ]
		then
			installf $PKGINST $VARMERGE/installedff f 664 bin bin
		fi

		installf -f $PKGINST		# Commit the changes
	else
		if [ -f $VARMERGE/fonts.src ]
		then
			chmod 664 $VARMERGE/fonts.src
			chgrp bin $VARMERGE/fonts.src
			chown bin $VARMERGE/fonts.src
		fi
	
		if [ -f $VARMERGE/installedff ]
		then
			chmod 664 $VARMERGE/installedff
			chgrp bin $VARMERGE/installedff
			chown bin $VARMERGE/installedff
		fi
	fi
}

#  Function to remove the XCRT fonts.
#
Remove_Fonts()
{
	rm -f $SS_DIR/inst_font.sh
	rmdir $SS_DIR > /dev/null 2>&1
	if [ -s $VARMERGE/installedff ]
	then
		for i in `cat $VARMERGE/installedff`
		do
			if [ -f $i ]
			then
				echo "remove $i" >> $LOG_FILE
				rm -f $i
			fi
		done
	fi
	
	# Delete files created by the font installation.
	if [ "$INSTALLTYPE" = "pkgadd" ]
	then
		removef $PKGNAME $VARMERGE/installedff
		removef $PKGNAME $VARMERGE/fonts.src
		rm -f $VARMERGE/installedff
		rm -f $VARMERGE/fonts.src
		removef -f $PKGNAME
	else
		rm -f $VARMERGE/installedff
		rm -f $VARMERGE/fonts.src
	fi
}

if [ "$INST_OR_REM" = "$REMOVE_OPTION" ]
then
	Remove_Fonts
else
	Install_Fonts
fi
	
# Run "mkfontdir" to complete the installation or removal.
if [ -x $XBIN/mkfontdir ]
then
	MFD=$XBIN/mkfontdir

else
	echo "No mkfontdir command" >> $LOG_FILE
	exit 1
fi

if [ -n "$MFD" -a -x "$MFD" ]
then
	cd $FONTDIR
	$MFD
fi
echo "Font $INST_OR_REM done" >> $LOG_FILE

exit 0
