/*    
	utils.c	2.20
    	Copyright 1997 Willows Software, Inc. 

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public License as
published by the Free Software Foundation; either version 2 of the
License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; see the file COPYING.LIB.  If
not, write to the Free Software Foundation, Inc., 675 Mass Ave,
Cambridge, MA 02139, USA.


For more information about the Willows Twin Libraries.

	http://www.willows.com	

To send email to the maintainer of the Willows Twin Libraries.

	mailto:twin@willows.com 

 */


#include "windows.h"
#include "utils.h"
#include "Log.h"
#include "ctype.h"

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "debugdefs.h"
 
/*
 *	semantics:
 *	debug = [class]{+|-}channel[=stream]
 *	1	class is one of builtin 8
 *		       debug fixme fail	warn stub log call ret
 *
 *      2	channels are builtin
 *			API, LIB, X11, MSG, INT, DOS, BIN, SYS
 *		or any user defined
 *			ie. WARN(profile,".twinrc does not exist\n");
 *		or the special class all
 *	
 *	3	stream is one of
 *		0	no output, same as -channel
 *		1	stdout
 *		2	stderr
 *		3	logfile as defined by WIN_OPENLOG
 *
 *	[tools]
 *	debug = warn+api,trace-lib,-X11,+sys=2
 *
 *	debug class is left to right
 *
 */

static int      opnlog_fd;
static char    *opnlog_file;
static char    *opnlog_fmt;

static int
opnlog()
{
        char	*fn;

	if (opnlog_fd)
		return(opnlog_fd); 

	if (opnlog_file)
		fn = opnlog_file;
	else
		fn = (char *)getenv("WIN_OPENLOG");

	if ( fn ) {
		char *p;

		if( *fn == '-') 
			return (opnlog_fd = 1);

		/* get rid of old style mask... */
		if ((p = strchr(fn,' ')))
			*p = 0;	
		return (opnlog_fd = creat(fn, 0666));
	}

	return 0;
}

void
DumpMemory(char *s,char *p,int n)
{
	int i,j;
	char x;
	char   *chrptr;
	char	buffer[132];
	char	temp[80];

	if(n==0)
		return;

	logstr(LF_DEBUG,"%s:\n",s?s:"DumpMemory");
	if(p == 0) {
		logstr(LF_DEBUG,"NULL pointer\n");
		return;
	}
	for(i=0;i<n;i++) {
		sprintf(buffer,"%x: ",(unsigned int) p);
		chrptr = p;

		for(j=0;j<16;j++) {
			x=*p++;
			x &= 0xff;
			sprintf(temp,"%2.2x ",x&0xff);
			strcat(buffer,temp);
		}		
		strcat(buffer,"   ");
		p = chrptr;
		chrptr = buffer + strlen(buffer);
		for(j=0;j<16;j++) {
			if(isprint(*p))
				*chrptr++ = *p;
			else	*chrptr++ = '.';
			p++;
		}	
		*chrptr = 0;

		i += 16;
		logstr(LF_CONSOLE,"%s\n",buffer);
	}
}

/*VARARGS */
void
logstr(unsigned long flg, ...)
{
	va_list args;
	static int apidepth;
	int  fd;
	char *fmt;
	char *header = 0;
	char title[32];
	short class,channel;

	va_start(args, flg);

	/* point to fmt and args... */
	fmt = va_arg(args, char *);
	channel = flg>>8;
	class   = flg & 0xff;

	fd = debug_msg_enabled[channel][class];
	switch(fd) {
	case 0:
		break;
	case 1:
		break;
	case 2:
		break;
	case 3: 
		fd = opnlog();
	}

	if(fd) {
		if(opnlog_fmt == 0)
			opnlog_fmt = WinMalloc(64*1024);

		switch(class) {
			case dbcl_DEBUG:
				break;

			case dbcl_FIXME:
			case dbcl_WARN:
			case dbcl_STUB:
			case dbcl_TRACE:
				/* print out a header */
				sprintf(title,"%s %s         ",
					(char *)debug_ch_name[channel],
					(char *)debug_cl_name[class]);
				sprintf(opnlog_fmt,"%-14.14s : ",title);
				header = title;
				break;
			case dbcl_CALL:
			case dbcl_RET:
			case dbcl_FAIL:
				if(class == dbcl_RET || class == dbcl_FAIL)
					apidepth--;
				/* print out a header */
				sprintf(title,"%-3.3s %-5.5s %2d",
					(char *)debug_ch_name[channel],
					(char *)debug_cl_name[class],
					apidepth);
				if(class == dbcl_CALL)
					apidepth++;
				sprintf(opnlog_fmt,"%-14.14s : ",title);
				header = opnlog_fmt;
				break;
		}

		if(header) {
			write(fd, opnlog_fmt, strlen(opnlog_fmt));
		}
		
		/* print out remainder of message locally */
		vsprintf(opnlog_fmt,fmt,args);
		write(fd, opnlog_fmt, strlen(opnlog_fmt));
	}

	va_end(args);
}

#if 0
	APICALL(API,"GetTickCount()\n");
	APIRET(API,"GetTickCount()\n");
	TRACE(X11,"This is a message\n");
	TRACE(MSG,"[HWND=%x MSG=%x WPARAM=%x LPARAM=%x]\n",h,m,w,l);

	FAIL(INT,"failed\n");
	FAIL(DOS,"failed\n");
	TRACE(SYS,"Driver call\n");
	STUB(API,"CreateThread()\n");
	DEBUG(BIN,"PE Loader: find debug symbols\n");
	FIXME(LIB,"WindowsDirectory = %s\n",GetWindowsDirectory());
	WARN(profile,".twinrc does not exist\n");
#endif
