/*--------------------------------------------------------------------------
**      glue.c                   Twine           
**
**              
**------------------------- < License Information > ------------------------
**
**  
**      This file is released under terms of an LGPL.  Readers of 
**  this file may modify or use this source code so long as they make all
**  modifications publically available.  Please refer to TWINE-LICENSE 
**  for further details.
**  
**
**      The original Copyright is in 1999, by CodeWeavers, Inc.
**
**      
**------------------------ < File Content Description > --------------------
**
**  Module:         
**	glue.c 
**
**  Description:
**      interface between wine debugger and twine
**
**  Functions defined:
**
**	should be no functions exported.
**
**	A single .init routine is defined which calls back to twine
**	with the address of the debugger entry point. this function
**	is then called every time the library wants to execute a
**	debugger function.  Similar to a window procedure, it takes
**	parameters based on the function being called.
**
**	void wine_debug_proc(int opcode,int wparam,int lparam)
**    
**------------------------- < Revision Information > -----------------------
** $Header: /cVs/Willows/twin/debugger/glue.c,v 1.2 1999/03/22 21:52:55 jching Exp $
**
**      Full Revision history at bottom of file
**      
**--------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
**              E X T E R N A L     I N C L U D E S 
**--------------------------------------------------------------------------*/

#include "windows.h"
#include "twindll.h"
#include "dlfcn.h"
#include "Debug.h"
#include "debugger.h"

void  DEBUG_Main( int signal );
int   DEBUG_ProcessElfObject(const char * filename, void *load_offset);

/*----------------------------------------------------------------------------
**              I N T E R N A L     I N C L U D E S
**--------------------------------------------------------------------------*/


/*------------------------- < Full Revision History > ----------------------
** $Log: glue.c,v $
** Revision 1.2  1999/03/22 21:52:55  jching
** Twine 1.0 merge, code cleanup and added NOTE as per Wine license.
**
**-------------------------------------------------------------------------*/


LIBENTRY_DECL(debug_init);

/*  
 * Initialization Entry Point.
 * called when debugger shared library is loaded.
 * calls TWIN_DebuggerProc to initialize callback.
 */

void wine_debug_proc(int ,int ,int );

void
debug_init()
{
	fprintf(stderr,"LoadLibrary: DEBUGGER\n");	
	TWIN_DebuggerProc(0,0,(void *) wine_debug_proc);
}

/*
 * Interface to load symbols from a shared library
 * pass in the shared library name.  This will be
 * hooked into from the native library loadlibrary
 */
void DEBUG_LoadLibrary(char *name)
{
	Dl_info info;
	void *hso;
	void *addr;
	int rc;

	/*
    	 * The following is a way to find out where a shared
	 * library is located.  It takes two steps, first 
	 * get a KNOWN address in the library, then using
	 * that address, lookup the ld.so information that
	 * describes where it is in memory
	 */
	hso = dlopen(name,RTLD_LAZY);
	addr = dlsym(hso,"_DYNAMIC");
	dlclose(hso);

	rc = dladdr(addr,&info);

	if(rc) {
		DEBUG_ProcessElfObject(info.dli_fname, info.dli_fbase);
	}
}

void DEBUG_InitCVDataTypes()
{
	//printf("InitCVDataTypes\n");
}

int 
DEBUG_RegisterELFDebugInfo(int load_addr,u_long size,char *filename)
{
	void *addr = (void *)load_addr;
	fprintf(stderr,"Load symbols: %s %p:%ld\n",
		filename,addr,size);
	return 0;
}

int 
PROFILE_GetWineIniString( const char *section, const char *key_name,
		const char *defkey,char *result,int len)
{
	char *symfile;
	if((symfile = getenv("TWIN_SYMFILE")))
		strcpy(result,symfile);
	else
		strcpy(result,"/tmp/wine.sym");
	return strlen(result);
}

void debuggerbreak();

void
wine_debug_proc(int opcode,int wparam,int lparam)
{
	FILE *init_file;
    	char fname[256];

	if(opcode == TWIN_DBGLOAD) {
    
    		init_file = fopen(".windbginit", "r");
    		if (!init_file) {
			sprintf(fname, "%s/.windbginit", getenv("HOME"));
			init_file = fopen(fname, "r");
    		}


    		if (init_file)
			debuggerbreak();
		return;
	}

	if(opcode == TWIN_DBGADDR) {
		char *p = 0;
		Dl_info info;

#if	0
		struct name_hash *nh;

            	DBG_ADDR address = { NULL, 0, wparam };

		p = DEBUG_FindNearestSymbol( &address, TRUE, &nh, 0, 0);

#else
		{
			char *errstr;
			info.dli_fname = 0;
			info.dli_fbase = 0;
			info.dli_sname = 0;
			info.dli_saddr = 0;

			dladdr((void *) wparam,&info);
			errstr = dlerror();

			p = (char *) info.dli_sname;
		}
#endif

		if(p) {
			sprintf((char *)lparam,"%s",p);
		} else {
			sprintf((char *)lparam,"[%x]",wparam); 
		}

		return;
	}

 	/* call wine debugger */
	wine_debug(wparam,(SIGCONTEXT *) lparam);
}
