
namespace eval ::graph:: {
    variable Graph
    
    set ::graph::Graph(_proc) {
        set cmd [lshift args]
        
        switch $cmd {
            "add" {
                ::graph::Add %W $args
            }
            "configure" {
                ::graph::Configure %W $args
            }
            "seriesconfigure" {
                ::graph::SeriesConfigure %W $args
            }
            default {
                eval {::graph::Graph(%W)} {$cmd} $args 
            }
        }        
    }
}

proc ::graph::Add {name varArgs} {
    variable Graph

    set series [lshift varArgs]
    set x [lshift varArgs]
    set y [lshift varArgs]

    # assert {![info exists Graph($name:$series:X)]}
    
    lappend Graph($name:_series) $series
    
    set Graph($name:$series:marker) none
    set Graph($name:$series:radius) 0
    set Graph($name:$series:type) line
    set Graph($name:$series:X) {}
    set Graph($name:$series:Y) {}
    
    SeriesConfigure $name [linsert $varArgs 0 $series -x $x -y $y]

    set series
}

proc ::graph::Configure {name varArgs} {
    variable Graph

    foreach {op arg} $varArgs {
        switch -exact -- $op {
            default {
                Graph($name) configure $op $arg
            }
        }
    }
}

proc ::graph::SeriesConfigure {name varArgs} {
    variable Graph

    set series [lshift varArgs]

    assert {[info exists Graph($name:$series:X)]}

    set recalc 0
    
    foreach {op arg} $varArgs {
        switch -exact -- $op {
            "-x" {
                set Graph($name:$series:X) $arg
                set Graph($name:_xmax) [max $Graph($name:_xmax) [lmax $arg]]
                set recalc 1
            }
            
            "-y" {
                set Graph($name:$series:Y) $arg
                set Graph($name:_ymax) [max $Graph($name:_ymax) [lmax $arg]]
                set recalc 1
            }

            "-type" {
                set Graph($name:$series:type) $arg
            }
            
            "-color" {
                set Graph($name:$series:color) $arg
            }
            
            "-marker" {
                set Graph($name:$series:marker) $arg
            }
            
            "-radius" {
                set Graph($name:$series:radius) $arg
            }
            
            default {
                puts "$op $arg"
                assert 0
            }
        }
    }

    if {$recalc} {
        Recalc $name
    }

    Redraw $name
}

proc ::graph::x2p {name x} {
    variable Graph
    expr {int(round($Graph($name:_xoffset)
                    + ($x * $Graph($name:_xscale))))}
}

proc ::graph::y2p {name y} {
    variable Graph
    expr {int(round($Graph($name:_height)
                    - ($Graph($name:_yoffset)
                       + ($y * $Graph($name:_yscale)))))}
}

proc ::graph::Redraw {name} {
    variable Graph
    if {!$Graph($name:_draw)} {
        after idle "::graph::Draw $name"
        set Graph($name:_draw) 1
    }
}

proc ::graph::Draw {name} {
    variable Graph

    set Graph($name:_draw) 0

    $name delete all
    
    $name create line [x2p $name 0] [y2p $name 0] \
        [x2p $name 0] [y2p $name $Graph($name:_ymax)]
    $name create line [x2p $name 0] [y2p $name 0] \
        [x2p $name $Graph($name:_xmax)] [y2p $name 0]

    foreach series $Graph($name:_series) {
        switch $Graph($name:$series:type) {
            line {        
                DrawLineSeries $name $series
            }

            scatter {        
                DrawScatterSeries $name $series
            }

            bar {
                DrawBarSeries $name $series
            }
            
            default {
                assert 0
            }
        }
    }    
}

proc ::graph::DrawLineSeries {name series} {
    variable Graph

    set num [llength $Graph($name:$series:X)]
    
    for {set i 0} {$i < $num} {incr i} {
        set x [lindex $Graph($name:$series:X) $i]
        set y [lindex $Graph($name:$series:Y) $i]
        
        if {[info exists prevx]} {
            $name create line [x2p $name $prevx] [y2p $name $prevy] \
                [x2p $name $x] [y2p $name $y] -fill $Graph($name:$series:color)
        }

        DrawMarker $name $series $x $y
        
        set prevx $x
        set prevy $y
    }   
}

proc ::graph::DrawScatterSeries {name series} {
    variable Graph

    set num [llength $Graph($name:$series:X)]
    
    for {set i 0} {$i < $num} {incr i} {
        set x [lindex $Graph($name:$series:X) $i]
        set y [lindex $Graph($name:$series:Y) $i]        
        DrawMarker $name $series $x $y
    }   
}

proc ::graph::DrawBarSeries {name series} {
    variable Graph

    set num [llength $Graph($name:$series:X)]
    
    for {set i 0} {$i < $num} {incr i} {
        set x [lindex $Graph($name:$series:X) $i]
        set y [lindex $Graph($name:$series:Y) $i]

        set x1 [x2p $name [expr {$x - $Graph($name:$series:radius)}]]
        set y1 [y2p $name $y]
        set x2 [x2p $name [expr {$x + $Graph($name:$series:radius)}]]
        set y2 [y2p $name 0]
        
        $name create rectangle $x1 $y1 $x2 $y2 \
            -fill $Graph($name:$series:color)
    }   
}

proc ::graph::DrawMarker {name series x y} {
    variable Graph

    switch $Graph($name:$series:marker) {
        none {
        }
        
        circle {
            set x1 [expr {[x2p $name $x] - $Graph($name:$series:radius)}]
            set y1 [expr {[y2p $name $y] - $Graph($name:$series:radius)}]
            set x2 [expr {[x2p $name $x] + $Graph($name:$series:radius)}]
            set y2 [expr {[y2p $name $y] + $Graph($name:$series:radius)}]
            
            $name create oval $x1 $y1 $x2 $y2 \
                -fill "" -outline black
        }
        
        square {
            set x1 [expr {[x2p $name $x] - $Graph($name:$series:radius)}]
            set y1 [expr {[y2p $name $y] - $Graph($name:$series:radius)}]
            set x2 [expr {[x2p $name $x] + $Graph($name:$series:radius)}]
            set y2 [expr {[y2p $name $y] + $Graph($name:$series:radius)}]
            
            $name create rectangle $x1 $y1 $x2 $y2 \
                -fill "" -outline black
        }
        
        diamond {
            set x1 [x2p $name $x]
            set x0 [expr {$x1 - $Graph($name:$series:radius) - 1}]
            set x2 [expr {$x1 + $Graph($name:$series:radius) + 1}]
            
            set y1 [y2p $name $y]
            set y0 [expr {$y1 - $Graph($name:$series:radius) - 1}]
            set y2 [expr {$y1 + $Graph($name:$series:radius) + 1}]
            
            $name create polygon $x0 $y1 $x1 $y2 $x2 $y1 $x1 $y0 $x0 $y1 \
                -fill "" -outline black
        }
        
        cross {
            set x1 [x2p $name $x]
            set x0 [expr {$x1 - $Graph($name:$series:radius) - 1}]
            set x2 [expr {$x1 + $Graph($name:$series:radius) + 1}]
            
            set y1 [y2p $name $y]
            set y0 [expr {$y1 - $Graph($name:$series:radius) - 1}]
            set y2 [expr {$y1 + $Graph($name:$series:radius) + 1}]
            
            $name create line $x1 $y0 $x1 [expr {$y2 + 1}]
            $name create line $x0 $y1 [expr {$x2 + 1}] $y1
        }
        
        x {
            set x1 [expr {[x2p $name $x] - $Graph($name:$series:radius)}]
            set y1 [expr {[y2p $name $y] - $Graph($name:$series:radius)}]
            set x2 [expr {[x2p $name $x] + $Graph($name:$series:radius)}]
            set y2 [expr {[y2p $name $y] + $Graph($name:$series:radius)}]
            
            $name create line $x1 $y1 [expr {$x2 + 1}] [expr {$y2 + 1}]
            $name create line $x1 $y2 [expr {$x2 + 1}] [expr {$y1 - 1}]
        }
        
        default {
            assert 0
        }
    }
}

proc ::graph::Expose {name} {
    variable Graph
    
    set Graph($name:_width) [winfo width $name]
    set Graph($name:_height) [winfo height $name]
    
    Recalc $name
    Redraw $name
}

proc ::graph::Recalc {name} {
    variable Graph

    set Graph($name:_xscale) [expr {(1.0*$Graph($name:_width)
                                     - 2*$Graph($name:_xoffset))
                                    / $Graph($name:_xmax)}]
    set Graph($name:_yscale) [expr {(1.0*$Graph($name:_height)
                                     - 2*$Graph($name:_yoffset))
                                    / $Graph($name:_ymax)}]
}

proc graph {name args} {
    canvas $name
    
    rename $name ::graph::Graph($name)
    
    regsub -all {%W} $::graph::Graph(_proc) $name body
    proc $name {args} $body

    set ::graph::Graph($name:_series) {}
    set ::graph::Graph($name:_draw) 0
    set ::graph::Graph($name:_xmax) 0
    set ::graph::Graph($name:_ymax) 0
    set ::graph::Graph($name:_xoffset) 30
    set ::graph::Graph($name:_yoffset) 30
    set ::graph::Graph($name:_xscale) 0
    set ::graph::Graph($name:_yscale) 0
    set ::graph::Graph($name:_width) [winfo reqwidth $name]
    set ::graph::Graph($name:_height) [winfo reqheight $name]

    ::graph::Configure $name $args
    
    bind $name <Expose> { ::graph::Expose %W }
    
    set name
}
