# tabs.tcl


namespace eval ::tabs:: {
    variable Tabs
    
    set ::tabs::Tabs(_proc) {
        set cmd [lshift args]
        
        switch $cmd {
            "add" {
                ::tabs::Add %W $args
            }
            "configure" {
                ::tabs::Configure %W $args
            }
            "getActiveByIndex" {
                ::tabs::GetActiveByIndex %W
            }
            "getActiveByName" {
                ::tabs::GetActiveByName %W
            }
            default {
                eval {::tabs::Tabs(%W)} {$cmd} $args 
            }
        }    
    }
}

proc ::tabs::GetActiveByIndex {name} {
    variable Tabs
    return $Tabs($name:_active)
}

proc ::tabs::GetActiveByName {name} {
    variable Tabs
    return [lindex $Tabs($name:_tabs) $Tabs($name:_active)]
}

proc ::tabs::Add {name varArgs} {
    variable Tabs
    set tab [lindex $varArgs 0]
    lappend Tabs($name:_tabs) $tab
    set num [expr {[llength $Tabs($name:_tabs)] - 1}]
    frame $name.f$num 
    Redraw $name
    return $name.f$num
}

proc ::tabs::Configure {name varArgs} {
    variable Tabs
    
    foreach {op arg} $varArgs {
        switch -exact -- $op {
            default {
                Tabs($name) configure $op $arg
            }
        }
    }
}

proc ::tabs::Redraw {name} {
    variable Tabs
    if {!$Tabs($name:_draw)} {
        after idle "::tabs::Draw $name"
        set Tabs($name:_draw) 1
    }
}

proc ::tabs::MakeActive {name indx} {
    variable Tabs
    set Tabs($name:_active) $indx
    Redraw $name
    if {[info exists Tabs($name:_onActivate)]} {
        eval $Tabs($name:_onActivate)
    }
}

## xxx move to stcl.tcl
proc tabs::OffsetRect {rect xinc yinc} {
    set ret {}
    lappend ret [expr {[lindex $rect 0] - $xinc}]
    lappend ret [expr {[lindex $rect 1] - $yinc}]
    lappend ret [expr {[lindex $rect 2] + $xinc}]
    lappend ret [expr {[lindex $rect 3] + $yinc}]
    set ret
}

proc ::tabs::Draw {name} {
    variable Tabs
    
    set width [expr {[winfo reqwidth $name] - 2}]
    set height [expr {[winfo reqheight $name] - 2}]

    set tabs $Tabs($name:_tabs)
    set numTabs [llength $Tabs($name:_tabs)]
    set active $Tabs($name:_active)

    set xTabOffset 10
    set yTabOffset 3
    set arcOffset 3
    set yActiveOffset 3
    set xFrameOffset 10
    set yFrameOffset 10

    set startX [expr {$xTabOffset + $arcOffset + 1}]
    set startY [expr {$yTabOffset + 2*$yActiveOffset + 1}]

    set Tabs($name:_draw) 0

    $name delete all

    for {set i 0} {$i < $numTabs} {incr i} {
        set t [lindex $tabs $i]
        
        if {$i == $active} {
            set yText [expr {$startY - $yActiveOffset}]
        } else {
            set yText $startY
        }
        
        set id [$name create text $startX $yText -text $t -anchor nw]
        set bb [$name bbox $id]

        if {$i == $active} {
            set xinc [expr {$xTabOffset + $arcOffset - 1}]
            set yinc [expr {$yActiveOffset + 1}]
            set r [OffsetRect $bb $xinc $yinc]
            set activeBbox $r
            
#             set s [OffsetRect $bb 6 3]
            
#             $name create line \
#                 [lindex $s 0] [lindex $s 1] \
#                 [lindex $s 2] [lindex $s 1] \
#                 [lindex $s 2] [lindex $s 3] \
#                 [lindex $s 0] [lindex $s 3] \
#                 [lindex $s 0] [lindex $s 1] \
#                 -stipple gray50
            
        } else {
            set r [OffsetRect $bb [expr {$xTabOffset - 1}] [expr {$yTabOffset - 1}]]
        }
        
        if {$i != ($active + 1)} {
            $name create line \
                [lindex $r 0] \
                [expr {[lindex $r 1] + $arcOffset}] \
                [lindex $r 0] \
                [lindex $r 3] \
                -fill white
            
            $name create arc \
                [lindex $r 0] \
                [lindex $r 1] \
                [expr {[lindex $r 0] + 2*$arcOffset}] \
                [expr {[lindex $r 1] + 2*$arcOffset}] \
                -outline white -start 90 -extent 90 -style arc
        }
        
        $name create line \
            [expr {[lindex $r 0] + $arcOffset}] \
            [lindex $r 1] \
            [expr {[lindex $r 2] - $arcOffset}] \
            [lindex $r 1] \
            -fill white
        
        $name create arc \
            [lindex $r 2] \
            [lindex $r 1] \
            [expr {[lindex $r 2] - 2*$arcOffset}] \
            [expr {[lindex $r 1] + 2*$arcOffset}] \
            -outline white -start 45 -extent 45 -style arc
        
        if {$i != ($active - 1)} {
            $name create arc \
                [lindex $r 2] \
                [lindex $r 1] \
                [expr {[lindex $r 2] - 2*$arcOffset}] \
                [expr {[lindex $r 1] + 2*$arcOffset}] \
                -outline black -start 0 -extent 45 -style arc
            
            $name create line \
                [lindex $r 2] \
                [expr {[lindex $r 1] + $arcOffset}] \
                [lindex $r 2] \
                [lindex $r 3] \
                -fill black
        }
        
        if {$i == $active} {
             $name create line \
                 [expr {[lindex $r 2] -1}] \
                 [expr {[lindex $r 1] + $arcOffset}] \
                 [expr {[lindex $r 2] - 1}] \
                 [lindex $r 3] \
                 -fill gray
        } else {
            set id [$name create rectangle \
                        [lindex $r 0] [lindex $r 1] \
                        [lindex $r 2] [lindex $r 3] \
                        -fill {} -outline {}]
            $name bind $id <1> "::tabs::MakeActive $name $i"
        }
        
        set startX [expr {[lindex $bb 2] + 2*$xTabOffset}]
    }

    set startY [lindex $activeBbox 3]
    $name create line 1 $startY [lindex $activeBbox 0] $startY -fill white
    $name create line [lindex $activeBbox 2] $startY $width $startY -fill white
    $name create line 1 $startY 1 $height -fill white
    $name create line 2 [expr {$height -1}] [expr {$width -1}] [expr {$height -1}] -fill gray
    $name create line 1 $height $width $height -fill black
    $name create line [expr {$width -1}] [expr {$startY +1}] \
        [expr {$width -1}] $height -fill gray
    $name create line $width $startY $width [expr {$height+1}] -fill black

    ## size and place frame
    $name.f$active configure \
        -width [expr {$width - 2*$xFrameOffset}] \
        -height [expr {$height - $startY - 2*$yFrameOffset}]

    $name create window $xFrameOffset [expr {$startY + $yFrameOffset}] \
        -anchor nw -window $name.f$active
    
}

proc tabs {name args} {
    canvas $name
    
    rename $name ::tabs::Tabs($name)
    
    regsub -all {%W} $::tabs::Tabs(_proc) $name body
    proc $name {args} $body
    
    set ::tabs::Tabs($name:_draw) 0
    set ::tabs::Tabs($name:_tabs) {}
    set ::tabs::Tabs($name:_active) 0

    ::tabs::Configure $name $args
    ::tabs::Redraw $name
    
    set name
}



