# tree.tcl


namespace eval ::tree:: {
    variable Tree

    switch $tcl_platform(platform) {
        unix {
            set Tree(_font) \
                -adobe-helvetica-medium-r-normal-*-11-110-100-100-p-56-iso8859-1
        }
        windows {
            set Tree(_font) \
                -adobe-helvetica-medium-r-normal-*-14-100-100-100-p-76-iso8859-1
        }
    }

    set Tree(_proc) {
        set cmd [lshift args]        
        switch $cmd {
            "add" {
                set pathName [lshift args]
                ::tree::Add %P $pathName $args
            }
            "configure" {
                set pathName [lshift args]
                if {$pathName == "canvas"} {
                    eval {::tree::Tree(%P)} {$cmd} $args 
                } else {
                    ::tree::Configure %P $pathName $args
                }
            }
            "cget" {
                set pathName [lshift args]
                if {$pathName == "canvas"} {
                    eval {::tree::Tree(%P)} {$cmd} $args 
                } else {
                    ::tree::CGet %P $pathName $args
                }
            }
            "select" {
                set pathName [lshift args]
                ::tree::SetSelection %P $pathName $args
            }
            "bind" {
                set pathName [lshift args]
                if {$pathName == "canvas"} {
                    eval {::tree::Tree(%P)} {$cmd} $args
                } else {
                    ::tree::Bind %P $pathName $args
                }
            }
            "open" {
                set pathName [lshift args]
                ::tree::Open %P $pathName 1 $args
            }
            "close" {
                set pathName [lshift args]
                ::tree::Open %P $pathName 0 $args
            }
            default {
                eval {::tree::Tree(%P)} {$cmd} $args 
            }
        }
    }

    set maskdata "#define solid_width 9\n#define solid_height 9"
    append maskdata {
        static unsigned char solid_bits[] = {
            0xff, 0x01, 0xff, 0x01, 0xff, 0x01, 0xff, 0x01, 0xff, 0x01, 0xff, 0x01,
            0xff, 0x01, 0xff, 0x01, 0xff, 0x01
        };
    }
    
    set data "#define open_width 9\n#define open_height 9"
    append data {
        static unsigned char open_bits[] = {
            0xff, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x7d, 0x01, 0x01, 0x01,
            0x01, 0x01, 0x01, 0x01, 0xff, 0x01
        };
    }

    image create bitmap ::tree::openbm -data $data -maskdata $maskdata \
        -foreground black -background white

    set data "#define closed_width 9\n#define closed_height 9"
    append data {
        static unsigned char closed_bits[] = {
            0xff, 0x01, 0x01, 0x01, 0x11, 0x01, 0x11, 0x01, 0x7d, 0x01, 0x11, 0x01,
            0x11, 0x01, 0x01, 0x01, 0xff, 0x01
        };
    }

    image create bitmap ::tree::closedbm -data $data -maskdata $maskdata \
        -foreground black -background white

    image create photo ::tree::dir -data {
        R0lGODdhEAAQAPIAAAAAAHh4eLi4uPj4APj4+P///wAAAAAAACwAAAAAEAAQAAADPVi63P4w
        LkKCtTTnUsXwQqBtAfh910UU4ugGAEucpgnLNY3Gop7folwNOBOeiEYQ0acDpp6pGAFArVqt
        hQQAO///
    }
    
    image create photo ::tree::file -data {
        R0lGODdhEAAQAPIAAAAAAHh4eLi4uPj4+P///wAAAAAAAAAAACwAAAAAEAAQAAADPkixzPOD
        yADrWE8qC8WN0+BZAmBq1GMOqwigXFXCrGk/cxjjr27fLtout6n9eMIYMTXsFZsogXRKJf6u
        P0kCADv/
    }
}


proc ::tree::Add {tree path varArgs} {
    variable Tree
    
    set p [split $path {/}]
    set tail [lpop p]
    set dir [join $p "/"]

    if {![info exists Tree($tree:$dir:children)]} {
        error "parent item \"$dir\" is missing"
    }
    
    if {[lsearch -exact $Tree($tree:$dir:children) $tail] >= 0} {
        error "item \"$path\" already exists"
    }
    
    lappend Tree($tree:$dir:children) $tail

    set Tree($tree:$path:children) {}
    set Tree($tree:$path:open) 0
    set Tree($tree:$path:icon) {}
    set Tree($tree:$path:text) $tail
    set Tree($tree:$path:rightText) {}
    
    Configure $tree $path $varArgs
}

proc ::tree::Configure {tree path varArgs} {
    variable Tree

    foreach {op arg} $varArgs {
        switch -exact -- $op {
            "-icon" {
                set Tree($tree:$path:icon) $arg 
            }
            
            "-text" {
                set Tree($tree:$path:text) $arg
            }
            
            "-righttext" {
                set Tree($tree:$path:rightText) $arg
            }
            
            "-open" {
                set Tree($tree:$path:open) [expr {$arg != 0}]
            }

            default {
                puts "$op $arg"
                error "not implemented"
            }
        }
    }

    Redraw $tree
}

proc ::tree::CGet {tree path varArgs} {
    variable Tree

    set op [lshift varArgs]

    switch -exact -- $op {
        "-icon" {
            return $Tree($tree:$path:icon)
        }
        
        "-text" {
            return $Tree($tree:$path:text)
        }
        
        "-righttext" {
            return $Tree($tree:$path:rightText)
        }
        
        "-open" {
            return $Tree($tree:$path:open)
        }
        
        default {
            puts "$op"
            error "not implemented"
        }
    }
}

proc ::tree::SetSelection {tree path varArgs} {
    variable Tree
    set Tree($tree:_selection) $path
    DrawSelection $tree
}

proc ::tree::Bind {tree path varArgs} {
    variable Tree

    set pattern [lshift varArgs]
    set command [lshift varArgs]

    set Tree($tree:$path:bind:$pattern) $command
    
    catch {Tree($tree) bind $Tree($tree:$path:iconTag) $pattern $command}
    catch {Tree($tree) bind $Tree($tree:$path:textTag) $pattern $command}
}

proc ::tree::Open {tree path open varArgs} {
    variable Tree

    set slash ""
    set star ""
    
    foreach arg $varArgs {
        switch -exact -- $arg {
            "-recursive" {
                set star "*"
            }

            "-children" {
                set slash "/"
                set star "*"
            }

            default {
                puts $arg
                error "not implemented"
            }
        }
    }
    
    foreach {ent val} [array get Tree "$tree:$path$slash$star:open"] {
        set Tree($ent) $open
    }
    
    Redraw $tree
}

proc ::tree::Redraw {tree} {
    variable Tree
    if {!$Tree($tree:_draw)} {
        after idle "::tree::Draw $tree"
        set Tree($tree:_draw) 1
    }
}

proc ::tree::Draw {tree} {
    variable Tree
    set Tree($tree:_draw) 0
    $tree delete all
    set Tree($tree:_y) 10
    DrawDir $tree . 20
    $tree config -scrollregion [$tree bbox all]
    DrawSelection $tree
}

proc ::tree::UnionRect {r1 r2} {
    set r {}

    if {[lindex $r1 0] < [lindex $r2 0]} {
        lappend r [lindex $r1 0]
    } else {
        lappend r [lindex $r2 0]
    }

    if {[lindex $r1 1] < [lindex $r2 1]} {
        lappend r [lindex $r1 1]
    } else {
        lappend r [lindex $r2 1]
    }

    if {[lindex $r1 2] > [lindex $r2 2]} {
        lappend r [lindex $r1 2]
    } else {
        lappend r [lindex $r2 2]
    }

    if {[lindex $r1 3] > [lindex $r2 3]} {
        lappend r [lindex $r1 3]
    } else {
        lappend r [lindex $r2 3]
    }

    set r
}

proc ::tree::DrawDir {tree dir in} {
    variable Tree
    
    set start $Tree($tree:_y)
    set y $start
    incr Tree($tree:_y) 18
    
    set x $in

    if {$dir != "."} {
        $tree create line $x $y [expr $x+13] $y -fill gray50
        incr x 12
    }
    
    set bbox [list $x $y $x $y]
    set x1 $x
    
    if {$Tree($tree:$dir:icon) != {}} {
        set t [$tree create image $x $y -image $Tree($tree:$dir:icon) -anchor w \
                   -tag treeNode]
        set Tree($tree:$dir:iconTag) $t
        set bbox [UnionRect $bbox [$tree bbox $t]]
        incr x1 20
    }
    
    set t [$tree create text $x1 $y -text $Tree($tree:$dir:text) \
               -font $Tree(_font) -anchor w -tag treeNode]
    
    set Tree($tree:$dir:textTag) $t    
    set bbox [UnionRect [$tree bbox $t] $bbox]

    if {$Tree($tree:$dir:rightText) != {}} {
        set t [$tree create text [winfo reqwidth $tree] $y \
                   -text $Tree($tree:$dir:rightText) \
                   -font $Tree(_font) -anchor e -tag treeNode]
        set bbox [UnionRect [$tree bbox $t] $bbox]
    }

    set t [eval {$tree} create rectangle $bbox -fill {{}} -outline {{}}]
    set Tree($tree:$dir:tag) $t

    foreach {ent val} [array get Tree "$tree:$dir:bind:*"] {
        regexp "$tree:$dir:bind:(.*)" $ent dummy pat
        regsub {%D} $val $dir val
        Tree($tree) bind $t $pat $val
    }
        
    if {$Tree($tree:$dir:children) != {}} {
        if {$Tree($tree:$dir:open)} {
            set image ::tree::openbm
            
            set x2 [expr {$x + 7}]
            
            foreach child $Tree($tree:$dir:children) {
                set end $Tree($tree:_y)
                DrawDir $tree "$dir/$child" $x2
            }
            
            set j [$tree create line $x2 $start $x2 [expr {$end + 1}] -fill gray50]
            $tree lower $j
        } else {
            set image ::tree::closedbm
        }
        
        if {$dir != "."} {
            set box [$tree create image $in $y -image $image]
            set toggle [expr {!$Tree($tree:$dir:open)}]
            $tree bind canvas $box <1> "$tree configure $dir -open $toggle"
        }
    }
}

proc ::tree::DrawSelection {tree} {
    variable Tree

    if {$Tree($tree:_selindx) != {}} {
        $tree delete $Tree($tree:_selindx)
        set Tree($tree:_selindx) {}
    }
    
    set dir $Tree($tree:_selection)

    if {$dir == {}} {
        return
    }
    
    if {![info exists Tree($tree:$dir:textTag)]} {
        return
    }
    
    set bbox [$tree bbox $Tree($tree:$dir:textTag)]

    if {[llength $bbox] != 4} {
        return
    }

    set Tree($tree:_selindx) [eval $tree create rectangle $bbox -fill skyblue -outline {{}}]
    $tree lower $Tree($tree:_selindx)
}

proc tree {tree args} {
    eval canvas {$tree} $args

    rename $tree ::tree::Tree($tree)
    
    regsub -all {%P} $::tree::Tree(_proc) $tree body
    proc $tree {args} $body

    set ::tree::Tree($tree:.:children) {}
    set ::tree::Tree($tree:.:open) 1
    set ::tree::Tree($tree:.:icon) {}
    set ::tree::Tree($tree:.:text) {}
    set ::tree::Tree($tree:.:rightText) {}
    
    set ::tree::Tree($tree:_draw) 0
    set ::tree::Tree($tree:_selection) {}
    set ::tree::Tree($tree:_selindx) {}
}




