#!/usr/bin/wish 

option add *Font -*-helvetica-medium-r-*-*-12-120-*-*-*-*-iso8859-*
option add *Button.Font -*-helvetica-medium-r-*-*-10-100-*-*-*-*-iso8859-*
option add *selectBorderWidth 0
option add *Radiobutton.selectColor grey
option add *Radiobutton.highlightBackground [. cget -background]
option add *Checkbutton.selectColor grey
option add *Checkbutton.highlightBackground [. cget -background]
#option add *Menu.selectColor grey
#option add *Menu.highlightBackground [. cget -background]
option add *Canvas.highlightBackground [. cget -background]

set bold -*-helvetica-bold-r-*-*-12-120-*-*-*-*-iso8859-*

set tk_strictMotif 1

source "lib/util.tcl"
source "lib/tabs.tcl"
source "lib/dialog.tcl"
source "lib/help.tcl"
source "lib/htmlButton.tcl"
source "lib/shell.tcl"
source "helpText.tcl"

if {[string compare [wm title .] [file tail $argv0]] != 0} {
    puts "Toolbox already running"
    exit
}

set cmdLine(iconify) 0
set cmdLine(minimize) 0

foreach arg $argv {
    switch -- $arg {
        "--iconify" {
            incr cmdLine(iconify)
        }
        "--minimize" {
            incr cmdLine(minimize)
        }
        default {
            puts "usage: $argv0 \[--iconify\] \[--minimize\]"
            exit
        }
    }
}

wm title . VMwareToolbox


###
### Create tabs
###

tabs .t -width 230 -height 220


###
### Main menu
###


menu .m -tearoff false

bind . <3> {
    tk_popup .m %X %Y
}


###
### General panel
###

set w [.t add "General"]

image create photo logoimage -file logo.gif
canvas $w.logo -width 200p -height 1.0i -background white
$w.logo create image 100p .5i -anchor c -image logoimage
pack $w.logo -side top

label $w.msg -text "VMware Toolbox for Linux 
[about]

Copyright VMware, Inc. (c) 1999
All rights reserved."

pack $w.msg -pady 10

.m add command -label "VMware Toolbox [about]" -state disabled
set menuEntries 1


###
### Devices panel
###

.m add separator
incr menuEntries

set w [.t add "Devices"]
      
foreach d [lreverse [getDeviceList]] {
    set name [lindex $d 0]
    set uid [lindex $d 1]
    set enabled [lindex $d 2]
    
    set deviceNames($uid) $name
    
    checkbutton $w.d_$uid -text "$name" \
	-command  "EnableDevice $uid" \
	-onvalue 1 -offvalue 0 \
        -variable deviceStates($uid) \
        -anchor w
    
    .m add checkbutton -label $name  \
	-command  "EnableDevice $uid" \
	-onvalue 1 -offvalue 0 \
        -variable deviceStates($uid)
    incr menuEntries

    if { $enabled } {
        set deviceStates($uid) 1
    } else {
        set deviceStates($uid) 0
    }
    
    pack $w.d_$uid -fill x
}   

proc EnableDevice {uid} {
    global deviceStates setDeviceStateResult deviceNames
    
    setDeviceState $uid $deviceStates($uid)
    
    if {!$setDeviceStateResult} {
        set msg "Couldn't connect to device $deviceNames($uid). It is probably attached to another virtual machine."
        DialogWin .deviceError "Removable Device Error" $msg {} 0 {OK}
        set deviceStates($uid) [expr {!$deviceStates($uid)}]
    }
}

proc PeriodicallyUpdateDeviceState { } {
    global deviceStates
    
    foreach d [getDeviceList] {
        set uid [lindex $d 1]
        set enabled [lindex $d 2]
        
        if {$enabled} {
            set deviceStates($uid) 1
        } else {
            set deviceStates($uid) 0
        }
    }      
    
    after 2000 PeriodicallyUpdateDeviceState   
}

PeriodicallyUpdateDeviceState


###
### Options panel
###

.m add separator
incr menuEntries

array set options { 
    autograb     "Auto Grab"
    autorelease  "Auto Release"
    autoraise    "Auto Raise"
    autoscroll   "Auto Scroll" 
    copypaste    "Copy & Paste"
    hidecursor   "Hide Cursor"
}

set guiState [getGUIState]

set w [.t add "Options"]

foreach opt [lsort -dictionary [array names options]] {
    if {[lsearch -exact $guiState $opt] != -1} {
        set curGuiState($opt) 1
    } else {
        set curGuiState($opt) 0
    }
    
    checkbutton $w.$opt -text $options($opt) \
        -onvalue 1 -offvalue 0 \
        -variable curGuiState($opt) \
        -command ApplyOption \
        -anchor w
    
    .m add checkbutton -label $options($opt) \
        -onvalue 1 -offvalue 0 \
        -variable curGuiState($opt) \
        -command ApplyOption
    incr menuEntries
    
    pack $w.$opt -fill x -side top
}

proc ApplyOption {} {
    global curGuiState
    
    set options {}
    
    foreach i [array names curGuiState] {
        if {$curGuiState($i)} {
            lappend options $i
        }
    }
    
    setGUIState $options
}


###
### Wipe panel
###

##
## needs to be in experimental build
##
 
#set w [.t add "Wiper"]


###
### Minimize/maximize window
###

image create photo iconimage -file icon.gif
canvas .icon -width 52p -height 52p -background white
.icon create image 26p 26p -anchor c -image iconimage

set winSize max

proc minimize {} {
    global minEntry winSize
    .m entryconfigure $minEntry -label "Maximize Window" -command maximize
    pack forget .t .exitButton .helpButton .minButton
    pack .icon -side top
    set winSize min
}

proc maximize {} {
    global minEntry winSize
    .m entryconfigure $minEntry -label "Minimize Window" -command minimize
    pack forget .icon
    pack .t -fill both -expand 1 -padx 5 -pady 5 
    pack .exitButton .helpButton .minButton -side right \
        -expand 1 -fill x -padx 5 -pady 5
    set winSize max
}

.m add separator
incr menuEntries
.m add command -label "Minimize Window" -command minimize
set minEntry $menuEntries
incr menuEntries

bind . <Double-1> {
    if {$winSize == "min"} {
        maximize
    }
}

###
### Help window
###

proc help {{panel {}}} {
    global help
    if {$panel == {}} {
        set panel [.t getActiveByName]
    }
    if {[info exists help($panel.text)]} {
        set txt $help($panel.text)
    } else {
        set txt "No help available for this topic."
    }
    if {[info exists help($panel.urls)]} {
        set urls $help($panel.urls)
    } else {
        set urls {}
    }
    helpWindow .help "VMware Toolbox for Linux" $txt $urls
}

.m add separator
.m add cascade -label "Help" -menu .m.help
.m add command -label "www.vmware.com" -command "helpOpenURL www.vmware.com"
incr menuEntries 3

menu .m.help -tearoff 0
.m.help add command -label "General Help" -command "help General"
.m.help add command -label "Devices Help" -command "help Devices"
.m.help add command -label "Options Help" -command "help Options"


###
### Global Buttons
###

.m add separator
.m add command -label "Exit" -command exit

button .minButton -text "Minimize" -width 7 -command minimize
button .helpButton -text "Help" -width 7 -command help
button .exitButton -text "Exit" -width 7 -command exit

if {$cmdLine(minimize)} {
    minimize
} else {
    maximize
}

if {$cmdLine(iconify)} {
    wm iconify .
}

wm resizable . 0 0


###
### Track cursor, cut and paste. Taken from old toolbox.
###

set selectionBuffer ""

proc handleSelection { offset maxBytes } {
   global selectionBuffer

   set last [expr $offset + $maxBytes - 1]
   return [string range $selectionBuffer $offset $last]
}

selection handle . handleSelection

set lastPos [queryCursor]

set cursorVisible 1
set selectionWanted 0
set SELECTION_GET_WAIT	3
set SELECTION_GET_MAX	6

proc checkPosition { firstTime } {
   global curGuiState lastPos cursorVisible selectionWanted selectionGetCount
   global SELECTION_GET_WAIT SELECTION_GET_MAX
   global selectionBuffer

   set vmwarePos [getCursorPos]

   set vmwareX [lindex $vmwarePos 0]
   set vmwareY [lindex $vmwarePos 1]

   set lastX [lindex $lastPos 0]
   set lastY [lindex $lastPos 1]

   if { $vmwareX == -100 } {
      if { $cursorVisible } {
	 set cursorVisible 0

	 if { $curGuiState(hidecursor) } {
	    warpCursor 2000 2000
	 }

	 set lastPos { -1 -1 }
	 if { ! $firstTime && $curGuiState(copypaste) } {
	    if { [catch { selection get } ourSelection ] } {
	       setSelection
	    } else {
	       setSelection $ourSelection
	    }
	 }
      }
   } else {
      if { ! $cursorVisible } {
	 set cursorVisible 1
      }

      if { $firstTime } {
	 set selectionWanted 1
	 set selectionGetCount 0
      } elseif { $vmwareX != -1 && ($lastX != $vmwareX || $lastY != $vmwareY) } {
	 warpCursor $vmwareX $vmwareY
	 set selectionWanted 1
	 set selectionGetCount 0
      }

      if { $selectionWanted } {
	 if { $selectionGetCount == 0 } {
	    selection clear
	    set selectionBuffer " "
	    selection own .	    
	 }
	 incr selectionGetCount

	 if { $selectionGetCount > $SELECTION_GET_WAIT } {
	    if { ! [catch { getSelection } newSelection] } {
	       set selectionWanted 0
	       set selectionBuffer $newSelection
	       selection own .
	    } elseif { $selectionGetCount > $SELECTION_GET_MAX } {
	       set selectionWanted 0
	    }
	 }
      }

      set currentPos [queryCursor]
      set currentX [lindex $currentPos 0]
      set currentY [lindex $currentPos 1]

      if { $firstTime || ($currentX != $lastX || $currentY != $lastY) } {
	 setCursorPos $currentX $currentY
	 set lastPos $currentPos
      }
   }

   after 250 { checkPosition 0 }      
}

if { [catch { checkPosition 1 } msg] } {
   puts $msg
}


###
### Not in current product. I think.
###


# # Update the state of the wipe button and accelerator
# proc update_wipe { } {
#    global wiperDB

#    if {$wiperDB(running) == "1"} {
#       .top.right.panel.wiper.commandbutton configure -state disabled
#       bind .top.right.panel.wiper.commandbutton <KeyPress-Return> {}
#    } else {
#       .top.right.panel.wiper.commandbutton configure -state normal
#       bind .top.right.panel.wiper.commandbutton <KeyPress-Return> wipe
#    }
# }

# # Structure of the wiper database (wiperDB)
# #  running: Is there is a wiping process in progress?
# #  partitions: list of wipable partition numbers
# #  <partition number>.wipe: 1 if the partition must be wiped
# #                    .disk: VM disk device number

# # Build the Wiper panel
# proc makeWiper { } {
#    global font wiperDB

#    frame .top.right.panel.wiper

#    message .top.right.panel.wiper.title -text {Select the partition(s) you want to wipe:} -width 220 -font $font
#    pack .top.right.panel.wiper.title -side top

#    frame .top.right.panel.wiper.partlist
#    set partitionList [getWipablePartitionList]
#    for {set i 0} {$i < [expr [llength $partitionList] / 2]} {incr i 1} {
#       set device_name [lindex $partitionList [expr 2 * $i]]
#       set mount_point [lindex $partitionList [expr 2 * $i + 1]]

#       # Convert the guest partition device name into a VM IDE device number
#       # The cleanest way to do this (but this is boring to code) is to stat the
#       # name, and to look at dev_t:
#       #  3  0 ..63  -> 0
#       #  3  64..127 -> 1
#       #  22 0 ..63  -> 2
#       #  22 64..127 -> 3
#       if {[string range $device_name 0 6] != "/dev/hd"} {
#          # Assert_not_implemented()
#          puts "Your Linux installation uses non standard device names in /etc/mtab. The toolbox cannot handle them. Please report this bug to VMware, Inc."
#          exit 1
#       }
#       set wiperDB(${i}.disk) [string trimright $device_name 0123456789]
#       set wiperDB(${i}.disk) [string range $wiperDB(${i}.disk) 7 7]
#       scan $wiperDB(${i}.disk) "%c" wiperDB(${i}.disk)
#       # 97 is the ascii code of "a"
#       set wiperDB(${i}.disk) [expr $wiperDB(${i}.disk) - 97]
#       if {[GetDiskWipable $wiperDB(${i}.disk)] == "1"} {
#          checkbutton .top.right.panel.wiper.partlist.$i -text $mount_point -variable wiperDB(${i}.wipe) -font $font -anchor w
#          # By default, wipe all partitions
#          set wiperDB(${i}.wipe) 1
#          pack .top.right.panel.wiper.partlist.$i -side top -fill x
#          lappend wiperDB(partitions) $i
#       }
#    }
#    pack .top.right.panel.wiper.partlist -side top

#    #button .top.right.panel.wiper.commandbutton -text {Wipe} -underline 0 -default active -command wipe -font $font 
#    button .top.right.panel.wiper.commandbutton -text {Wipe} -default active -command wipe -font $font 
#    pack .top.right.panel.wiper.commandbutton -side bottom
# }

# # Callback function for the wiper button
# proc wiper { } {
#    unpackRight
#    pack .top.right.panel.wiper -side left
#    # I don't know why this bugs after wipe is called --hpreg
#    #bind . <Alt-KeyPress-w> wipe
#    focus .top.right.panel.wiper.commandbutton
#    update_wipe
# }

# proc SetHasBeenWiped {i} {
#    puts "Has been wiped: $i"
# }

# # Callback function for the wipe button
# proc wipe { } {
#    global wiperDB

#    # Disallow reentrancy
#    set wiperDB(running) 1
#    update_wipe

#    foreach i $wiperDB(partitions) {
#       if {$wiperDB(${i}.wipe) == "1"} {
# 	doWipePartitions [lindex [.top.right.panel.wiper.partlist.$i configure -text] 4]
#         SetDiskWiped $wiperDB(${i}.disk)
#       }
#    }

#    set wiperDB(running) 0
#    update_wipe
# }
